// $Id: config.cpp,v 1.5 2007/12/02 01:50:45 hvengel Exp $
//  
//  Lttle cms Profiler
// Copyright (C) 2005 - 2007 Hal Engel
//
// THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THIS SOFTWARE.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// Version 1.11

#include "lprofmain-priv.h"
#include "lprofmain.h"
#include "qtlcmswidgets.h" 

#include <qdir.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qtextcodec.h>
#include <qurloperator.h>
#include <qsettings.h>
#include <qassistantclient.h>
#include <qradiobutton.h>

QUrlOperator urlOp;

#if !defined(__WIN32__) && (defined(_WIN32) || defined(WIN32))
#define __WIN32__
#endif

 
#if defined (__APPLE__)
    const QString lprofBaseDir = "/Library/Application Support/lprof";
#else
    const QString lprofBaseDir = "/.lprof";
#endif

static QMutex mutexTempFile;

QString get_QTDIR()
{
    // qDebug("get_home_dir");
    const char* qtdir= "QTDIR";
    return (QString) getenv(qtdir);
}

void init_env()
{
    // qDebug("lprofMain: init_env()");
    
    const QString ConfigFileconst= lprofBaseDir + "/config/lprof.cfg";
    const QString measfileconst= lprofBaseDir + "/temp/measurement.cgt";
    const QString WorkTempFileNameconst = lprofBaseDir + "/temp/$$q60$$.$$$";
    const QString templateconst = lprofBaseDir + "/templates";
    const QString measure_dir_const = lprofBaseDir + "/measurements";
    const QString monMeasFile_const = lprofBaseDir + "/temp/monmeasurement.cgt";
    const QString monCalFile_const = lprofBaseDir + "/temp/moncalibration.cgt";
    const QString tempDir_const = lprofBaseDir + "/temp";
    const QString ref_dirconst = lprofBaseDir + "/target_refs";
    QDir d;
# ifdef __WIN32__
    QString win = getenv("SystemRoot");
        // lets use only forward slashes, not backslashes
    win.replace( '\\', "/" );
    win +=  "/system32/spool/drivers/color";
    mon_profile = win;
    in_profile= win;
    outprofile = win;
# elif defined (__APPLE__)
    mon_profile = "/Library/ColorSync/Profiles";
    in_profile = "/Library/ColorSync/Profiles";
    outprofile = d.homeDirPath() + "/Library/ColorSync/Profiles";
# else
    mon_profile = "/usr/share/color/icc";
    in_profile = "/usr/share/color/icc";
    outprofile = d.homeDirPath() + "/.color/icc";
# endif /* __WIN32__ */
        
    QString home = d.homeDirPath();

    measurement_dir = home + measure_dir_const;

    measfile = home + measfileconst;
    
    templates = home + templateconst;
    
    ref_dir = home + ref_dirconst;
    
    temp_dir = home + tempDir_const;
    
    WorkTempFileName = home + WorkTempFileNameconst;
    
    strcpy(sys.hdr.ref_dir, (const char*)ref_dir.local8Bit());

    strcpy(sys.hdr.temp_file, home + WorkTempFileNameconst);
    strcpy(sys.hdr.MonMeasurementSheet, home + monMeasFile_const);
    strcpy(monmeasFile, sys.hdr.MonMeasurementSheet);
    strcpy(sys.hdr.MonCalibrationSheet, home + monCalFile_const);
    strcpy(moncalFile, sys.hdr.MonCalibrationSheet);
    
    // qDebug("lprofMain: init_env() done");

}

void lprofMain::init_mon()
{
    // qDebug("init_mon");
    monitorValues.items.wppos=0;
    monitorValues.items.ppos=0;
    monitorValues.items.temp=0;
    monitorValues.items.linked=1;
    monitorValues.items.rchecked=TRUE;
    monitorValues.items.gchecked=FALSE;
    monitorValues.items.bchecked=FALSE;
    monitorValues.items.rgamma= 2.4;
    monitorValues.items.ggamma= 2.4;
    monitorValues.items.bgamma= 2.4;
}

void init_marks()
{
    Marks.ur_x=measure_view_H;
    Marks.ur_y=0;

    Marks.ul_x=0;
    Marks.ul_y=0;

    Marks.lr_x=measure_view_H;
    Marks.lr_y=measure_view_V;

    Marks.ll_x=0;
    Marks.ll_y=measure_view_H;

    corners_ready=FALSE;

}

void lprofMain::init_measurement_tool()
{
    // qDebug("lprofMain::init_measurement_tool");
    TheImage        = NULL;
    TheGrid         = NULL;
    CurrentTemplate = "*";
    CurrentSafeFrame = -1;

    int w = CanvasFrame -> width();
    int h = CanvasFrame -> height();
    if (CanvasFrameLayout) delete CanvasFrameLayout;
    CanvasFrameLayout =new QVBoxLayout( CanvasFrame );
    CanvasFrameLayout->setSpacing( 1 );
    CanvasFrameLayout->setMargin( 1 );
    if (Canvas) delete Canvas;
    Canvas = new QCanvas(measure_view_H, measure_view_V);
    Canvas -> setAdvancePeriod(30);
    if (FigEditor) delete FigEditor;
    FigEditor = new FigureEditor(*this, *Canvas, CanvasFrame);
    QObject::connect(FigEditor, SIGNAL(WPPointsSelected()), this, SLOT(setRGBpatchValues()));
    FigEditor -> clear();
    FigEditor-> resize(w, h);
    FigEditor->setHScrollBarMode(QScrollView::Auto);
    FigEditor->setVScrollBarMode(QScrollView::Auto);

    CanvasFrameLayout->addWidget(FigEditor);

}

void setupHelp(QString* g_path)
{
    bool found = false;
    bool done = false;
    char pathSep = ':';

# ifdef __WIN32__
    pathSep = ';';
    const QString assist = "assistant.exe";
#elif defined (__APPLE__)
    const QString assist = "assistant.app/Contents/MacOS/assistant";
# else
    const QString assist = "assistant";
# endif /* __WIN32__ */
    
    QString Path = getenv("PATH");
    QString execpath = (QString) get_QTDIR() + QString("/bin");
    QFileInfo fileinfo;

    while (!done)
    {
        fileinfo.setFile(execpath + "/" + assist);
        if (fileinfo.exists())
        {
            done = true;
            found=true;
        }
        else
        {
            if (Path.length() <=0 || (Path.length() == execpath.length())) done=true;
            else
            {
                execpath = Path.left(Path.find(pathSep));
                Path = Path.right(Path.length() - Path.find(pathSep) - 1);
            }
        }
    }
    QStringList cmdLst;

    if (!found)  // not is QTDIR/bin and not in system PATH
                 // must be Windows.  Look in same location
                 // as lprof.exe
    {
        // FIXME may not be corrected
        // currentDirPath should point to <prefix>/share/lprof/data
        // execution path should be <prefix>/bin
        
        execpath = QDir::currentDirPath().replace(QDir::currentDirPath().length() - 16, 16, "bin");
        fileinfo.setFile(execpath + "/" + assist);
        if (fileinfo.exists()) 
            found=true;
        else
        {
            execpath = lprofApp -> applicationDirPath ();  // is it in the same location as lprof?
            qDebug("setupHelp - execpath = " + execpath);
            fileinfo.setFile(execpath + "/" + assist);
            if (fileinfo.exists()) 
                found=true;
        }
    }
    if (help) delete help;
    help = new QAssistantClient(execpath);  // so that help calls in other locations don't blow up
    if (found)
    {
       // setup help directory for current system language code

       // If QTextCodec::locale() is not supprted then default to english help files
       // currently only english and Russian is supported

        QString path;
        QString test = (QString) QTextCodec::locale();
        if ((test.find((QString) "C", 0, false) >= 0))
            path = QDir::currentDirPath() + "/help/" + "en";
        else
            if (test == (QString) "ru")
                path = QDir::currentDirPath() + "/help/" + (QString) QTextCodec::locale();
        else // since at this point only english and Russian
               // help pages exist
               // everyone else still gets english help
               // This will need to be updated whenever a new
               // language is added to the help files
            path = QDir::currentDirPath() + "/help/" + "en";
       // qDebug("path used in the command list = " + path);
        g_path->insert(0, path);

        cmdLst << "-profile" << path + QDir::separator() + QString("lprof-help.adp");
        help->setArguments( cmdLst );
       
        // qDebug("help path = " + path);
       
        if (!QFile::exists(path + "/lprof-help.html"))
        {
            QMessageBox mb( QTranslator::tr((QString)"Help system files not found"),
                            QTranslator::tr((QString)"The path to the help files is not\n"
                                    "correct.  Please contact LProf support with\n"
                                    "the contents of this message.\n"
                                    "The current help path is set to:\n") + path,
                                    QMessageBox::Critical,
                                    QMessageBox::Ok | QMessageBox::Default,
                                    QMessageBox::NoButton,  QMessageBox::NoButton);

            mb.exec();
        }
    }
    else  // let the user know that help will not work
    {
        QMessageBox mb( QTranslator::tr((QString)"Help system executable not found"),
                        QTranslator::tr((QString)"The QAssistantClient executable was not\n"
                                "found.  Make sure that assistant(.exe)is\n"
                                "located either in your PATH or in the\n"
                                "$QTDIR/bin directory.   Help will not\n"
                                "be availble until this is corrected.\n" ),
                                QMessageBox::Critical,
                                QMessageBox::Ok | QMessageBox::Default,
                                QMessageBox::NoButton,  QMessageBox::NoButton);

        mb.exec();
    }
}

void lprofMain::templateFileCopied( QNetworkOperation *op )
{
    // qDebug("templateFileCopied()");

    if ( !op )
        return;

    if ( op->state() == QNetworkProtocol::StFailed )
    {
        // The copy operation failed
        // Let users know there is a problem
        // qDebug("Error on template file copy");
        QMessageBox::critical( this, tr( "ERROR" ), op->protocolDetail() );
    }
    else if(op->operation () == QNetworkProtocol::OpPut )
    {
        // qDebug("Template file saved - operation successful. %i", copyNumTempFile);
        mutexTempFile.lock();
        copyNumTempFile = copyNumTempFile - 1;
        mutexTempFile.unlock();
    }
    // qDebug("copyNumTempFile = %i", copyNumTempFile);

}

void lprofMain::Create_Config_Dir(const QString data_path)
{
    QString files;
    QDir* dir = new QDir ();

    // check to see if config directory exists
    // if not create it and move template files into it
    // home_path =  dir->homeDirPath() + QString("/") + QString(".lprof");
    if (!dir->exists(dir->homeDirPath() + lprofBaseDir, TRUE))
    {
        dir->mkdir(dir->homeDirPath() + lprofBaseDir, TRUE);

# ifdef __WIN32__
            // hide the user configuration directory
        QString home_path = dir->homeDirPath() + lprofBaseDir;
        files = QString("attrib +h \"") +
                dir->convertSeparators(home_path) + QString("\"");
        system(files.latin1());
# endif /* __WIN32__ */

        if (!dir->exists(dir->homeDirPath() + lprofBaseDir + "/config", TRUE))
            dir->mkdir(dir->homeDirPath() + lprofBaseDir + "/config", TRUE);

        if (!dir->exists(dir->homeDirPath() + lprofBaseDir + "/target_refs", TRUE))
            dir->mkdir(dir->homeDirPath() + lprofBaseDir + "/target_refs", TRUE);

        if (!dir->exists(dir->homeDirPath() + lprofBaseDir + "/temp", TRUE))
            dir->mkdir(dir->homeDirPath() + lprofBaseDir + "/temp", TRUE);

        if (!dir->exists(dir->homeDirPath() + lprofBaseDir + "/templates", TRUE))
            dir->mkdir(dir->homeDirPath() + lprofBaseDir + "/templates", TRUE);

        if (!dir->exists(dir->homeDirPath() + lprofBaseDir + "/measurements", TRUE))
            dir->mkdir(dir->homeDirPath() + lprofBaseDir + "/measurements", TRUE);
    }
    // Copy the template files into the users config directory
    // but first check to make sure the data/templates directory is found
    if (dir->exists(data_path + QString("/template/")))
    {
        QFile f;
        QDir d (data_path + QString("/template/"));
        d.setFilter(QDir::Files | QDir::Hidden);
        const QFileInfoList *list = d.entryInfoList();
        QFileInfoListIterator it( *list );
        QFileInfo *fi;
        while ( (fi = it.current()) != 0 )
        {
            ++it;
            if (fi->fileName().contains("ITX", FALSE))
            {
                f.setName(dir->homeDirPath() +
                        lprofBaseDir + "/templates/"+
                        fi->fileName());
                if (!f.exists())
                {
                    mutexTempFile.lock();
                    copyNumTempFile = copyNumTempFile + 1;
                    urlOpTempFile.copy( data_path + QString("/template/") +
                            fi->fileName(), dir->homeDirPath() +
                            lprofBaseDir + "/templates/" );
                    mutexTempFile.unlock();
                }
            }
        }
    } 
}

void lprofMain::templateFilesExist()
{
    // qDebug("lprofMain::templateFilesExist()");
    // Copy the template files into the users config directory
    bool OK = true;
    QFile f;
    
    QDir d (QDir::currentDirPath() + QString("/template/"));
    if (!d.exists())
    {
        QMessageBox::critical(this, QTranslator::tr("data/templates directory not found."), 
                              QTranslator::tr("This means that something is wrong with the LProf installation.\nLProf may not fuction correctly.\nPlease contact LProf support for assitance in correcting the problem."));
    }
    else
    {
        d.setFilter(QDir::Files | QDir::Hidden);
        const QFileInfoList *list = d.entryInfoList();
        QFileInfoListIterator it( *list );
        QFileInfo *fi;
        // for testing
        // QMessageBox::critical(this, "Test wait", "Remove template file to create error.");
    
        while ( (fi = it.current()) != 0 )
        {
            ++it;
            if (fi->fileName().contains("ITX", FALSE))
            {
                f.setName(d.homeDirPath() +
                        lprofBaseDir + "/templates/" +
                        fi->fileName());
                if (!f.exists())
                {
                    OK = false;
                }
            }
        }
        if (!OK)
            QMessageBox::critical(this, QTranslator::tr((QString)"Pick Template Files Not Installed"), QTranslator::tr((QString)"One or more pick templates are missing.\nAs a result LProf may not run correctly."));
    }
    // qDebug("done lprofMain::templateFilesExist()");
}

void lprofMain::ReferenceFileExists()
{
    QDir dir;
    dir.setPath(ref_dir);
     // qDebug(ref_dir);
     //  qDebug("Reference file count = %i", dir.entryList().count());
    if (dir.entryList().count() == 2)  // because . and .. are counted
    {
        QMessageBox::warning(this, QTranslator::tr((QString)"No reference files found"),
                             QTranslator::tr((QString)"Please go to the preferences tab\n"
                                     "and install one or more reference files.\n"
                                     "You will not be able to use the Camera/Scanner\n"
                                     "profiler until at least one reference file is\n"
                                     "installed.\n" ));

    }
}

BOOL lprofMain::LoadConfig()
{
    QString temp;
    target_template hold;

    // qDebug("lprofMain::LoadConfig");
    if (lprofSettings) delete lprofSettings;
    lprofSettings = new QSettings();
    lprofSettings->setPath( "lprof.sourceforge.net", "lprof", QSettings::User);
    lprofSettings->insertSearchPath( QSettings::Unix, QDir::homeDirPath() + lprofBaseDir + "/config" );

    SelectTemplate->Dir = lprofSettings->readEntry("/LPROF/Files/TEMPLATE_DIR", "");

    // do set up serial devices
    populatePortandInstListView();
    
    if (SelectTemplate->Dir.length() > 0)
    {
        SelectTemplate->setCurrItem(0);

        MonitorProfileSelector ->Dir = lprofSettings->readEntry( "/LPROF/Files/MONITOR_PROFILES_DIR", "");
        MonitorProfileSelector ->RestrictColorspace = icSigRgbData;
        MonitorProfileSelector ->RestrictClass = icSigDisplayClass;
        MonitorProfileSelector -> FillValues();
        MonitorProfileSelector ->setFilename(lprofSettings->readEntry( "/LPROF/Files/MONITOR_PROFILE", ""));

        InputProfileSelector->Dir = (lprofSettings->readEntry( "/LPROF/Files/INPUT_PROFILES_DIR", ""));
        InputProfileSelector-> RestrictColorspace = icSigRgbData;
        InputProfileSelector->FillValues();
        InputProfileSelector->setFilename(lprofSettings->readEntry( "/LPROF/Files/INPUT_PROFILE", ""));

        SafeFrame -> setValue(lprofSettings->readNumEntry( "/LPROF/numbers/SAFE_FRAME", 0));

        TargetSelector->Dir = lprofSettings->readEntry("/LPROF/Files/TARGET_DIR", "");
        TargetSelector->setCurrItem(0);

        targetList->clear();
        const int n=lprofSettings->readNumEntry( "/LPROF/numbers/NUMBER_TARGETS", 0);
        ComboBoxIntent -> setCurrentItem (lprofSettings -> readNumEntry("/LPROF/numbers/intent_curr_item", 0));

        for (int i=0; i < n; i++)
        {
            // file target to template list with information
            temp.sprintf("/LPROF/Files/TARGET%i", i);
            hold.targetFile=lprofSettings->readEntry(temp, "");
            temp.sprintf("/LPROF/Files/TEMPLATE%i", i);
            hold.templateFile=lprofSettings->readEntry(temp, "" );
            add_target(hold);
        }

        if ((uint)TargetSelector->count() > targetList->count())
        {
            // we know that at least one target has been added since the last time the
            // the config file was saved.  So we must up date the targetList->
           // This should never happen since it is an indication that the program crashed
           // the last time it was run or the user manually copied a reference file to the
           // hidden configuration directory
            qDebug("Target not in targetList");
        }

        init_targetlist();
        slotTargetRefChanged();
        OutputFileEdit_Mon -> setText(lprofSettings->readEntry("/LPROF/Files/MON_OUTPUT_PROFILE", ""));
        OutputFileEdit -> setText(lprofSettings->readEntry("/LPROF/Files/OUTPUT_PROFILE", ""));

        dsizes.reffileH = lprofSettings -> readNumEntry("/LPROF/numbers/refFile_h",0);
        if (dsizes.reffileH == 0)
        {
            QWidget *d = QApplication::desktop();
            if (d -> height() <= 768)
            {
                dsizes.reffileH = 145;
                dsizes.reffileW = 395;
                dsizes.profparmsH = 335;
                dsizes.profparmsW = 475;
                dsizes.profidH = 307;
                dsizes.profidW = 372;
                dsizes.profparmsmonH = 250;
                dsizes.profparmsmonW = 300;
                dsizes.profchH = 440;
                dsizes.profchW = 622;
                dsizes.monvalH = 251;
                dsizes.monvalW = 520;
                dsizes.lprofH = 512;
                dsizes.lprofW = 792;
            }
            else if (d -> height() <= 1024 && d -> height() > 768)
            {
                dsizes.reffileH = 155;
                dsizes.reffileW = 456;
                dsizes.profparmsH = 385;
                dsizes.profparmsW = 585;
                dsizes.profparmsmonH = 350;
                dsizes.profparmsmonW = 400;
                dsizes.profidH = 424;
                dsizes.profidW = 462;
                dsizes.profchH = 500;
                dsizes.profchW = 750;
                dsizes.monvalH = 280;
                dsizes.monvalW = 607;
                dsizes.lprofH = 595;
                dsizes.lprofW = 924;
            }
        }
        else
        {
            dsizes.reffileW = lprofSettings -> readNumEntry("/LPROF/numbers/refFile_w",0);
            dsizes.profparmsH = lprofSettings -> readNumEntry("/LPROF/numbers/profparms_h",0);
            dsizes.profparmsW = lprofSettings -> readNumEntry("/LPROF/numbers/profparms_w",0);
            dsizes.profparmsmonH = lprofSettings -> readNumEntry("/LPROF/numbers/profparmsmon_h",0);
            dsizes.profparmsmonW = lprofSettings -> readNumEntry("/LPROF/numbers/profparmsmon_w",0);
            dsizes.profidH = lprofSettings -> readNumEntry("/LPROF/numbers/profid_h",0);
            dsizes.profidW = lprofSettings -> readNumEntry("/LPROF/numbers/profid_w",0);
            dsizes.profchH = lprofSettings -> readNumEntry("/LPROF/numbers/profch_h",0);
            dsizes.profchW = lprofSettings -> readNumEntry("/LPROF/numbers/profch_w",0);
            dsizes.monvalH = lprofSettings -> readNumEntry("/LPROF/numbers/monval_h",0);
            dsizes.monvalW = lprofSettings -> readNumEntry("/LPROF/numbers/monval_w",0);
            dsizes.lprofH = lprofSettings -> readNumEntry("/LPROF/numbers/lprof_h",0);
            dsizes.lprofW = lprofSettings -> readNumEntry("/LPROF/numbers/lprof_w",0);
        }
        return TRUE;
    }
          //  means no config file found this is first time app has been run
          // config file will be created when program is closed
          // Set everything to default values

    SafeFrame -> setValue(60);
    SelectTemplate -> Dir = templates;
    SelectTemplate->FillValues();
    SelectTemplate->setCurrItem(0);
    TargetSelector->Dir = ref_dir;
    ;
    TargetSelector->setCurrItem(0);
    MonitorProfileSelector ->Dir = mon_profile;
    MonitorProfileSelector ->FillValues();
    InputProfileSelector ->Dir = in_profile;
    InputProfileSelector->FillValues();

    QWidget *d = QApplication::desktop();
    if (d -> height() <= 768)
    {
        dsizes.reffileH = 145;
        dsizes.reffileW = 395;
        dsizes.profparmsH = 335;
        dsizes.profparmsW = 475;
        dsizes.profidH = 307;
        dsizes.profidW = 372;
        dsizes.profchH = 440;
        dsizes.profchW = 622;
        dsizes.monvalH = 251;
        dsizes.monvalW = 520;
        dsizes.lprofH = 512;
        dsizes.lprofW = 792;
    }
    else if (d -> height() <= 1024 && d -> height() > 768)
    {
        dsizes.reffileH = 155;
        dsizes.reffileW = 456;
        dsizes.profparmsH = 385;
        dsizes.profparmsW = 585;
        dsizes.profidH = 424;
        dsizes.profidW = 462;
        dsizes.profchH = 500;
        dsizes.profchW = 750;
        dsizes.monvalH = 280;
        dsizes.monvalW = 607;
        dsizes.lprofH = 595;
        dsizes.lprofW = 924;
    }
    return FALSE;

}

void lprofMain::SaveConfig()
{
    QString temp;
    // qDebug("lprofMain::SaveConfig");

    lprofSettings->writeEntry("/LPROF/Files/TARGET_DIR", TargetSelector->Dir);
   // lprofSettings->writeEntry("/LPROF/numbers/TARGET_NUM", (int)TargetSelector->getCurrItem());
    lprofSettings->writeEntry("/LPROF/numbers/NUMBER_TARGETS", (int) targetList->count());
    if (!targetList->isEmpty())
        for (uint i=0; i<=targetList->count()-1; i++)
    {
            // add target to template information
        temp.sprintf("/LPROF/Files/TARGET%i", i);
        lprofSettings->writeEntry(temp, targetList->at(i)->targetFile);
        temp.sprintf("/LPROF/Files/TEMPLATE%i", i);
        lprofSettings->writeEntry(temp, targetList->at(i)->templateFile);
    }

    lprofSettings->writeEntry("/LPROF/Files/OUTPUT_PROFILE", OutputFileEdit -> text());
    lprofSettings->writeEntry("/LPROF/Files/MON_OUTPUT_PROFILE",  OutputFileEdit_Mon ->text());
    lprofSettings->writeEntry("/LPROF/Files/TEMPLATE_DIR", SelectTemplate->Dir);
    // lprofSettings->writeEntry("/LPROF/numbers/TEMPLATE_NUM", (int) SelectTemplate->getCurrItem());
    lprofSettings->writeEntry("/LPROF/Files/MONITOR_PROFILES_DIR", MonitorProfileSelector->Dir);
    lprofSettings->writeEntry("/LPROF/Files/MONITOR_PROFILE", MonitorProfileSelector->getFilename());
    lprofSettings->writeEntry("/LPROF/Files/INPUT_PROFILES_DIR", InputProfileSelector->Dir);
    lprofSettings->writeEntry("/LPROF/Files/INPUT_PROFILE", InputProfileSelector->getFilename());
    lprofSettings->writeEntry("/LPROF/numbers/SAFE_FRAME", (int) SafeFrame->value());
    lprofSettings -> writeEntry("/LPROF/numbers/lprof_h", this -> height());
    lprofSettings -> writeEntry("/LPROF/numbers/lprof_w", this -> width());
    lprofSettings -> writeEntry("/LPROF/numbers/refFile_h",dsizes.reffileH);
    lprofSettings -> writeEntry("/LPROF/numbers/refFile_w",dsizes.reffileW);
    lprofSettings -> writeEntry("/LPROF/numbers/monval_h",dsizes.monvalH);
    lprofSettings -> writeEntry("/LPROF/numbers/monval_w",dsizes.monvalW);
    lprofSettings -> writeEntry("/LPROF/numbers/profparms_h",dsizes.profparmsH);
    lprofSettings -> writeEntry("/LPROF/numbers/profparms_w",dsizes.profparmsW);
    lprofSettings -> writeEntry("/LPROF/numbers/profparmsmon_h",dsizes.profparmsmonH);
    lprofSettings -> writeEntry("/LPROF/numbers/profparmsmon_w",dsizes.profparmsmonW);
    lprofSettings -> writeEntry("/LPROF/numbers/profid_h",dsizes.profidH);
    lprofSettings -> writeEntry("/LPROF/numbers/profid_w",dsizes.profidW);
    lprofSettings -> writeEntry("/LPROF/numbers/profch_h",dsizes.profchH);
    lprofSettings -> writeEntry("/LPROF/numbers/profch_w",dsizes.profchW);
    lprofSettings -> writeEntry("/LPROF/numbers/intent_curr_item",ComboBoxIntent -> currentItem());

//     lprofSettings->~QSettings();
    delete lprofSettings;
//    lprofSettings = 0;
//     targetList -> ~QPtrList();
//    delete targetList;
//    targetList = 0;
}

void lprofMain::init_targetlist()
{
    SelectTemplate->FillValues();
    QString TargetFile = TargetSelector -> getFilename();
    QString TemplateFile = SelectTemplate -> getFilename();
    int i = TargetSelector->getCurrItem();
    int n = SelectTemplate->getCurrItem();
    // reinitialize TargetSelector so that it has the new target ref file/template file pair if one was added
    // This could cause the possition of the currently selected target to change in the drop down
    // So we must get the filename from the curent item to make sure that this item is selected
    // when this is finished and before TargetSelector is reintitalized with FillValues
    TargetSelector->FillValues();  // why dons't this work?
    SelectTemplate->FillValues();

    // find new possition of currently selected target
    int j;
    for (j=0; j<TargetSelector->count(); j++)
    {
        TargetSelector->setCurrItem(j);
        if (TargetSelector->getFilename()==TargetFile)
        {
            i=j;
            j=TargetSelector->count();
        }
    }

    for (j=0; j<SelectTemplate->count(); j++)
    {
        SelectTemplate->setCurrItem(j);
        if (SelectTemplate->getFilename()==TargetFile)
        {
            n=j;
            j=SelectTemplate->count();
        }
    }
    // Now add new target and it's template to the target list
    for (j=0; j<TargetSelector->count(); j++)
    {
        TargetSelector->setCurrItem(j);
        for (int k=0; k<SelectTemplate->count(); k++)
        {
            SelectTemplate->setCurrItem(k);

            if (!targetList->isEmpty())  // targetList has items and we start at the begining
                for (uint m=0; m<targetList->count(); m++)
                    if (targetList->at(m)->targetFile==TargetSelector->getFilename() &&
                        targetList->at(m)->templateFile==SelectTemplate->getFilename())
            {
                targetList->at(m)->targPos=j;
                targetList->at(m)->tempPos=k;
            }
        }
    }

    TargetSelector->setCurrItem(i);
    SelectTemplate->setCurrItem(n);
}

void lprofMain::saveProfileDetails()
{
    QString slash;
    bool cs_profile = true;
    QDir d;
    QString copy, profile;

    // qDebug("lprofMain::saveProfileDetails");

    if (TabProfiler -> currentPageIndex()== 0)
    {
        profile = OutputFileEdit -> text().mid(OutputFileEdit -> text().findRev('/') + 1,
                                         OutputFileEdit -> text().length() - OutputFileEdit -> text().findRev('/') - 1);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/Files/OUTPUT_PROFILE", OutputFileEdit -> text());

        urlOp.copy(measfile, d.homeDirPath() +
                lprofBaseDir + "/measurements/" + profile + QString(".cgt"), false, false);

        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/Files/MEASUREMENT_SHEET",  profile + ".cgt");
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/ur_x", Marks.ur_x);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/ur_y", Marks.ur_y);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/lr_x", Marks.lr_x);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/lr_y", Marks.lr_y);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/ul_x", Marks.ul_x);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/ul_y", Marks.ul_y);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/ll_x", Marks.ll_x);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/ll_y", Marks.ll_y);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/Files/IT8Image", image_file_name);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/numbers/RsplSmoothness", sys.RsplSmoothness);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/boolians/lOptRGCV", sys.lOptRGCV);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/CLUT_POINTS", (int) sys.hdr.CLUTPoints);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/PROFILE_VERBOSITY_LEVEL", (int) sys.hdr.ProfileVerbosityLevel);

        lprofSettings->writeEntry( "/LPROF/profile-" + profile + "/numbers/TEMPLATE_NUM", SelectTemplate->getCurrItem() );
        lprofSettings->writeEntry( "/LPROF/profile-" + profile + "/numbers/TARGET_NUM", TargetSelector->getCurrItem());
        lprofSettings->writeEntry( "/LPROF/profile-" + profile + "/numbers/TEMPLATE_Filename", SelectTemplate->getFilename() );
        lprofSettings->writeEntry( "/LPROF/profile-" + profile + "/numbers/TARGET_Filename", TargetSelector->getFilename());

        // Profile viewing conditions lprofSettings
        // from profile parms dialog
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/surround", sys.hdr.device.surround);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/La", sys.hdr.device.La);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/Yb", sys.hdr.device.Yb);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/PCSsurround", sys.hdr.PCS.surround);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/PCSLa", sys.hdr.PCS.La);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/PCSYb", sys.hdr.PCS.Yb);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/deviceVC",  sys.hdr.viewingConditionsPredefDevice);
        lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/PCSVC", sys.hdr.viewingConditionsPredefPCS);

        // profile ID
        lprofSettings->writeEntry("/LPROF/profile-" +  profile + "/text/manufacturer", sys.hdr.Manufacturer);
        lprofSettings->writeEntry("/LPROF/profile-" +  profile + "/text/model", sys.hdr.Model);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/copyright", sys.hdr.Copyright);
        lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/description", sys.hdr.Description);

    }
    else if (TabProfiler -> currentPageIndex()== 1)
    {
        profile = OutputFileEdit_Mon -> text().mid(OutputFileEdit_Mon -> text().findRev('/') + 1,
                                             OutputFileEdit_Mon -> text().length() - OutputFileEdit_Mon -> text().findRev('/') - 1);
        cs_profile = false;
        lprofSettings->writeEntry("/LPROF/profile-"    +
                (QString) profile +
                "/Files/MON_OUTPUT_PROFILE",
                OutputFileEdit_Mon -> text());
        if (!RadioCoarse->isChecked())
        {
            QString meas_file =  d.homeDirPath() + lprofBaseDir + "/measurements/" + (QString) profile + QString(".cgt");

            urlOp.copy(MonMeasurementSheet->getFilename(), meas_file, false, false);

            QString cal_file =  d.homeDirPath() + lprofBaseDir + "/measurements/" + (QString)"cal-" + (QString) profile + QString(".cgt");

            urlOp.copy(MonCalibrationSheet->getFilename(), cal_file, false, false);

            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/Files/MEASUREMENT_SHEET",  meas_file);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/Files/MON_CAL_SHEET",  cal_file);

            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/SAFE_FRAME", (int) SafeFrame->value());
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/boolians/Course", false);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/CLUT_POINTS", (int) sys.hdr.CLUTPoints);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/PROFILE_VERBOSITY_LEVEL", (int) sys.hdr.ProfileVerbosityLevel);

          // lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/boolians/lUseCIECAM02", sys.hdr.lUseCIECAM02);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/surround", sys.hdr.device.surround);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/La", sys.hdr.device.La);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/Yb", sys.hdr.device.Yb);

          // profile ID
            lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/manufacturer", sys.hdr.Manufacturer);
            lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/model", sys.hdr.Model);
            lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/copyright", sys.hdr.Copyright);
            lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/description", sys.hdr.Description); 


        }
        else
        {
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/whitePpos", monitorValues.items.wppos);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/primePos", monitorValues.items.ppos);

            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/rgamma", monitorValues.items.rgamma);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/ggamma", monitorValues.items.ggamma);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/bgamma", monitorValues.items.bgamma);

            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/red_x", sys.hdr.Primaries.Red.x);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/red_y", sys.hdr.Primaries.Red.y);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/green_x", sys.hdr.Primaries.Green.x);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/green_y", sys.hdr.Primaries.Green.y);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/blue_x", sys.hdr.Primaries.Blue.x);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/blue_y", sys.hdr.Primaries.Blue.y);

            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/whitePoint", monitorValues.items.temp);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/boolians/Course", true);
            lprofSettings->writeEntry("/LPROF/profile-" +  profile + "/boolians/linked", monitorValues.items.linked);

            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/CLUT_POINTS", (int) sys.hdr.CLUTPoints);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/PROFILE_VERBOSITY_LEVEL", (int) sys.hdr.ProfileVerbosityLevel);

          // lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/boolians/lUseCIECAM02", sys.hdr.lUseCIECAM02);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/surround", sys.hdr.device.surround);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/La", sys.hdr.device.La);
            lprofSettings->writeEntry("/LPROF/profile-"  + profile + "/numbers/Yb", sys.hdr.device.Yb);

          // profile ID
            lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/manufacturer", sys.hdr.Manufacturer);
            lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/model", sys.hdr.Model);
            lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/copyright", sys.hdr.Copyright);
            lprofSettings->writeEntry("/LPROF/profile-" + profile + "/text/description", sys.hdr.Description);
        }
    }
    else qDebug("Something wrong");  // will be needed later for printer profiler tab
}

void lprofMain::getConfigParmsandID(QString profile)
{
    // qDebug("lprofMain::getConfigParmsandID");
    sys.hdr.CLUTPoints = lprofSettings->readNumEntry("/LPROF/profile-"  + profile + "/numbers/CLUT_POINTS", 10);
    sys.hdr.ProfileVerbosityLevel = lprofSettings->readNumEntry("/LPROF/profile-"  + profile + "/numbers/PROFILE_VERBOSITY_LEVEL", 0);
    sys.RsplSmoothness = lprofSettings->readNumEntry("/LPROF/profile-"  + profile + "/numbers/RsplSmoothness", 0);
    sys.lOptRGCV = lprofSettings->readBoolEntry("/LPROF/profile-"  + profile + "/boolians/lOptRGCV", true);
         // sys.hdr.lUseCIECAM02 = lprofSettings->readBoolEntry("/LPROF/profile-"  + profile + "/boolians/lUseCIECAM02", false);
    sys.hdr.device.surround = lprofSettings->readNumEntry("/LPROF/profile-"  + profile + "/numbers/surround", 1);
    sys.hdr.device.La = lprofSettings->readDoubleEntry("/LPROF/profile-"  + profile + "/numbers/La", 20);
    sys.hdr.device.Yb = lprofSettings->readDoubleEntry("/LPROF/profile-"  + profile + "/numbers/Yb", 20);
    sys.hdr.PCS.surround = lprofSettings->readNumEntry("/LPROF/profile-"  + profile + "/numbers/PCSsurround", 1);
    sys.hdr.PCS.La = lprofSettings->readDoubleEntry("/LPROF/profile-"  + profile + "/numbers/PCSLa", 20);
    sys.hdr.PCS.Yb = lprofSettings->readDoubleEntry("/LPROF/profile-"  + profile + "/numbers/PCSYb", 20);
    sys.hdr.viewingConditionsPredefDevice = lprofSettings->readNumEntry("/LPROF/profile-"  + profile + "/numbers/deviceVC", userdef);
    sys.hdr.viewingConditionsPredefPCS = lprofSettings->readNumEntry("/LPROF/profile-"  + profile + "/numbers/PCSVC", userdef);

         // profile ID information
    strcpy(sys.hdr.Manufacturer, lprofSettings->readEntry("LPROF/profile-"  + profile + "/text/manufacturer", sys.hdr.Manufacturer));
    strcpy(sys.hdr.Model, lprofSettings->readEntry("LPROF/profile-"  + profile + "/text/model", sys.hdr.Model));
    strcpy(sys.hdr.Copyright, lprofSettings->readEntry("LPROF/profile-"  + profile + "/text/copyright", sys.hdr.Copyright));
    strcpy(sys.hdr.Description, lprofSettings->readEntry("LPROF/profile-"  + profile + "/text/description", sys.hdr.Description));

}
