// $Id: profileparmsmon.cpp,v 1.3 2007/10/05 19:09:18 hvengel Exp $
//  Little cms - profiler construction set
//Copyright (C) 2007 Hal Engel
//
// THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THIS SOFTWARE.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <qbuttongroup.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcombobox.h>

#include "profileparmsmon.h"
#include <lcmsprf.h>

static PROFILERDATA sys;
static bool initialize = true;

QString parms_mon_get_QTDIR()
{
    // qDebug("get_home_dir");
    const char* qtdir= "QTDIR";
    return (QString) getenv(qtdir);
}

vcIndex ProfileParmsMon::getCVindex (QString text)
{
    vcIndex value;

    value = iso3664p2;
    if (text == VCqStringValues[cie1161995])
        value = cie1161995;
    else if (text == VCqStringValues[iso3664p1])
        value = iso3664p1;
    else if (text == VCqStringValues[brightOutdoors])
        value = brightOutdoors;
    else if (text == VCqStringValues[cutSheet])
        value = cutSheet;
    else if (text == VCqStringValues[iso3664p2_pcs])
        value = iso3664p2_pcs;
    else if (text == VCqStringValues[sRGB])
        value = sRGB;
    else if (text == VCqStringValues[sRGBannexD])
        value = sRGBannexD;
    else if (text == VCqStringValues[iso9241])
        value = iso9241;
    else if (text == VCqStringValues[projDim])
        value = projDim;
    else if (text == VCqStringValues[projDark])
        value = projDark;
    else if (text == VCqStringValues[brightOutdoors])
        value = brightOutdoors;
    else if (text == VCqStringValues[userdef])
        value = userdef;

    return value;
}

int ProfileParmsMon::currItemFromIndex (int index, BOOL PCS)
{
    if (!PCS)
    {
        switch(index)
        {
            case sRGB: return 0;
            case sRGBannexD: return 1;
            case iso9241: return 2;
            case projDark: return 3;
            case projDim: return 4;
            case userdef: return 5;
        }
    }
    else
    {
        switch(index)
        {
            case iso3664p2_pcs: return 0;
            case userdef: return 1;
        }
    }
    return 0;
}


void ProfileParmsMon::setVCpresetList()
{
    deviceVCcomboBox -> clear();
    PCSVCcomboBox -> clear();
    // qDebug("current Tab index = %d", sys.hdr.ProfilerTab);

    source_or_dest_VC_groupBox -> setTitle (tr("Assumed display viewing conditions"));
    deviceVCcomboBox -> insertItem (tr(VCqStringValues[sRGB]), 0);
    deviceVCcomboBox -> insertItem (tr(VCqStringValues[sRGBannexD]), 1);
    deviceVCcomboBox -> insertItem (tr(VCqStringValues[iso9241]), 2);
    deviceVCcomboBox -> insertItem (tr(VCqStringValues[projDark]), 3);
    deviceVCcomboBox -> insertItem (tr(VCqStringValues[projDim]), 4);
    deviceVCcomboBox -> insertItem (tr(VCqStringValues[userdef]), 5);

    PCSVCcomboBox -> insertItem (tr(VCqStringValues[iso3664p2_pcs]), 0);
    PCSVCcomboBox  -> insertItem (tr(VCqStringValues[userdef]), 1);
}

ProfileParmsMon::ProfileParmsMon(QWidget *parent )
    : ProfileParmsMonBase(parent, "", 0, Qt::WStyle_SysMenu | Qt::WStyle_MinMax )
{

    passSys a;

    a.get(&sys);

    deviceLaSpinBox -> setMinValue(10.0);
    deviceLaSpinBox -> setLineStep(1.0);
    deviceLaSpinBox -> setMaxValue(3500.0);
    deviceLaSpinBox -> setValue(100.0);

    deviceYbSpinBox -> setMinValue(1.0);
    deviceYbSpinBox -> setLineStep(1.0);
    deviceYbSpinBox -> setMaxValue(100.0);
    deviceYbSpinBox -> setValue(20.0);

    PCSLaSpinBox -> setMinValue(10.0);
    PCSLaSpinBox -> setLineStep(1.0);
    PCSLaSpinBox -> setMaxValue(3500.0);
    PCSLaSpinBox -> setValue(100.0);

    PCSYbSpinBox -> setMinValue(1.0);
    PCSYbSpinBox -> setLineStep(1.0);
    PCSYbSpinBox -> setMaxValue(100.0);
    PCSYbSpinBox -> setValue(20.0);

    setVCpresetList();
    ValuesToControls();
}

ProfileParmsMon::~ProfileParmsMon()
{
    dialogSize ds;

    ds.put(profParms, this -> height(), this -> width());
}

void ProfileParmsMon::ValuesToControls()
{
    setVCpresetList();

    switch (sys.hdr.CLUTPoints)
    {
        case 0: RB0 -> setChecked(TRUE); break;
        case 9: RB10 -> setChecked(TRUE); break;
        case 21: RB20 -> setChecked(TRUE); break;
        case 33: RB33 -> setChecked(TRUE); break;
        default:  RB16 -> setChecked(TRUE);
    }

    switch (sys.hdr.ProfileVerbosityLevel)
    {
        case 0: MinimumTagsRadio -> setChecked(TRUE); break;
        case 1: NormalTagsRadio  -> setChecked(TRUE); break;
        case 2:
        default: VerboseTagsRadio -> setChecked(TRUE); break;
    }

    deviceYbSpinBox -> setValue(sys.hdr.device.Yb);
    deviceLaSpinBox -> setValue(sys.hdr.device.La);
    PCSYbSpinBox -> setValue(sys.hdr.PCS.Yb);
    PCSLaSpinBox -> setValue(sys.hdr.PCS.La);
    deviceSurroundCombo -> setCurrentItem(sys.hdr.device.surround);
    PCSSurroundCombo -> setCurrentItem(sys.hdr.PCS.surround);

    deviceVCcomboBox -> setCurrentItem(currItemFromIndex(sys.hdr.viewingConditionsPredefDevice, FALSE));
    PCSVCcomboBox -> setCurrentItem(currItemFromIndex(sys.hdr.viewingConditionsPredefPCS, TRUE));

    slotChangeStrategy();
}

void ProfileParmsMon::ControlsToValues()
{
    // qDebug("ProfileParmsMon:ControlsToValues())");
    QString Buff;
    sys.hdr.device.surround = deviceSurroundCombo -> currentItem();
    sys.hdr.PCS.surround = PCSSurroundCombo -> currentItem();
    sys.hdr.viewingConditionsPredefDevice = getCVindex (deviceVCcomboBox -> currentText());
    sys.hdr.viewingConditionsPredefPCS = getCVindex (PCSVCcomboBox -> currentText());

    sys.hdr.device.Yb = deviceYbSpinBox -> value();
    sys.hdr.device.La = deviceLaSpinBox -> value();
    sys.hdr.PCS.Yb = PCSYbSpinBox -> value();
    sys.hdr.PCS.La = PCSLaSpinBox -> value();

    if (RB0 -> isChecked())
       sys.hdr.CLUTPoints = 0;
    else
    if(RB10 -> isChecked())
        sys.hdr.CLUTPoints = 9;
    else
    if (RB33 -> isChecked())
        sys.hdr.CLUTPoints = 33;
    else
    if (RB20 -> isChecked())
        sys.hdr.CLUTPoints = 21;
    else
        sys.hdr.CLUTPoints = 17;

    sys.hdr.ProfileVerbosityLevel = 0;

    if (NormalTagsRadio -> isChecked())
	sys.hdr.ProfileVerbosityLevel = 1;
    else
        if (VerboseTagsRadio -> isChecked())
		sys.hdr.ProfileVerbosityLevel = 2;
}

void ProfileParmsMon::slotChangeStrategy()
{
    if (deviceVCcomboBox -> currentText() == VCqStringValues[userdef])
    {
       deviceSurroundCombo -> setEnabled(TRUE);
       deviceLabelSurround -> setEnabled(TRUE);

       deviceLabelLa       -> setEnabled(TRUE);
       deviceLabelYb       -> setEnabled(TRUE);
       deviceLaSpinBox     -> setEnabled(TRUE);
       deviceYbSpinBox     -> setEnabled(TRUE);
    }
    else
    {
       deviceSurroundCombo -> setEnabled(FALSE);
       deviceLabelSurround -> setEnabled(FALSE);

       deviceLabelLa       -> setEnabled(FALSE);
       deviceLabelYb       -> setEnabled(FALSE);
       deviceLaSpinBox     -> setEnabled(FALSE);
       deviceYbSpinBox     -> setEnabled(FALSE);
    }
    if (PCSVCcomboBox -> currentText() == VCqStringValues[userdef])
    {
       PCSSurroundCombo -> setEnabled(TRUE);
       PCSLabelSurround -> setEnabled(TRUE);

       PCSLabelLa       -> setEnabled(TRUE);
       PCSLabelYb       -> setEnabled(TRUE);
       PCSLaSpinBox     -> setEnabled(TRUE);
       PCSYbSpinBox     -> setEnabled(TRUE);
    }
    else
    {
       PCSSurroundCombo -> setEnabled(FALSE);
       PCSLabelSurround -> setEnabled(FALSE);

       PCSLabelLa       -> setEnabled(FALSE);
       PCSLabelYb       -> setEnabled(FALSE);
       PCSLaSpinBox     -> setEnabled(FALSE);
       PCSYbSpinBox     -> setEnabled(FALSE);
    }
}

void ProfileParmsMon::slotVCpresetChanged()
{
    // qDebug("in slotVCpresetChanged");

    sys.hdr.viewingConditionsPredefDevice = getCVindex(deviceVCcomboBox -> currentText());

    switch (sys.hdr.viewingConditionsPredefDevice)
    {
    case iso3664p2: // ISO-3664 P2 Practical Reflection Print
           // qDebug("in slotVCpresetChanged case 0 source");
          deviceSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           deviceYbSpinBox -> setValue (20.0);                    /* Grey world */
           deviceLaSpinBox -> setValue (32.0);
           break;
    case iso3664p2_pcs: // ISO-3664 P2 ICC refence medium
           // qDebug("in slotVCpresetChanged case 0 source");
          deviceSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           deviceYbSpinBox -> setValue (20.0);                    /* Grey world */
           deviceLaSpinBox -> setValue (32.0);
           break;
    case cie1161995: // CIE 116-1995 dE color difference metric
           // qDebug("in slotVCpresetChanged case 1 source");
          deviceSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           deviceYbSpinBox -> setValue (18.0);                   /* Grey world */
           deviceLaSpinBox -> setValue (64.0);                  /* Bright */
           break;
    case iso3664p1:
            // ISO-3664 P1 Critical print evaluation environment
           // qDebug("in slotVCpresetChanged case 2 source");
          deviceSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           deviceYbSpinBox -> setValue (20.0);                   /* Grey world */
           deviceLaSpinBox -> setValue (127.0);                  /* Bright */
           break;
    case brightOutdoors: // Original scene - Bright Outdoors
            // qDebug("in slotVCpresetChanged case 8 source");
           deviceSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
            deviceYbSpinBox -> setValue (20.0);                   /* Grey world */
            deviceLaSpinBox -> setValue (2000.0);                /* Bright Outdoors */
            break;
    case cutSheet: // Cut Sheet Transparencies on a viewing box
             // qDebug("in slotVCpresetChanged case 9 source");
            deviceSurroundCombo -> setCurrentItem( CUTSHEET_SURROUND);  /* Cut sheet viewing conditions */
             deviceYbSpinBox -> setValue (20.0);                   /* Grey world */
             deviceLaSpinBox -> setValue (53.0);                  /* Dim, adapted to slide ? */
             break;
    case sRGB: // Monitor in darkened work environment sRGB
            // qDebug("in slotVCpresetChanged case 3 source");
           deviceSurroundCombo -> setCurrentItem( DIM_SURROUND);                /* Dim viewing conditions */
            deviceYbSpinBox -> setValue (20.0);                   /* Grey world */
            deviceLaSpinBox -> setValue (4.0);                   /* Darkened work environment */
            break;
    case sRGBannexD: // Monitor in typical work environment sRGB annex D
            // qDebug("in slotVCpresetChanged case 4 source");
           deviceSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
            deviceYbSpinBox -> setValue (20.0);                   /* Grey world */
            deviceLaSpinBox -> setValue (22.0);                  /* Typical work environment */
            break;
    case iso9241: // Monitor in bright work environment ISO 9241
            // qDebug("in slotVCpresetChanged case 5 source");
           deviceSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
            deviceYbSpinBox -> setValue (20.0);                   /* Grey world */
            deviceLaSpinBox -> setValue (32.0);                  /* Bright work environment */
            break;
    case projDim: // Projector in dim environment
            // qDebug("in slotVCpresetChanged case 6 source");
           deviceSurroundCombo -> setCurrentItem( DIM_SURROUND);                /* Dim viewing conditions */
            deviceYbSpinBox -> setValue (20.0);                   /* Grey world */
            deviceLaSpinBox -> setValue ( 10.0);                  /* Adaptation is from display */
            break;
    case projDark: // Projector in dark environment
            // qDebug("in slotVCpresetChanged case 7 source");
           deviceSurroundCombo -> setCurrentItem( DARK_SURROUND);               /* Dark viewing conditions */
            deviceYbSpinBox -> setValue (20.0);                   /* Grey world */
            deviceLaSpinBox -> setValue (10.0);                  /* Adaptation is from display */
            break;
    case userdef: // user defined don't do anything
             // qDebug("in slotVCpresetChanged case 10 source");
             break;
    default:
           // qDebug("in slotVCpresetChanged case default");
           deviceSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           deviceYbSpinBox -> setValue (20.0);                    /* Grey world */
           deviceLaSpinBox -> setValue (32.0);                  /* Compromise brightness */
           break;
    }

    sys.hdr.viewingConditionsPredefPCS = getCVindex(PCSVCcomboBox -> currentText());

    switch (sys.hdr.viewingConditionsPredefPCS)
    {
    case iso3664p2: // ISO-3664 P2 Practical Reflection Print
           // qDebug("in slotVCpresetChanged case 0 source");
           PCSSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           PCSYbSpinBox -> setValue (20.0);                    /* Grey world */
           PCSLaSpinBox -> setValue (32.0);
           break;
    case iso3664p2_pcs: // ISO-3664 P2 ICC refence medium
           // qDebug("in slotVCpresetChanged case 0 source");
           PCSSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           PCSYbSpinBox -> setValue (20.0);                    /* Grey world */
           PCSLaSpinBox -> setValue (32.0);
           break;
    case cie1161995: // CIE 116-1995 dE color difference metric
           // qDebug("in slotVCpresetChanged case 1 source");
           PCSSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           PCSYbSpinBox -> setValue (18.0);                   /* Grey world */
           PCSLaSpinBox -> setValue (64.0);                  /* Bright */
           break;
    case iso3664p1:
            // ISO-3664 P1 Critical print evaluation environment
           // qDebug("in slotVCpresetChanged case 2 source");
           PCSSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           PCSYbSpinBox -> setValue (20.0);                   /* Grey world */
           PCSLaSpinBox -> setValue (127.0);                  /* Bright */
           break;
    case brightOutdoors: // Original scene - Bright Outdoors
            // qDebug("in slotVCpresetChanged case 8 source");
            PCSSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
            PCSYbSpinBox -> setValue (20.0);                   /* Grey world */
            PCSLaSpinBox -> setValue (2000.0);                /* Bright Outdoors */
            break;
    case cutSheet: // Cut Sheet Transparencies on a viewing box
             // qDebug("in slotVCpresetChanged case 9 source");
             PCSSurroundCombo -> setCurrentItem( CUTSHEET_SURROUND);  /* Cut sheet viewing conditions */
             PCSYbSpinBox -> setValue (20.0);                   /* Grey world */
             PCSLaSpinBox -> setValue (53.0);                  /* Dim, adapted to slide ? */
             break;
    case sRGB: // Monitor in darkened work environment sRGB
            // qDebug("in slotVCpresetChanged case 3 source");
            PCSSurroundCombo -> setCurrentItem( DIM_SURROUND);                /* Dim viewing conditions */
            PCSYbSpinBox -> setValue (20.0);                   /* Grey world */
            PCSLaSpinBox -> setValue (4.0);                   /* Darkened work environment */
            break;
    case sRGBannexD: // Monitor in typical work environment sRGB annex D
            // qDebug("in slotVCpresetChanged case 4 source");
            PCSSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
            PCSYbSpinBox -> setValue (20.0);                   /* Grey world */
            PCSLaSpinBox -> setValue (22.0);                  /* Typical work environment */
            break;
    case iso9241: // Monitor in bright work environment ISO 9241
            // qDebug("in slotVCpresetChanged case 5 source");
            PCSSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
            PCSYbSpinBox -> setValue (20.0);                   /* Grey world */
            PCSLaSpinBox -> setValue (32.0);                  /* Bright work environment */
            break;
    case projDim: // Projector in dim environment
            // qDebug("in slotVCpresetChanged case 6 source");
            PCSSurroundCombo -> setCurrentItem( DIM_SURROUND);                /* Dim viewing conditions */
            PCSYbSpinBox -> setValue (20.0);                   /* Grey world */
            PCSLaSpinBox -> setValue ( 10.0);                  /* Adaptation is from display */
            break;
    case projDark: // Projector in dark environment
            // qDebug("in slotVCpresetChanged case 7 source");
            PCSSurroundCombo -> setCurrentItem( DARK_SURROUND);               /* Dark viewing conditions */
            PCSYbSpinBox -> setValue (20.0);                   /* Grey world */
            PCSLaSpinBox -> setValue (10.0);                  /* Adaptation is from display */
            break;
    case userdef: // user defined don't do anything
             // qDebug("in slotVCpresetChanged case 10 source");
             break;
    default:
           // qDebug("in slotVCpresetChanged case default");
           PCSSurroundCombo -> setCurrentItem( AVG_SURROUND);    /* Average viewing conditions */
           PCSYbSpinBox -> setValue (20.0);                    /* Grey world */
           PCSLaSpinBox -> setValue (32.0);                  /* Compromise brightness */
           break;
    }
    slotChangeStrategy();
}


void ProfileParmsMon::slotOK()
{
    passSys a;
    dialogSize ds;

    ds.put(profParmsMon, this -> height(), this -> width());
    ControlsToValues();
    a.put(sys);
}

void ProfileParmsMon::slotCancel()
{
    dialogSize ds;

    ds.put(profParmsMon, this -> height(), this -> width());
}

void ProfileParmsMon::slotHelpButton()
{
     QString path = get_global_help_path()->local8Bit() + "/profile-parms.html";
     // qDebug(path);
     getHelp()->openAssistant ();
     getHelp()->showPage(path);
}

