// $Id: installreffile.cpp,v 1.54 2007/10/17 21:52:19 hvengel Exp $
//  Little cms - profiler construction set
//Copyright (C) 2005 - 2007 Hal Engel, 2007 Joseph L. Simmon III
//
// THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THIS SOFTWARE.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#include "installreffile.h"
#include <lprofmain.h>
#include <qtlcmswidgets.h>
#include <qurloperator.h>
#include "main.h"
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qmutex.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qurloperator.h>
#include <qdir.h>
#include <qdatetime.h>

#if !defined(__WIN32__) && (defined(_WIN32) || defined(WIN32))
#define __WIN32__
#endif

static QMutex mutex;
static QString LogoSerial, LogoDescriptor;
static bool refToOld;

int strMonthToInt(QString m)
{
    if (m == QDate::shortMonthName(1) || m == QDate::longMonthName(1))
        return 1;
    else if (m == QDate::shortMonthName(2) || m == QDate::longMonthName(2))
        return 2;
    else if (m == QDate::shortMonthName(3) || m == QDate::longMonthName(3))
        return 3;
    else if (m == QDate::shortMonthName(4) || m == QDate::longMonthName(4))
        return 4;
    else if (m == QDate::shortMonthName(5) || m == QDate::longMonthName(5))
        return 5;
    else if (m == QDate::shortMonthName(6) || m == QDate::longMonthName(6))
        return 6;
    else if (m == QDate::shortMonthName(7) || m == QDate::longMonthName(7))
        return 7;
    else if (m == QDate::shortMonthName(8) || m == QDate::longMonthName(1))
        return 8;
    else if (m == QDate::shortMonthName(9) || m == QDate::longMonthName(9))
        return 9;
    else if (m == QDate::shortMonthName(10) || m == QDate::longMonthName(10))
        return 10;
    else if (m == QDate::shortMonthName(11) || m == QDate::longMonthName(11))
        return 11;
    else if (m == QDate::shortMonthName(12) || m == QDate::longMonthName(12))
        return 12;
    else 
    {
        qDebug("Month not found = " + m);
        return 0;
    }
    
}

void checkHCTAge(QString fn)
{
    QString fileStringTraversal;

    int month, year, day;
     
    QFile ReferenceFileInput(fn);
    ReferenceFileInput.open(IO_ReadOnly);
    QTextStream inTargetStream(&ReferenceFileInput);

    bool dateComplete = FALSE;

    inTargetStream >> fileStringTraversal;

    while(fileStringTraversal != "BEGIN_DATA" || !dateComplete)
    {
        inTargetStream >> fileStringTraversal;
        if(fileStringTraversal == "MONTH")
            inTargetStream >> month;
        if(fileStringTraversal == "DAY")
            inTargetStream >> day;
        if(fileStringTraversal == "YEAR")
        {
            inTargetStream >> year;
            dateComplete = TRUE;
        }
    }
    QDate date = QDate::currentDate();
    refToOld = (((date.month() - month) + ((date.year() - year) * 12)) > 36);

    ReferenceFileInput.close();
}

void checkIT8Age(QString fn)
{
    LCMSHANDLE hIT8 = cmsxIT8LoadFromFile((const char *) fn.local8Bit());
    QString created = cmsxIT8GetProperty(hIT8, "CREATED");
    QDate date = QDate::currentDate();
    refToOld = (((date.month() - strMonthToInt(created.left(created.find(" ")))) + ((date.year() - created.right(4).toInt()) * 12)) > 36);
    refToOld = false;
    cmsxIT8Free(hIT8);
}


QString inst_get_QTDIR()
{
    const char* qtdir= "QTDIR";
    return (QString) getenv(qtdir);
}


InstallRefFile::InstallRefFile(QWidget *parent)
    :InstallRefFileBase(parent), urlOp("/")
{
    QString temp;

    ref_file a;
    a.get(a);
    dirs.template_dir = a.items.template_dir;
    dirs.ref_dir = a.items.ref_dir;
    dirs.temp_dir = a.items.temp_dir;
    settings = a.items.settings;
    
    TargetSelector->Dir = settings -> readEntry("/LPROF/Files/referencefile_default_dir", QDir::homeDirPath());
    TargetSelector->setFilename("");
    
    if (!QString::compare(QDir::homeDirPath() + QString ("/"), TargetSelector->ReadDir()) == 0)
        TargetSelector->FillValues();
    
    connect( &urlOp, SIGNAL( finished( QNetworkOperation *) ), this, SLOT( slotFileCopied( QNetworkOperation * ) ) );
    
    connect(TargetSelector, SIGNAL(valueChanged()), this, SLOT(slotTargetSelectorChanged()));

    copyNum=0;
    
    slotTargetSelectorChanged();

    slotTemplateChanged();
}

void InstallRefFile::slotTargetSelectorChanged()
{
    // qDebug("InstallRefFile::slotTargetSelectorChanged()");
    // for some reason this enbles and disables the Help button????
    buttonOk -> setEnabled(TargetSelector -> count() != 0);
    currentDir -> setText(TargetSelector->Dir);
    // qDebug(TargetSelector->Dir);    
}

InstallRefFile::~InstallRefFile()
{
    dialogSize ds;
    ds.put(refile, this -> height(), this -> width());
}


void InstallRefFile::slotFileCopied( QNetworkOperation *op )
{
    QString errorInfo;

    if ( !op )
        return;

    mutex.lock();
    if ( op && op->state() == QNetworkProtocol::StFailed )
        // The copy operation failed
        // Let users know there is a problem
        QMessageBox::critical( this, tr( "ERROR" ), op->protocolDetail() );
    else if (op -> operation () == QNetworkProtocol::OpPut)
    {
        copyNum = copyNum - 1;
        QMessageBox::information( this, tr( "Reference File Installed" ), tr("The reference file was succesfully installed"));
    }
    if (copyNum == 0)
        buttonCancel->setEnabled(true);
    mutex.unlock();
}

TemplateType InstallRefFile::CheckTemplateType()
{
    QFile ReferenceFile(TargetSelector->getFilename());
    ReferenceFile.open(IO_ReadOnly); 
    QTextStream checkSets(&ReferenceFile); 
    QString currentString;

    bool logo = false;	
    int SetValue = 0;
    LogoSerial = "";
    LogoDescriptor = "";

    // Begin Error and Type Checking.
    while(!ReferenceFile.atEnd())
    {
        checkSets >> currentString;
        if (currentString == "BEGIN_DATA")
            break;  // no longer looking at the header section
        else if (currentString == "LGOROWLENGTH")
            logo = true;
        else if (currentString == "NUMBER_OF_SETS" || currentString == "PATCHES_ACTIVE")
            checkSets >> SetValue;
        else if (logo && currentString == "CREATED")
        {
            checkSets >> currentString;
            LogoSerial = currentString.right(currentString.length() - 1).left(currentString.length() - 2);
            int year = LogoSerial.mid(LogoSerial.findRev('/') + 1, LogoSerial.length() - LogoSerial.findRev('/')).toInt();
            int month = LogoSerial.left(LogoSerial.find('/')).toInt();   
            checkSets >> currentString;
            checkSets >> currentString;
            checkSets >> currentString;
            LogoSerial = LogoSerial + " " + currentString;
            QDate date = QDate::currentDate();
            refToOld = (((date.month() - month) + ((date.year() - year) * 12)) > 36);
         } 
    }
  
    if (logo)
    {
        int pos = TargetSelector->getFilename().findRev('/', TargetSelector->getFilename().length() -1, FALSE); 
        LogoDescriptor = TargetSelector->getFilename().right(TargetSelector->getFilename().length() - (pos + 1));
    }

    ReferenceFile.close();

    switch(SetValue)
    {
        case 24:
            return colorChecker_24;
            break;
        case 140:
            return colorChecker_SG;
            break;
        case 240:
            return colorChecker_DC;
            break;
        case 252:
            checkIT8Age(TargetSelector->getFilename());
            return It8_19column;
            break;
        case 264:
            checkIT8Age(TargetSelector->getFilename());
            return It8_kodak;
            break;
        case 288:
            if (logo)
                return i1_scanner_14;
            else
            {
                checkIT8Age(TargetSelector->getFilename());
                return It8_22column;
            }
            break;
        default:
            if(SetValue > 288 && SetValue <= 638)
            {
                checkHCTAge(TargetSelector->getFilename());
                return hct;
            }
            else 
                return invalid;  
            break;
    }
}

void InstallRefFile::ConvertHCTrefToCGATS()
{
    QString fileStringTraversal, dayParse, 
            yearParse, manufacturerParse, 
            monthConvertToStringFormat,	IDfieldParse_1, 
            IDfieldParse_2, IDfieldParse_3, 
            IDfieldParse_4, IDfieldParse_5, 
            IDfieldParse_6, mediacodeParse, 
            refnumberParse, reffiletypeParse;

    int monthParse;
    double setnumberGrabFromFile;
     
    QFile ReferenceFileInput(TargetSelector->getFilename());
    ReferenceFileInput.open(IO_ReadOnly);
    QTextStream inTargetStream(&ReferenceFileInput);

    int dateCompletionFlag = 0;

    QString monthStringFormat[12] = {"Jan", "Feb", "March", "April", "May", 
                                    "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec"};

    inTargetStream >> reffiletypeParse;

    while(fileStringTraversal != "BEGIN_DATA")
    {
        inTargetStream >> fileStringTraversal;

        if (dateCompletionFlag == 0)
        {
            if(fileStringTraversal == "MONTH")
                inTargetStream >> monthParse;
            if(fileStringTraversal == "DAY")
                inTargetStream >> dayParse;
            if(fileStringTraversal == "YEAR")
            {
                inTargetStream >> yearParse;
                dateCompletionFlag = 1;
            }
        }

        if(fileStringTraversal == "File")
            inTargetStream >> refnumberParse;
        if(fileStringTraversal == "MEDIA_CODE")
            inTargetStream >> mediacodeParse;
        if(fileStringTraversal == "MEDIA_MANUFACTURER")
            inTargetStream >> manufacturerParse;
        if(fileStringTraversal == "SAMPLE_ID")
            inTargetStream >> IDfieldParse_1 >> IDfieldParse_2 >> 
                        IDfieldParse_3 >> IDfieldParse_4 >> 
                        IDfieldParse_5 >> IDfieldParse_6;
        if(fileStringTraversal == "PATCHES_ACTIVE")
            inTargetStream >> setnumberGrabFromFile;
    }

    //	************* HEADER CREATION ************************

    QString createdPropertyText, materialPropertyText, serialPropertyText;

    monthConvertToStringFormat = monthStringFormat[monthParse - 1];

    createdPropertyText.append(monthConvertToStringFormat);
    createdPropertyText.append(", ");
    createdPropertyText.append(dayParse);
    createdPropertyText.append(" ");
    createdPropertyText.append(yearParse);

    materialPropertyText.append(manufacturerParse);
    materialPropertyText.append(" ");
    materialPropertyText.append(mediacodeParse);

    serialPropertyText.append(reffiletypeParse);
    serialPropertyText.append(" ");
    serialPropertyText.append(refnumberParse);

    LCMSHANDLE it8 = cmsxIT8Alloc();
    cmsxIT8SetSheetType(it8, "LCMS");
    cmsxIT8SetProperty(it8, "ORIGINATOR", (const char *) "HutchColor");
    cmsxIT8SetProperty(it8, "CREATED", (const char *) createdPropertyText);
    cmsxIT8SetProperty(it8, "SERIAL", (const char *) serialPropertyText);
    cmsxIT8SetProperty(it8, "MATERIAL", (const char *) materialPropertyText);
    cmsxIT8SetPropertyDbl(it8, "NUMBER_OF_SETS", setnumberGrabFromFile);
    cmsxIT8SetPropertyDbl(it8, "NUMBER_OF_FIELDS", 7);

    int nFields = 0;

    cmsxIT8SetDataFormat(it8, nFields++, "SAMPLE_ID");

    cmsxIT8SetDataFormat(it8, nFields++, IDfieldParse_1);
    cmsxIT8SetDataFormat(it8, nFields++, IDfieldParse_2);
    cmsxIT8SetDataFormat(it8, nFields++, IDfieldParse_3);
    cmsxIT8SetDataFormat(it8, nFields++, IDfieldParse_4);
    cmsxIT8SetDataFormat(it8, nFields++, IDfieldParse_5);
    cmsxIT8SetDataFormat(it8, nFields++, IDfieldParse_6);

    //	************* DATA CREATION ********************

    char patch_id_field[12];
    double field1, field2, field3, field4, field5, field6;

    for (int i = 0; i < setnumberGrabFromFile; i++)
    {
        inTargetStream >> patch_id_field;
        inTargetStream >> field1 >> field2 >> field3 >> field4 >> field5 >> field6;

        if(patch_id_field == "END_DATA")
            break;

        cmsxIT8SetDataSet(it8, patch_id_field, "SAMPLE_ID", patch_id_field);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "XYZ_X", field1);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "XYZ_Y", field2);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "XYZ_Z", field3);

        cmsxIT8SetDataSetDbl(it8, patch_id_field, "LAB_L", field4);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "LAB_A", field5);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "LAB_B", field6);
    }

    cmsxIT8SaveToFile(it8, files.items.targetFile);

    ReferenceFileInput.close();
}

QString InstallRefFile::CreateCustomHCTtemplate()
{
    QString monthStringFormat[12] = {"Jan", "Feb", "March", "April", "May", 
                                    "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec"};

    int i, j;

    // Begin read-in from old target file using CGATS.
    LCMSHANDLE it8read = cmsxIT8LoadFromFile(files.items.targetFile);

    QString originatorTextParse, setnumberParse, 
            createdTextParse, serialTextGrab, 
            newProductionDateText, newCreatedText,	
            monthParse, yearParse;

    originatorTextParse = cmsxIT8GetProperty(it8read, "ORIGINATOR");
    setnumberParse = cmsxIT8GetProperty(it8read, "NUMBER_OF_SETS");
    serialTextGrab = cmsxIT8GetProperty(it8read, "SERIAL");
    createdTextParse = cmsxIT8GetProperty(it8read, "CREATED");
    
    serialTextGrab.remove(" ");
    
    int textCutOff;
    monthParse = createdTextParse.copy();
    textCutOff = monthParse.find(",", 0, true);
    monthParse.remove(textCutOff, monthParse.length());
    createdTextParse.remove(0, textCutOff + 2);
    createdTextParse.remove(0, createdTextParse.find(" ") + 1);
    yearParse = createdTextParse.copy();

    newProductionDateText = yearParse;
    newProductionDateText.append(":");

    QString month_lookup[12][2] = 
    {
        {"Jan", "01"}, {"Feb", "02"},  {"March", "03"},
        {"April", "04"}, {"May", "05"},  {"June", "06"},
        {"July", "07"}, {"Aug", "08"},  {"Sept", "09"},
        {"Oct", "10"},  {"Nov", "11"},  {"Dec", "12"}
    };

    for(i = 0; i < 12; i++)
    if(monthParse == month_lookup[i][0])
    {
        newProductionDateText.append(month_lookup[i][1]);
        break;
    }

    newCreatedText = monthParse;
    newCreatedText.append(", ");
    newCreatedText.append(yearParse);
		
    LCMSHANDLE it8 = cmsxIT8Alloc();

    cmsxIT8SetSheetType(it8, "TEMPLATE");
    cmsxIT8SetProperty(it8, "ORIGINATOR", (const char *) originatorTextParse);
    cmsxIT8SetProperty(it8, "DESCRIPTOR", (const char *) "HCT 4096 Template");
    cmsxIT8SetProperty(it8, "CREATED", (const char *) newCreatedText);
    cmsxIT8SetProperty(it8, "MANUFACTURER", (const char *) "lprof");
    cmsxIT8SetProperty(it8, "PROD_DATE", (const char *) newProductionDateText);
    cmsxIT8SetPropertyDbl(it8, "NUMBER_OF_FIELDS", 5);
    cmsxIT8SetPropertyDbl(it8, "NUMBER_OF_SETS", setnumberParse.toDouble(0));

    int nFields = 0;
    double field1, field2, field3, field4;

    cmsxIT8SetDataFormat(it8, nFields++, "SAMPLE_ID");
    cmsxIT8SetDataFormat(it8, nFields++, "POS_X");
    cmsxIT8SetDataFormat(it8, nFields++, "POS_Y");
    cmsxIT8SetDataFormat(it8, nFields++, "SIZE_X");
    cmsxIT8SetDataFormat(it8, nFields++, "SIZE_Y");

    QString newTemplateFile = dirs.template_dir + (QString) "/" + serialTextGrab + ".ITX2";
    cmsxIT8SaveToFile(it8, newTemplateFile);

    //	File Streaming code will traverse together until both source and target files
    //	match at the same position (after "BEGIN_DATA").

    QString fileStringTraversalCmp, fileStringTraversal;

    QFile newTargetFile(files.items.targetFile);
    newTargetFile.open(IO_ReadOnly);
    QTextStream newTargetStream(&newTargetFile);

    while(fileStringTraversal != "BEGIN_DATA")
        newTargetStream >> fileStringTraversal;

    QFile sourceTargetFile(dirs.template_dir + (QString) "/" + "hutchtone.ITX");
    sourceTargetFile.open(IO_ReadOnly);
    QTextStream compare(&sourceTargetFile);
	   
    while(fileStringTraversalCmp != "BEGIN_DATA")
        compare >> fileStringTraversalCmp;

    // Begin writing and comparison of HCT patches (from old to new)

    char patch_id_field[12];
    char original_patch_id[12];

    for (i = 0; i < setnumberParse.toInt(0, 10); i++)
    {
        QString dummyParse;

        newTargetStream >> patch_id_field;
        for (j = 0; j < 6; j++)
            newTargetStream >> dummyParse;

        do
        {
            compare >> original_patch_id;
            compare >> field1 >> field2 >> field3 >> field4;
        } while(strcmp(patch_id_field, original_patch_id) == 1);

        cmsxIT8SetDataSet(it8, patch_id_field, "SAMPLE_ID", patch_id_field);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "POS_X", field1);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "POS_Y", field2);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "SIZE_X", field3);
        cmsxIT8SetDataSetDbl(it8, patch_id_field, "SIZE_Y", field4);
    }

    cmsxIT8SaveToFile(it8, newTemplateFile);
            
    newTargetFile.close();
    sourceTargetFile.close();

    return (newTemplateFile);
}

void InstallRefFile::slotOK()
{	
    refToOld = FALSE;
    bool success = FALSE;
    if (TargetSelector -> count() != 0)  // something is wrong no file is availble to process
    {    	
        files.items.targetFile = dirs.ref_dir + 
                TargetSelector->getFilename().mid(TargetSelector->getFilename().findRev("/", -1),
                        TargetSelector->getFilename().length());
                
        dialogSize ds;
        
        switch(CheckTemplateType())
        {
            case hct:
                if (refNotToOld())
                {
                    ConvertHCTrefToCGATS();
                    files.items.templateFile = CreateCustomHCTtemplate();
                    success = TRUE;
                }
                break;
    
            case colorChecker_24:
                CopyTemplateFunction(colorChecker_24);
                break;
    
            case It8_22column:
                if (refNotToOld())
                {
                      CopyTemplateFunction(It8_22column);
                }
                break;
    
            case It8_kodak:
                if (refNotToOld())
                {
                    CopyTemplateFunction(It8_kodak);
                }
                break;
    
            case It8_19column:
                if (refNotToOld())
                {
                    CopyTemplateFunction(It8_19column);
                }
                break;
    
            case colorChecker_DC:
                if (refNotToOld())
                {				
                    CopyTemplateFunction(colorChecker_DC);
                    ColorCheckerConvert();
                    success = TRUE;
                }		
                break;
    
            case colorChecker_SG:
                if (refNotToOld())
                {
                    CopyTemplateFunction(colorChecker_SG);
                    ColorCheckerConvert();
                    success = TRUE;
                }	
                break;
                
            case i1_scanner_14:
                if (refNotToOld())
                {
                    CopyTemplateFunction(i1_scanner_14);
                    ColorCheckerConvert();
                    success = TRUE;
                }	
                break;
    
            case invalid:
                QMessageBox::critical(this, tr("Error"), tr("Invalid reference file!"));	
                break;
    
            default:               
                break;
        }
        if (success)
            QMessageBox::information( this, tr( "Reference File Installed" ), tr("The reference file was succesfully installed"));	
    
        files.put(files);	
        ds.put(refile, this -> height(), this -> width());
    }
    else
        QMessageBox::critical( this, tr( "ERROR no reference file available" ), tr("Please specificy a directory that contains one or more reference files." ));

}

void InstallRefFile::slotCancel()
{
    dialogSize ds;

    ds.put(refile, this -> height(), this -> width());

    settings -> writeEntry("/LPROF/Files/referencefile_default_dir", TargetSelector->ReadDir());
}

void InstallRefFile::slotHelpButton()
{
     QString path = get_global_help_path()->local8Bit() +  "/inst-ref.html";
     getHelp()->openAssistant ();
     getHelp()->showPage(path);
}

void InstallRefFile::CopyTemplateFunction(TemplateType templateCatagory)
{
    QString templateFileName;
    BOOL copy = TRUE;
    
    switch(templateCatagory)
    {
        case colorChecker_24:
            templateFileName = "colorchecker-24p.ITX";
            break;
        case colorChecker_SG:
            templateFileName = "colorchecker-sg.ITX";
            copy = FALSE;
            break;
        case colorChecker_DC:
            templateFileName = "colorchecker-dc.ITX";
            copy = FALSE;
            break;
        case It8_22column:
            templateFileName = "IT8_22.ITX";
            break;
        case It8_kodak:
            templateFileName = "KODAK.ITX";
            break;
        case It8_19column:
            templateFileName = "IT8_19.ITX";
            break;
        case i1_scanner_14:
            templateFileName = "eyeone-template.ITX";
            copy = FALSE;
            break;
    }
                            
    files.items.templateFile = dirs.template_dir + (QString) "/" + templateFileName;
    files.items.targetFile = dirs.ref_dir + 
        TargetSelector->getFilename().mid(TargetSelector->getFilename().findRev("/", -1),
                    TargetSelector->getFilename().length());
    QDir::cleanDirPath( files.items.targetFile );

    if (copy)
    { 
        files.items.targetFile = dirs.ref_dir + 
                TargetSelector->getFilename().mid(TargetSelector->getFilename().findRev("/", -1),
                                            TargetSelector->getFilename().length());
        mutex.lock();
        copyNum = copyNum + 1;
        // pull file names out of the dialog
        buttonCancel->setEnabled(false);
        // copy	the reference file to ~/<lprofBaseDir>/target_refs
        urlOp.copy( TargetSelector->getFilename(), dirs.ref_dir + (QString) "/");
        mutex.unlock();
    }
}

void InstallRefFile::CleanKEYWORDS(QString inFile, QString outFile)
{
    QFile ArgyllCGTSFile(inFile);
    QFile cleanCGATS(outFile);
    QString line;
    
    if (ArgyllCGTSFile.open(IO_ReadOnly)) 
    {
        QTextStream in(&ArgyllCGTSFile);
        if (cleanCGATS.open(IO_WriteOnly))
        {
            QTextStream out(&cleanCGATS);
            int i = 1;
            while ( !in.atEnd() ) 
            {
                line = in.readLine(); // line of text excluding '\n'
                if (line.left(7) != "KEYWORD")
                {
                    if (line.left(5) == "BEGIN")
                    {
                        line = "\n" + line;  // Add new line
                    }
                    out << line << "\n";
                }
            }
            ArgyllCGTSFile.close();
            cleanCGATS.close();
        }
        else
        {
            qDebug("Failed to open cleanCGATS" + outFile);
            qDebug("Error = " + cleanCGATS.errorString());
            ArgyllCGTSFile.close();
        }
    }
    else
    {
        qDebug("Failed to open ArgyllCGTSFile " + inFile);
        qDebug("Error = " + ArgyllCGTSFile.errorString());
    }
}

void InstallRefFile::ColorCheckerConvert()
{
    // Added support for colorchecker-SG and DC targets.
    // Also support other logo targets such as the i1 scanner 1.4 target
    // Assumes both ArgyllCMS files are in the lprof application directory:
    //    lprofApp -> applicationDirPath()   On Windows or in the path on
    // Other systems
    
    // *Fix-me request: Sometimes (or usually when there is no reference file installed),
    //  the user needs to hit the "install" button twice in order for the new target to
    //  be created.
    
    // Grab basename of Colorchecker file.
    QString CCFilename;
    
    int startOfFileName;
    startOfFileName = TargetSelector->getFilename().findRev("/", -1);
    CCFilename = TargetSelector->getFilename();		
    CCFilename.remove(0, startOfFileName + 1);
    CCFilename.remove(CCFilename.findRev(".", -1), 4);

    // Save current dir and switch into temp dir
    QString currPath = QDir::currentDirPath();
    QDir::setCurrent(dirs.temp_dir);
        
    // Utilize logo2cgats file.
    // build logo2cgats command and run it
    QString logo2cgats = "\"" + getAppExecDir() + "/logo2cgatslprof" + "\"" + " \"" + TargetSelector->getFilename() + "\"" + " temp";
    system(logo2cgats);
    
    // Utilize spec2cie file.
    // build spec2cie command line and run it 
    QString spec2cie = "\"" + getAppExecDir() + "/spec2cielprof" + "\"" 
            + " temp.ti3 temp2.ti3";
    system(spec2cie);
    
    QString fileName = dirs.ref_dir + "/" + CCFilename + " CGATS.txt";
   
        
    CleanKEYWORDS("temp2.ti3", fileName);
    
    // Save new target with "CGATS" label.
    files.items.targetFile = fileName;
    
    // Remove Temporary Files.
    QFile eraseTempFile("temp.ti3");
    // eraseTempFile.remove();
    QFile eraseTempFile2("temp2.ti3");
    // eraseTempFile2.remove();
            
    // restore current directory
    QDir::setCurrent(currPath);
    
    // Modify fields in reference file so that this shows up correctly in UI
    LCMSHANDLE hIT8 = cmsxIT8LoadFromFile((const char *) files.items.targetFile.local8Bit());
    cmsxIT8SetProperty(hIT8, "ORIGINATOR", (const char *) "LProf Reference File Installer");
    if (LogoDescriptor == "")
        LogoDescriptor = "Unknown - no decriptor found";
    if (LogoSerial == "")
        LogoSerial = "Unknown - no serial found";
    cmsxIT8SetProperty(hIT8, "DESCRIPTOR", (const char *) LogoDescriptor.local8Bit());
    cmsxIT8SetProperty(hIT8, "SERIAL", (const char *) LogoSerial.local8Bit());
    cmsxIT8SetProperty(hIT8, "CREATED", (const char *) LogoSerial.local8Bit());
    cmsxIT8SetProperty(hIT8, "MANUFACTURER", (const char *) "X-Rite");
    cmsxIT8SaveToFile(hIT8, (const char *) files.items.targetFile.local8Bit());
    
    // QDir::cleanDirPath( files.items.targetFile );
}


bool InstallRefFile::refNotToOld()
{
    // qDebug("checkDescription");
    
    if (refToOld)
    {
        QMessageBox mb( QTranslator::tr("Reference File is Old"),
                        QTranslator::tr("This reference file is old.\n"
                                "Targets need to be replaced every two to three years.\n"
                                "This reference file is at least 3 years old.\n"
                                "\nInstall this reference file anyway?"),
                                QMessageBox::Warning,
                                QMessageBox::Yes,
                                QMessageBox::No | QMessageBox::Default,
                                QMessageBox::NoButton );
        mb.setButtonText( QMessageBox::Yes, QTranslator::tr("Yes"));
        mb.setButtonText( QMessageBox::No, QTranslator::tr("No"));
        switch( mb.exec() )
        {
            case QMessageBox::Yes:
                return TRUE;
                break;
            case QMessageBox::No:
                return FALSE;
                break;
        }
    }
    else
        return TRUE;
}
