#include "wwfloatspinbox.h"
#include <qvalidator.h>
#include <math.h>
#include <qlineedit.h>

#if QT_VERSION >= 0x030300
    #include "qlocale.h"
#endif

wwFloatSpinBox::wwFloatSpinBox(QWidget *parent, const char *name) : QSpinBox(parent, name), m_precision(1) {
    setValidator(new QDoubleValidator(this));
    editor()->setAlignment(Qt::AlignRight);
    rangeChange();
    updateDisplay();
}



wwFloatSpinBox::~wwFloatSpinBox() {}

void wwFloatSpinBox::valueChange(){
    QSpinBox::valueChange();
    emit valueChanged(wwFloatSpinBox::value());
}

void wwFloatSpinBox::rangeChange(){
    QDoubleValidator *v = (QDoubleValidator*)validator();
    Q_ASSERT(v!=0);
    v->setRange(i2d(QSpinBox::minValue()), i2d(QSpinBox::maxValue()), m_precision);
    QSpinBox::rangeChange();
}

void wwFloatSpinBox::precisionChange(){
    rangeChange();
}

double wwFloatSpinBox::i2d(int v) const{
    return v/pow(double(10.0), m_precision);
}

int wwFloatSpinBox::d2i(double v) const{
    return (int)((v>=0 ? 1 : -1)*0.5+pow(double(10.0), m_precision)*v);
}

void wwFloatSpinBox::setValue( double value ) {
    QSpinBox::setValue(d2i(value));
}

void wwFloatSpinBox::setValue( int value ) {
    wwFloatSpinBox::setValue(i2d(value));
}


double wwFloatSpinBox::minValue( ) const {
    return i2d(QSpinBox::minValue());
}


double wwFloatSpinBox::maxValue( ) const {
    return i2d(QSpinBox::maxValue());
}


void wwFloatSpinBox::setMinValue( double v) {
    QSpinBox::setMinValue(d2i(v));
}


void wwFloatSpinBox::setMaxValue( double v) {
    QSpinBox::setMaxValue(d2i(v));
}


double wwFloatSpinBox::value( ) const {
    return i2d(QSpinBox::value());
}


QString wwFloatSpinBox::mapValueToText( int value ) {
#if QT_VERSION >= 0x030300
    QLocale loc;
    return loc.toString(i2d(value), 'f', m_precision);
#else
    return QString::number(i2d(value), 'f', m_precision);
#endif
}


int wwFloatSpinBox::mapTextToValue( bool * ok ) {
#if QT_VERSION >= 0x030300
    QLocale loc;
    return d2i(loc.toDouble(text(), ok));
#else
    return d2i(text().toDouble(ok)));
#endif
}


int wwFloatSpinBox::precision( ) const {
    return m_precision;
}

void wwFloatSpinBox::setPrecision( int p ) {
    if(p==m_precision) return;
    if(p<0)
        p = 0;
    if(p>6)
        p = 6;
    double val = wwFloatSpinBox::value();
    double ls = lineStep();
    double omax = wwFloatSpinBox::maxValue();
    double omin = wwFloatSpinBox::minValue();
    m_precision = p;
    wwFloatSpinBox::setMaxValue(omax);
    wwFloatSpinBox::setMinValue(omin);
    wwFloatSpinBox::setValue(val);
    setLineStep(ls);
    precisionChange();
    updateDisplay();
}


double wwFloatSpinBox::lineStep( ) const {
    return i2d(QSpinBox::lineStep());
}

void wwFloatSpinBox::setLineStep( double ls) {
    QSpinBox::setLineStep(d2i(ls));
}


double wwFloatSpinBox::doubleMaxValue() const { return wwFloatSpinBox::maxValue(); }
double wwFloatSpinBox::doubleMinValue() const { return wwFloatSpinBox::minValue(); }
double wwFloatSpinBox::doubleLineStep() const { return wwFloatSpinBox::lineStep(); }
void wwFloatSpinBox::setDoubleMaxValue(double v){ wwFloatSpinBox::setMaxValue(v); }
void wwFloatSpinBox::setDoubleMinValue(double v){ wwFloatSpinBox::setMinValue(v); }
void wwFloatSpinBox::setDoubleLineStep(double v){ wwFloatSpinBox::setLineStep(v); }


