#ifndef WWFLOATSPINBOX_H
#define WWFLOATSPINBOX_H

#include <qspinbox.h>

/**
 * @class wwFloatSpinBox
 * @brief Spin Box displaying real values
 *
 *
 */
class wwFloatSpinBox : public QSpinBox {
    Q_OBJECT
    Q_OVERRIDE( double maxValue READ doubleMaxValue  WRITE setDoubleMaxValue  )
    Q_OVERRIDE( double minValue READ doubleMinValue  WRITE setDoubleMinValue  )
    Q_OVERRIDE( double lineStep READ doubleLineStep  WRITE setDoubleLineStep  )
    Q_OVERRIDE( double value    READ value     WRITE setValue     )
    Q_PROPERTY( int   precision READ precision WRITE setPrecision )
public:
    wwFloatSpinBox(QWidget *parent = 0, const char *name = 0);
    ~wwFloatSpinBox();
    double minValue() const;
    double maxValue() const;
    void  setMinValue( double );
    void  setMaxValue( double );
    double lineStep() const;                                      ///< Returns the line step.
    double value() const;                                         ///< Returns the value of the spin box.
    int precision() const;                                        ///< Returns the precision of the spin box.
    double doubleMaxValue() const;
    double doubleMinValue() const;
    double doubleLineStep() const;
    void setDoubleMaxValue(double);
    void setDoubleMinValue(double);
    void setDoubleLineStep(double);
signals:
    void valueChanged( double );                                  ///< Emitted when value changes
public slots:
    virtual void setValue( double value );                        ///< Sets the value of the spin box to value.
    void setValue(int value);                                     ///< Sets the value of the spin box to value.
    virtual void setPrecision( int p);                            ///< Sets the precision of the spin box to p.
    virtual void setLineStep( double );                                  ///< Sets the line step.
protected:
    QString	mapValueToText( int value );
    int		mapTextToValue( bool* ok );
    int m_precision;
    void valueChange();
    void rangeChange();
    virtual void precisionChange();
    double i2d(int) const;
    int d2i(double) const;

};

#endif
