// $Id: monitorcal.cpp,v 1.55 2007/11/30 06:28:59 hvengel Exp $
//  Little cms Profiler
// Copyright (C) 2007 Hal Engel
//
// THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF
// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
// OF THIS SOFTWARE.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// Version 1.11

#if !defined(__WIN32__) && (defined(_WIN32) || defined(WIN32))
#define __WIN32__
#endif

#define COMPORT 1               /* Default com port 1..4 */

// ArgyllCMS instrument support stuff

extern "C"
{
#include "xspect.h"
#include "config.h"
#include "xicc.h"
#include "icoms.h"
#include "inst.h"
#include <icc.h>
#include "moncurve.h"
#include "ioerrorhandler.h"
}

// QT stuff

#include <qtextedit.h>
#include <qradiobutton.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qapplication.h>
#include <qwt_thermo.h>
#include <qprogressbar.h>
#include <qmessagebox.h>
#include <qworkspace.h>
#include <qslider.h>
#include <qpushbutton.h>

//LProf stuff

#include "lprofmain.h"
#include "main.h"
#include "monitorcal.h"
#include "moncalcontrol.h"

#include "LUTutil.h"


// liblprof
#include "lcmsprf.h"
                 
// for ArgyllCMS USB error trapping
#include <setjmp.h>

    int verb = 0;
    // int debug = 0;
    int docalib = 0;     /* Do a manual instrument calibration */
    int nocal = 0;       /* Disable auto calibration */
    int pspec = 0;       /* 1 = Print out the spectrum for each reading */
                         /* 2 = Plot out the spectrum for each reading */
    bool trans = false;       /* Use transmissioin mode */
    bool emiss = true;       /* Use emissive mode */
    bool ambient = false;     /* Use ambient mode */
    inst_opt_filter fe = inst_opt_filter_unknown;
                                           /* Filter configuration */
    char outname[MAXNAMEL+1] = "\000";     /* Output logfile name */
    char filtername[MAXNAMEL+1] = "\000";  /* Filter compensation */
    FILE *fp = NULL;                       /* Logfile */
    int comport = COMPORT;                 /* COM port used */
    instType itype = instUnknown;          /* No default target instrument */
    int dtype = 0;                         /* Display type, 0 = default, 1 = CRT, 2 = LCD */
    inst_capability  cap = inst_unknown;    /* Instrument capabilities */
    inst2_capability cap2 = inst2_unknown;  /* Instrument capabilities 2 */
    double lx, ly;                       /* Read location on xy table */
    static baud_rate br = baud_19200;           /* Target baud rate */
    // static inst *it = NULL;                     /* Instrument object */
    static inst_code rv;
    int button = 0;                   /* Instrument button is enabled */
    int spec = 0;                     /* Use spectral data flag */
    icxIllumeType illum = icxIT_D50;  /* Spectral defaults */
    xspect cust_illum;                /* Custom illumination spectrum */
    icxObserverType observ = icxOT_Judd_Voss_2;
    xspect sp;                        /* Last spectrum read */
    xsp2cie *sp2cie = NULL;           /* default conversion */
    xsp2cie *sp2cief[26];             /* FWA corrected conversions */
    double Lab[3] = { -10.0, 0, 0};   /* Last Lab */
    double rLab[3] = { -10.0, 0, 0};  /* Reference Lab */
    xspect rsp;                       /* Reference spectrum */
    int ix;                           /* Reading index number */

    double whitePoint, targetBrightness;
    sampleSize sSize;
    bool stop;
    bool measuring = false;
    operation op;
    static char measfile[256], calfile[256];
    int ramdac_size;
    gamma_ramp *orig_gammaRamp = NULL;
    gamma_ramp *new_gammaRamp = NULL;
    int screen;

    instData listedInst[instArraySize];
    controlData cd;
    const int small_size=96;
    const int medium_size=288;
    const int large_size=864;
    const int huge_size=2592;
    const int calibration_set_size=125;
    bool instInUse = FALSE;

// for calibration sample the gray, red, green and blue
// response curves

RGBtype patches_cal[calibration_set_size] =
{
    {0,  0, 0},
    {8,  8, 8}, {0,  0, 8}, {0,  8, 0}, {8,  0, 0},
    {16,  16, 16}, {0,  0, 16}, {0,  16, 0}, {16,  0, 0},
    {25,  25, 25}, {0,  0, 25}, {0,  25, 0}, {25,  0, 0},
    {33,  33, 33}, {0,  0, 33}, {0,  33, 0}, {33,  0, 0},
    {41,  41, 41}, {0,  0, 41}, {0,  41, 0}, {41,  0, 0},
    {49,  49, 49}, {0,  0, 49}, {0,  49, 0}, {49,  0, 0},
    {58,  58, 58}, {0,  0, 58}, {0,  58, 0}, {58,  0, 0},
    {66,  66, 66}, {0,  0, 66}, {0,  66, 0}, {66,  0, 0},
    {74,  74, 74}, {0,  0, 74}, {0,  74, 0}, {74,  0, 0},
    {82,  82, 82}, {0,  0, 82}, {0,  82, 0}, {82,  0, 0},
    {90,  90, 90}, {0,  0, 90}, {0,  90, 0}, {90,  0, 0},
    {99,  99, 99}, {0,  0, 99}, {0,  99, 0}, {99,  0, 0},
    {107,  107, 107}, {0,  0, 107}, {0,  107, 0}, {107,  0, 0},
    {115,  115, 115}, {0,  0, 115}, {0,  115, 0}, {115,  0, 0},
    {123,  123, 123}, {0,  0, 123}, {0,  123, 0}, {123,  0, 0},
    {132,  132, 132}, {0,  0, 132}, {0,  132, 0}, {132,  0, 0},
    {140,  140, 140}, {0,  0, 140}, {0,  140, 0}, {140,  0, 0},
    {148,  148, 148}, {0,  0, 148}, {0,  148, 0}, {148,  0, 0},
    {156,  156, 156}, {0,  0, 156}, {0,  156, 0}, {156,  0, 0},
    {165,  165, 165}, {0,  0, 165}, {0,  165, 0}, {165,  0, 0},
    {173,  173, 173}, {0,  0, 173}, {0,  173, 0}, {173,  0, 0},
    {181,  181, 181}, {0,  0, 181}, {0,  181, 0}, {181,  0, 0},
    {189,  189, 189}, {0,  0, 189}, {0,  189, 0}, {189,  0, 0},
    {197,  197, 197}, {0,  0, 197}, {0,  197, 0}, {197,  0, 0},
    {206,  206, 206}, {0,  0, 206}, {0,  206, 0}, {206,  0, 0},
    {214,  214, 214}, {0,  0, 214}, {0,  214, 0}, {214,  0, 0},
    {222,  222, 222}, {0,  0, 222}, {0,  222, 0}, {222,  0, 0},
    {230,  230, 230}, {0,  0, 230}, {0,  230, 0}, {230,  0, 0},
    {239,  239, 239}, {0,  0, 239}, {0,  239, 0}, {239,  0, 0},
    {247,  247, 247}, {0,  0, 247}, {0,  247, 0}, {247,  0, 0},
    {255,  255, 255}, {0,  0, 255}, {0,  255, 0}, {255,  0, 0}
};

RGBtype patches_small[small_size] =
{
    {66, 66, 66},
    {24, 120, 215},
    {187, 147, 76},
    {255, 255, 0},
    {222, 222, 222},
    {76, 131, 219},
    {183, 88, 183},
    {115, 115, 115},
    {16, 16, 16},
    {239, 16, 143},
    {179, 28, 36},
    {28, 179, 44},
    {64, 191, 64},
    {173, 173, 173},
    {199, 199, 167},
    {72, 72, 40},
    {165, 165, 165},
    {96, 96, 159},
    {20, 60, 68},
    {243, 219, 100},
    {175, 207, 207},
    {8, 8, 8},
    {123, 123, 123},
    {56, 215, 56},
    {203, 4, 92},
    {214, 214, 214},
    {0, 0, 255},
    {251, 84, 12},
    {16, 239, 175},
    {58, 58, 58},
    {74, 74, 74},
    {151, 247, 88},
    {60, 20, 203},
    {0, 255, 0},
    {230, 230, 230},
    {108, 36, 60},
    {247, 151, 247},
    {107, 107, 107},
    {25, 25, 25},
    {112, 143, 16},
    {52, 155, 163},
    {155, 52, 171},
    {191, 64, 191},
    {181, 181, 181},
    {135, 8, 231},
    {104, 231, 199},
    {156, 156, 156},
    {223, 223, 32},
    {147, 187, 195},
    {116, 92, 227},
    {48, 80, 80},
    {0, 0, 0},
    {132, 132, 132},
    {40, 40, 135},
    {139, 195, 28},
    {206, 206, 206},
    {255, 0, 255},
    {124, 211, 139},
    {143, 112, 48},
    {49, 49, 49},
    {82, 82, 82},
    {215, 56, 24},
    {44, 227, 124},
    {255, 0, 0},
    {239, 239, 239},
    {235, 163, 187},
    {120, 24, 120},
    {99, 99, 99},
    {33, 33, 33},
    {80, 48, 239},
    {36, 108, 20},
    {219, 243, 235},
    {128, 128, 128},
    {189, 189, 189},
    {8, 135, 104},
    {231, 104, 72},
    {148, 148, 148},
    {255, 255, 255},
    {159, 32, 96},
    {211, 124, 131},
    {84, 251, 4},
    {32, 159, 223},
    {255, 255, 255},
    {140, 140, 140},
    {167, 167, 8},
    {12, 68, 155},
    {197, 197, 197},
    {0, 255, 255},
    {92, 116, 108},
    {163, 235, 147},
    {207, 175, 112},
    {41, 41, 41},
    {90, 90, 90},
    {88, 183, 151},
    {171, 100, 251},
    {247, 247, 247}
};

RGBtype patches_medium[medium_size] =
{
    {16, 16, 16},
    {194, 194, 194},
    {246, 136, 172},
    {40, 40, 135},
    {178, 132, 25},
    {102, 6, 209},
    {205, 173, 217},
    {18, 90, 54},
    {147, 187, 195},
    {75, 29, 224},
    {223, 223, 32},
    {53, 99, 234},
    {117, 117, 117},
    {93, 93, 93},
    {148, 67, 138},
    {255, 0, 255},
    {206, 81, 148},
    {124, 211, 139},
    {177, 122, 86},
    {62, 189, 74},
    {245, 118, 225},
    {67, 148, 136},
    {135, 8, 231},
    {31, 224, 228},
    {219, 219, 219},
    {40, 40, 40},
    {255, 255, 255},
    {170, 170, 170},
    {103, 248, 156},
    {215, 56, 24},
    {19, 164, 121},
    {131, 76, 116},
    {74, 225, 173},
    {249, 50, 126},
    {52, 155, 163},
    {186, 13, 113},
    {112, 143, 16},
    {196, 115, 123},
    {142, 142, 142},
    {69, 69, 69},
    {125, 43, 194},
    {247, 247, 247},
    {63, 65, 5},
    {235, 163, 187},
    {66, 106, 197},
    {157, 157, 42},
    {30, 30, 169},
    {193, 233, 70},
    {108, 36, 60},
    {190, 192, 132},
    {243, 243, 243},
    {252, 170, 67},
    {65, 65, 65},
    {146, 146, 146},
    {132, 178, 59},
    {80, 48, 239},
    {250, 204, 49},
    {36, 108, 20},
    {185, 241, 62},
    {106, 66, 78},
    {195, 139, 52},
    {27, 45, 17},
    {151, 247, 88},
    {71, 89, 125},
    {166, 166, 166},
    {255, 255, 255},
    {45, 45, 45},
    {223, 223, 223},
    {158, 97, 101},
    {8, 135, 104},
    {194, 21, 9},
    {118, 245, 98},
    {189, 62, 201},
    {50, 249, 213},
    {251, 84, 12},
    {79, 208, 21},
    {0, 0, 255},
    {21, 194, 11},
    {89, 89, 89},
    {121, 121, 121},
    {37, 146, 91},
    {159, 32, 96},
    {107, 188, 1},
    {211, 124, 131},
    {26, 209, 94},
    {141, 110, 153},
    {70, 165, 50},
    {242, 69, 89},
    {56, 215, 56},
    {182, 73, 236},
    {190, 190, 190},
    {12, 12, 12},
    {20, 20, 20},
    {198, 198, 198},
    {119, 9, 45},
    {167, 167, 8},
    {51, 5, 152},
    {229, 133, 82},
    {78, 46, 90},
    {145, 217, 181},
    {20, 60, 68},
    {202, 156, 97},
    {96, 96, 159},
    {180, 226, 107},
    {113, 113, 113},
    {97, 97, 97},
    {212, 130, 202},
    {0, 255, 255},
    {142, 144, 212},
    {92, 116, 108},
    {241, 185, 22},
    {46, 78, 249},
    {181, 181, 161},
    {99, 53, 105},
    {199, 199, 167},
    {23, 105, 140},
    {215, 215, 215},
    {36, 36, 36},
    {255, 255, 255},
    {174, 174, 174},
    {230, 121, 29},
    {88, 183, 151},
    {146, 37, 248},
    {4, 203, 243},
    {201, 98, 46},
    {122, 177, 253},
    {179, 28, 36},
    {59, 140, 240},
    {239, 16, 143},
    {69, 242, 250},
    {138, 138, 138},
    {73, 73, 73},
    {93, 138, 35},
    {251, 251, 251},
    {47, 176, 180},
    {171, 100, 251},
    {98, 201, 38},
    {221, 94, 106},
    {22, 149, 193},
    {129, 42, 6},
    {76, 131, 219},
    {254, 1, 196},
    {239, 239, 239},
    {61, 61, 61},
    {150, 150, 150},
    {5, 51, 186},
    {207, 175, 112},
    {123, 77, 176},
    {163, 235, 147},
    {58, 114, 189},
    {233, 193, 205},
    {68, 12, 179},
    {154, 172, 144},
    {24, 120, 215},
    {198, 216, 252},
    {162, 162, 162},
    {255, 255, 255},
    {49, 49, 49},
    {227, 227, 227},
    {222, 160, 37},
    {12, 68, 155},
    {130, 212, 73},
    {86, 86, 129},
    {253, 253, 137},
    {34, 10, 102},
    {187, 147, 76},
    {111, 113, 244},
    {255, 255, 0},
    {29, 75, 99},
    {85, 85, 85},
    {125, 125, 125},
    {164, 19, 218},
    {32, 159, 223},
    {234, 61, 128},
    {84, 251, 4},
    {153, 82, 221},
    {14, 237, 26},
    {197, 38, 177},
    {115, 196, 216},
    {183, 88, 183},
    {55, 200, 109},
    {186, 186, 186},
    {8, 8, 8},
    {24, 24, 24},
    {202, 202, 202},
    {87, 168, 204},
    {231, 104, 72},
    {35, 244, 41},
    {165, 70, 18},
    {110, 141, 185},
    {209, 26, 245},
    {28, 179, 44},
    {138, 93, 33},
    {64, 191, 64},
    {244, 35, 43},
    {109, 109, 109},
    {101, 101, 101},
    {85, 3, 75},
    {128, 128, 128},
    {15, 17, 85},
    {219, 243, 235},
    {114, 58, 149},
    {173, 205, 122},
    {54, 54, 34},
    {226, 180, 232},
    {72, 72, 40},
    {150, 232, 13},
    {210, 210, 210},
    {32, 32, 32},
    {0, 0, 0},
    {178, 178, 178},
    {166, 184, 93},
    {120, 24, 120},
    {210, 228, 184},
    {6, 102, 114},
    {137, 161, 110},
    {90, 18, 30},
    {243, 219, 100},
    {43, 125, 65},
    {175, 207, 207},
    {101, 83, 27},
    {134, 134, 134},
    {77, 77, 77},
    {220, 11, 162},
    {255, 0, 0},
    {174, 49, 53},
    {44, 227, 124},
    {225, 74, 165},
    {94, 221, 233},
    {149, 22, 66},
    {2, 169, 133},
    {203, 4, 92},
    {127, 128, 69},
    {235, 235, 235},
    {57, 57, 57},
    {255, 255, 255},
    {154, 154, 154},
    {7, 152, 61},
    {143, 112, 48},
    {91, 236, 81},
    {227, 44, 211},
    {42, 129, 14},
    {169, 2, 141},
    {100, 171, 84},
    {218, 109, 208},
    {16, 239, 175},
    {134, 25, 188},
    {158, 158, 158},
    {255, 255, 255},
    {53, 53, 53},
    {231, 231, 231},
    {95, 33, 164},
    {139, 195, 28},
    {3, 85, 200},
    {213, 213, 2},
    {126, 126, 10},
    {161, 137, 229},
    {60, 20, 203},
    {238, 240, 117},
    {0, 255, 0},
    {156, 202, 226},
    {81, 81, 81},
    {130, 130, 130},
    {228, 210, 154},
    {48, 80, 80},
    {170, 252, 192},
    {116, 92, 227},
    {217, 145, 157},
    {10, 34, 237},
    {133, 229, 241},
    {83, 101, 57},
    {247, 151, 247},
    {39, 57, 220},
    {182, 182, 182},
    {4, 4, 4},
    {28, 28, 28},
    {206, 206, 206},
    {214, 41, 77},
    {104, 231, 199},
    {162, 117, 168},
    {38, 197, 145},
    {237, 14, 58},
    {82, 153, 118},
    {155, 52, 171},
    {11, 220, 160},
    {191, 64, 191},
    {117, 162, 170},
    {105, 105, 105}
};

RGBtype patches_large[large_size] =
{
    {117, 162, 170},
    {4, 4, 4},
    {102, 70, 142},
    {41, 104, 212},
    {49, 174, 31},
    {11, 220, 160},
    {92, 92, 92},
    {19, 26, 106},
    {88, 56, 112},
    {77, 251, 123},
    {82, 153, 118},
    {181, 181, 181},
    {75, 93, 189},
    {4, 76, 28},
    {23, 233, 101},
    {38, 197, 145},
    {191, 64, 191},
    {54, 35, 183},
    {122, 50, 22},
    {99, 181, 129},
    {104, 231, 199},
    {155, 52, 171},
    {59, 13, 25},
    {46, 206, 18},
    {247, 200, 241},
    {231, 231, 231},
    {237, 14, 58},
    {69, 115, 19},
    {92, 244, 132},
    {179, 196, 69},
    {143, 143, 143},
    {162, 117, 168},
    {25, 7, 127},
    {0, 128, 232},
    {206, 237, 134},
    {54, 54, 54},
    {214, 41, 77},
    {105, 23, 134},
    {124, 148, 208},
    {148, 251, 152},
    {140, 59, 83},
    {39, 57, 220},
    {42, 42, 42},
    {63, 253, 23},
    {224, 160, 124},
    {241, 111, 55},
    {83, 101, 57},
    {131, 131, 131},
    {67, 212, 213},
    {172, 228, 20},
    {181, 99, 131},
    {10, 34, 237},
    {219, 219, 219},
    {32, 161, 185},
    {222, 222, 130},
    {203, 29, 137},
    {116, 92, 227},
    {247, 151, 247},
    {130, 170, 238},
    {146, 90, 94},
    {149, 3, 215},
    {48, 80, 80},
    {133, 229, 241},
    {255, 129, 45},
    {230, 6, 186},
    {207, 17, 201},
    {193, 193, 193},
    {217, 145, 157},
    {189, 235, 235},
    {168, 40, 224},
    {178, 58, 11},
    {104, 104, 104},
    {170, 252, 192},
    {192, 190, 142},
    {216, 120, 68},
    {246, 54, 190},
    {16, 16, 16},
    {228, 210, 154},
    {156, 202, 226},
    {255, 255, 255},
    {132, 12, 41},
    {136, 72, 180},
    {144, 142, 127},
    {238, 240, 117},
    {80, 80, 80},
    {250, 114, 35},
    {208, 48, 152},
    {201, 183, 15},
    {161, 137, 229},
    {169, 169, 169},
    {187, 77, 46},
    {164, 108, 124},
    {183, 201, 5},
    {213, 213, 2},
    {0, 255, 0},
    {197, 51, 40},
    {234, 66, 38},
    {243, 197, 177},
    {139, 195, 28},
    {60, 20, 203},
    {155, 45, 121},
    {142, 238, 114},
    {103, 185, 193},
    {243, 243, 243},
    {126, 126, 10},
    {199, 89, 21},
    {212, 252, 108},
    {20, 228, 38},
    {155, 155, 155},
    {3, 85, 200},
    {185, 39, 31},
    {160, 160, 136},
    {74, 162, 242},
    {66, 66, 66},
    {95, 33, 164},
    {248, 103, 182},
    {236, 227, 224},
    {53, 219, 248},
    {45, 27, 51},
    {134, 25, 188},
    {30, 30, 30},
    {158, 221, 118},
    {112, 207, 76},
    {97, 31, 7},
    {218, 109, 208},
    {118, 118, 118},
    {194, 170, 27},
    {12, 196, 116},
    {21, 67, 227},
    {169, 2, 141},
    {207, 207, 207},
    {191, 129, 217},
    {86, 214, 106},
    {79, 81, 253},
    {227, 44, 211},
    {16, 239, 175},
    {34, 138, 142},
    {50, 122, 62},
    {143, 112, 48},
    {100, 171, 84},
    {111, 241, 29},
    {118, 118, 138},
    {64, 1, 89},
    {205, 205, 205},
    {42, 129, 14},
    {29, 203, 139},
    {8, 8, 128},
    {66, 42, 154},
    {116, 116, 116},
    {91, 236, 81},
    {65, 191, 231},
    {80, 112, 172},
    {31, 94, 246},
    {28, 28, 28},
    {7, 152, 61},
    {61, 234, 130},
    {37, 44, 72},
    {108, 100, 96},
    {121, 230, 55},
    {127, 128, 69},
    {68, 68, 68},
    {106, 2, 19},
    {32, 32, 8},
    {57, 167, 159},
    {2, 169, 133},
    {157, 157, 157},
    {28, 109, 77},
    {84, 124, 236},
    {71, 217, 149},
    {94, 221, 233},
    {245, 245, 245},
    {71, 25, 34},
    {14, 110, 242},
    {27, 173, 249},
    {44, 227, 124},
    {203, 4, 92},
    {115, 69, 49},
    {106, 194, 166},
    {133, 242, 103},
    {255, 0, 0},
    {149, 22, 66},
    {55, 73, 133},
    {36, 236, 252},
    {251, 141, 109},
    {167, 167, 167},
    {225, 74, 165},
    {73, 55, 143},
    {80, 176, 24},
    {186, 178, 98},
    {78, 78, 78},
    {174, 49, 53},
    {17, 15, 254},
    {9, 199, 53},
    {196, 203, 104},
    {255, 255, 255},
    {220, 11, 162},
    {101, 83, 27},
    {18, 18, 18},
    {118, 182, 63},
    {152, 184, 5},
    {128, 127, 206},
    {43, 125, 65},
    {106, 106, 106},
    {51, 186, 138},
    {232, 232, 160},
    {253, 43, 171},
    {90, 18, 30},
    {195, 195, 195},
    {79, 145, 73},
    {166, 198, 250},
    {191, 65, 109},
    {6, 102, 114},
    {175, 207, 207},
    {22, 131, 87},
    {210, 154, 30},
    {194, 106, 174},
    {120, 24, 120},
    {243, 219, 100},
    {139, 221, 201},
    {158, 30, 194},
    {159, 33, 58},
    {217, 217, 217},
    {137, 161, 110},
    {245, 163, 195},
    {236, 36, 84},
    {195, 85, 85},
    {129, 129, 129},
    {210, 228, 184},
    {177, 175, 119},
    {160, 96, 61},
    {190, 126, 150},
    {40, 40, 40},
    {166, 184, 93},
    {204, 250, 19},
    {212, 60, 216},
    {252, 20, 80},
    {232, 150, 7},
    {150, 232, 13},
    {56, 56, 56},
    {142, 46, 198},
    {128, 0, 104},
    {153, 135, 255},
    {226, 180, 232},
    {145, 145, 145},
    {243, 5, 6},
    {220, 116, 4},
    {197, 243, 147},
    {173, 205, 122},
    {233, 233, 233},
    {183, 9, 177},
    {174, 78, 146},
    {187, 141, 153},
    {219, 243, 235},
    {72, 72, 40},
    {227, 53, 1},
    {250, 178, 150},
    {38, 210, 8},
    {128, 128, 128},
    {54, 54, 34},
    {151, 105, 229},
    {132, 204, 156},
    {107, 252, 93},
    {179, 179, 179},
    {114, 58, 149},
    {205, 123, 251},
    {216, 184, 240},
    {27, 146, 3},
    {90, 90, 90},
    {15, 17, 85},
    {176, 47, 158},
    {168, 231, 84},
    {70, 229, 110},
    {2, 2, 2},
    {85, 3, 75},
    {244, 35, 43},
    {6, 6, 6},
    {230, 197, 15},
    {57, 152, 100},
    {33, 95, 174},
    {138, 93, 33},
    {94, 94, 94},
    {146, 154, 234},
    {120, 152, 144},
    {109, 91, 155},
    {209, 26, 245},
    {183, 183, 183},
    {203, 220, 62},
    {6, 230, 154},
    {31, 97, 13},
    {165, 70, 18},
    {64, 191, 64},
    {181, 163, 56},
    {90, 146, 246},
    {67, 21, 97},
    {231, 104, 72},
    {28, 179, 44},
    {43, 253, 169},
    {62, 62, 162},
    {119, 73, 113},
    {229, 229, 229},
    {110, 141, 185},
    {101, 211, 243},
    {124, 84, 100},
    {52, 69, 197},
    {141, 141, 141},
    {35, 244, 41},
    {17, 143, 23},
    {1, 64, 92},
    {78, 110, 7},
    {52, 52, 52},
    {87, 168, 204},
    {73, 182, 103},
    {92, 52, 49},
    {21, 124, 25},
    {13, 186, 210},
    {55, 200, 109},
    {44, 44, 44},
    {47, 14, 166},
    {96, 32, 252},
    {113, 239, 183},
    {115, 196, 216},
    {133, 133, 133},
    {99, 117, 54},
    {44, 100, 148},
    {53, 227, 3},
    {14, 237, 26},
    {221, 221, 221},
    {24, 41, 209},
    {94, 94, 2},
    {75, 157, 9},
    {84, 251, 4},
    {183, 88, 183},
    {2, 42, 110},
    {18, 218, 222},
    {213, 194, 151},
    {32, 159, 223},
    {197, 38, 177},
    {127, 1, 173},
    {102, 134, 58},
    {143, 208, 137},
    {191, 191, 191},
    {153, 82, 221},
    {61, 107, 107},
    {40, 168, 96},
    {242, 249, 74},
    {102, 102, 102},
    {234, 61, 128},
    {65, 63, 15},
    {88, 247, 196},
    {182, 245, 254},
    {14, 14, 14},
    {164, 19, 218},
    {29, 75, 99},
    {255, 255, 255},
    {5, 140, 168},
    {200, 136, 244},
    {208, 79, 63},
    {111, 113, 244},
    {82, 82, 82},
    {122, 241, 162},
    {144, 240, 216},
    {137, 119, 79},
    {34, 10, 102},
    {171, 171, 171},
    {60, 204, 173},
    {228, 172, 60},
    {247, 9, 69},
    {86, 86, 129},
    {255, 255, 0},
    {69, 179, 167},
    {170, 130, 102},
    {179, 5, 241},
    {12, 68, 155},
    {187, 147, 76},
    {219, 237, 57},
    {206, 46, 50},
    {231, 57, 65},
    {241, 241, 241},
    {253, 253, 137},
    {135, 153, 85},
    {148, 60, 44},
    {147, 101, 165},
    {153, 153, 153},
    {130, 212, 73},
    {249, 231, 95},
    {224, 96, 200},
    {202, 34, 114},
    {64, 64, 64},
    {222, 160, 37},
    {184, 166, 246},
    {172, 36, 160},
    {180, 92, 120},
    {172, 154, 178},
    {198, 216, 252},
    {32, 32, 32},
    {222, 30, 55},
    {240, 80, 204},
    {225, 159, 135},
    {154, 172, 144},
    {120, 120, 120},
    {130, 106, 90},
    {140, 68, 244},
    {149, 195, 99},
    {233, 193, 205},
    {209, 209, 209},
    {255, 65, 153},
    {214, 86, 234},
    {207, 209, 125},
    {163, 235, 147},
    {24, 120, 215},
    {162, 10, 14},
    {178, 250, 190},
    {93, 202, 127},
    {207, 175, 112},
    {68, 12, 179},
    {239, 113, 157},
    {246, 246, 10},
    {48, 240, 233},
    {203, 203, 203},
    {58, 114, 189},
    {157, 75, 11},
    {136, 136, 0},
    {98, 138, 122},
    {114, 114, 114},
    {123, 77, 176},
    {193, 63, 103},
    {208, 239, 45},
    {23, 213, 158},
    {26, 26, 26},
    {5, 51, 186},
    {188, 107, 3},
    {164, 172, 200},
    {76, 195, 128},
    {89, 71, 214},
    {254, 1, 196},
    {70, 70, 70},
    {234, 130, 146},
    {48, 208, 184},
    {41, 87, 47},
    {129, 42, 6},
    {159, 159, 159},
    {155, 236, 205},
    {116, 220, 12},
    {103, 121, 117},
    {221, 94, 106},
    {247, 247, 247},
    {199, 153, 161},
    {10, 162, 6},
    {19, 37, 145},
    {171, 100, 251},
    {76, 131, 219},
    {83, 229, 209},
    {74, 98, 70},
    {6, 115, 231},
    {253, 253, 253},
    {22, 149, 193},
    {39, 185, 53},
    {52, 28, 76},
    {123, 13, 237},
    {165, 165, 165},
    {98, 201, 38},
    {105, 151, 111},
    {112, 16, 248},
    {59, 50, 226},
    {76, 76, 76},
    {47, 176, 180},
    {25, 134, 150},
    {13, 4, 192},
    {68, 76, 232},
    {255, 255, 255},
    {93, 138, 35},
    {69, 242, 250},
    {20, 20, 20},
    {86, 22, 222},
    {25, 56, 132},
    {1, 254, 79},
    {59, 140, 240},
    {108, 108, 108},
    {35, 74, 59},
    {104, 104, 32},
    {125, 171, 43},
    {122, 177, 253},
    {197, 197, 197},
    {111, 49, 169},
    {38, 70, 122},
    {63, 193, 237},
    {4, 203, 243},
    {239, 16, 143},
    {30, 11, 64},
    {82, 26, 158},
    {66, 234, 46},
    {88, 183, 151},
    {179, 28, 36},
    {11, 93, 73},
    {30, 158, 66},
    {223, 224, 121},
    {215, 215, 215},
    {201, 98, 46},
    {117, 35, 67},
    {108, 164, 212},
    {131, 149, 22},
    {126, 126, 126},
    {146, 37, 248},
    {49, 47, 247},
    {33, 223, 188},
    {254, 190, 214},
    {38, 38, 38},
    {230, 121, 29},
    {77, 123, 146},
    {84, 188, 88},
    {188, 211, 17},
    {168, 87, 71},
    {23, 105, 140},
    {58, 58, 58},
    {15, 174, 70},
    {192, 192, 40},
    {217, 71, 191},
    {99, 53, 105},
    {147, 147, 147},
    {115, 133, 133},
    {156, 180, 68},
    {133, 51, 211},
    {46, 78, 249},
    {235, 235, 235},
    {56, 137, 50},
    {238, 142, 210},
    {251, 77, 217},
    {92, 116, 108},
    {199, 199, 167},
    {163, 245, 65},
    {186, 114, 214},
    {165, 83, 135},
    {0, 255, 255},
    {181, 181, 161},
    {215, 169, 165},
    {196, 12, 220},
    {235, 125, 221},
    {177, 177, 177},
    {241, 185, 22},
    {141, 187, 187},
    {152, 120, 176},
    {154, 18, 130},
    {88, 88, 88},
    {142, 144, 212},
    {240, 238, 222},
    {232, 40, 21},
    {198, 102, 238},
    {0, 0, 0},
    {212, 130, 202},
    {180, 226, 107},
    {8, 8, 8},
    {166, 6, 78},
    {184, 24, 228},
    {160, 222, 47},
    {202, 156, 97},
    {96, 96, 96},
    {210, 90, 170},
    {248, 24, 16},
    {237, 219, 27},
    {145, 217, 181},
    {185, 185, 185},
    {139, 29, 125},
    {134, 102, 26},
    {159, 225, 141},
    {229, 133, 82},
    {96, 96, 159},
    {245, 99, 119},
    {218, 18, 118},
    {195, 149, 225},
    {167, 167, 8},
    {20, 60, 68},
    {171, 125, 41},
    {190, 190, 34},
    {87, 232, 145},
    {227, 227, 227},
    {78, 46, 90},
    {229, 83, 115},
    {252, 212, 228},
    {36, 181, 117},
    {139, 139, 139},
    {51, 5, 152},
    {145, 15, 151},
    {128, 191, 220},
    {110, 205, 230},
    {50, 50, 50},
    {119, 9, 45},
    {200, 55, 230},
    {220, 180, 176},
    {29, 243, 112},
    {9, 119, 39},
    {182, 73, 236},
    {46, 46, 46},
    {174, 142, 39},
    {64, 128, 29},
    {81, 79, 87},
    {242, 69, 89},
    {135, 135, 135},
    {227, 244, 181},
    {60, 148, 36},
    {37, 19, 179},
    {141, 110, 153},
    {223, 223, 223},
    {151, 169, 81},
    {126, 254, 226},
    {107, 61, 233},
    {211, 124, 131},
    {56, 215, 56},
    {3, 213, 33},
    {26, 82, 182},
    {85, 67, 24},
    {159, 32, 96},
    {70, 165, 50},
    {95, 161, 77},
    {70, 38, 218},
    {16, 81, 10},
    {189, 189, 189},
    {26, 209, 94},
    {45, 155, 219},
    {56, 88, 208},
    {114, 122, 202},
    {100, 100, 100},
    {107, 188, 1},
    {97, 158, 238},
    {120, 88, 37},
    {55, 118, 126},
    {12, 12, 12},
    {37, 146, 91},
    {21, 194, 11},
    {255, 255, 255},
    {7, 38, 47},
    {72, 8, 116},
    {81, 206, 190},
    {79, 208, 21},
    {84, 84, 84},
    {90, 82, 66},
    {16, 112, 88},
    {9, 247, 207},
    {50, 249, 213},
    {173, 173, 173},
    {44, 61, 30},
    {100, 44, 188},
    {119, 137, 197},
    {118, 245, 98},
    {0, 0, 255},
    {101, 19, 71},
    {42, 2, 230},
    {51, 133, 113},
    {8, 135, 104},
    {251, 84, 12},
    {91, 109, 185},
    {78, 174, 178},
    {167, 248, 2},
    {239, 239, 239},
    {189, 62, 201},
    {7, 25, 213},
    {20, 188, 172},
    {211, 165, 229},
    {151, 151, 151},
    {194, 21, 9},
    {121, 103, 223},
    {96, 224, 72},
    {138, 225, 51},
    {62, 62, 62},
    {158, 97, 101},
    {57, 39, 119},
    {45, 164, 33},
    {244, 156, 57},
    {236, 91, 242},
    {71, 89, 125},
    {34, 34, 34},
    {94, 158, 182},
    {176, 144, 140},
    {161, 95, 199},
    {27, 45, 17},
    {122, 122, 122},
    {3, 233, 218},
    {204, 132, 180},
    {213, 3, 35},
    {106, 66, 78},
    {211, 211, 211},
    {127, 192, 26},
    {150, 150, 170},
    {143, 17, 61},
    {36, 108, 20},
    {151, 247, 88},
    {226, 202, 78},
    {242, 58, 254},
    {221, 75, 255},
    {80, 48, 239},
    {195, 139, 52},
    {175, 177, 221},
    {182, 54, 74},
    {175, 113, 105},
    {201, 201, 201},
    {185, 241, 62},
    {221, 139, 75},
    {200, 72, 64},
    {226, 10, 250},
    {112, 112, 112},
    {250, 204, 49},
    {129, 255, 39},
    {144, 48, 108},
    {150, 86, 31},
    {24, 24, 24},
    {132, 178, 59},
    {252, 170, 67},
    {255, 255, 255},
    {228, 108, 136},
    {204, 68, 1},
    {216, 198, 87},
    {190, 192, 132},
    {72, 72, 72},
    {170, 66, 210},
    {176, 80, 56},
    {169, 215, 175},
    {193, 233, 70},
    {161, 161, 161},
    {219, 45, 141},
    {244, 92, 140},
    {231, 249, 245},
    {157, 157, 42},
    {249, 249, 249},
    {135, 89, 225},
    {138, 34, 134},
    {147, 165, 17},
    {235, 163, 187},
    {108, 36, 60},
    {211, 101, 81},
    {202, 226, 198},
    {8, 216, 97},
    {251, 251, 251},
    {30, 30, 169},
    {167, 57, 181},
    {180, 156, 204},
    {91, 173, 14},
    {163, 163, 163},
    {66, 106, 197},
    {233, 23, 239},
    {240, 144, 120},
    {43, 193, 82},
    {74, 74, 74},
    {63, 65, 5},
    {152, 7, 23},
    {140, 132, 64},
    {100, 235, 9},
    {255, 255, 255},
    {125, 43, 194},
    {196, 115, 123},
    {22, 22, 22},
    {214, 150, 94},
    {17, 176, 236},
    {1, 127, 167},
    {186, 13, 113},
    {110, 110, 110},
    {162, 201, 186},
    {72, 200, 192},
    {93, 11, 203},
    {249, 50, 126},
    {199, 199, 199},
    {239, 177, 42},
    {54, 182, 202},
    {47, 49, 93},
    {131, 76, 116},
    {112, 143, 16},
    {157, 139, 191},
    {114, 186, 126},
    {98, 74, 206},
    {215, 56, 24},
    {52, 155, 163},
    {15, 145, 189},
    {22, 22, 42},
    {95, 97, 249},
    {213, 213, 213},
    {74, 225, 173},
    {85, 131, 163},
    {76, 4, 52},
    {4, 21, 149},
    {124, 124, 124},
    {19, 164, 121},
    {33, 223, 71},
    {49, 16, 13},
    {126, 62, 86},
    {36, 36, 36},
    {103, 248, 156},
    {109, 218, 115},
    {116, 28, 184},
    {61, 84, 144},
    {41, 214, 198},
    {31, 224, 228},
    {60, 60, 60},
    {11, 98, 178},
    {64, 64, 168},
    {89, 199, 63},
    {67, 148, 136},
    {149, 149, 149},
    {83, 37, 101},
    {28, 52, 196},
    {5, 179, 83},
    {62, 189, 74},
    {237, 237, 237},
    {40, 121, 129},
    {110, 14, 82},
    {123, 205, 89},
    {124, 211, 139},
    {135, 8, 231},
    {35, 117, 193},
    {58, 242, 86},
    {229, 147, 199},
    {255, 0, 255},
    {245, 118, 225},
    {87, 41, 37},
    {68, 140, 92},
    {171, 189, 157},
    {175, 175, 175},
    {177, 122, 86},
    {13, 59, 59},
    {24, 248, 48},
    {218, 209, 194},
    {86, 86, 86},
    {206, 81, 148},
    {113, 111, 95},
    {104, 168, 148},
    {134, 166, 174},
    {255, 255, 255},
    {148, 67, 138},
    {53, 99, 234},
    {10, 10, 10},
    {39, 134, 206},
    {248, 215, 164},
    {224, 31, 111},
    {75, 29, 224},
    {98, 98, 98},
    {82, 217, 43},
    {184, 216, 80},
    {173, 27, 91},
    {18, 90, 54},
    {187, 187, 187},
    {12, 157, 253},
    {198, 166, 90},
    {223, 33, 205},
    {102, 6, 209},
    {223, 223, 32},
    {117, 226, 247},
    {154, 210, 54},
    {131, 85, 161},
    {40, 40, 135},
    {147, 187, 195},
    {235, 189, 105},
    {254, 126, 98},
    {215, 105, 18},
    {225, 225, 225},
    {205, 173, 217},
    {165, 147, 51},
    {188, 20, 164},
    {163, 53, 245},
    {137, 137, 137},
    {178, 132, 25},
    {209, 207, 215},
    {192, 0, 156},
    {238, 78, 102},
    {48, 48, 48},
    {246, 136, 172},
    {136, 246, 166},
    {156, 116, 240}
};

RGBtype patches_huge[huge_size] =
{
    {171, 173, 88},
    {93, 148, 86},
    {208, 143, 139},
    {38, 197, 145},
    {255, 255, 255},
    {4, 76, 28},
    {163, 21, 123},
    {87, 10, 171},
    {13, 28, 181},
    {237, 110, 232},
    {122, 50, 22},
    {82, 153, 118},
    {165, 211, 111},
    {1, 224, 58},
    {228, 131, 2},
    {217, 194, 7},
    {47, 174, 191},
    {137, 183, 251},
    {104, 231, 199},
    {42, 42, 42},
    {88, 56, 112},
    {215, 73, 159},
    {35, 85, 80},
    {113, 79, 209},
    {156, 84, 126},
    {59, 13, 25},
    {11, 220, 160},
    {234, 253, 53},
    {124, 243, 2},
    {145, 222, 229},
    {134, 182, 107},
    {99, 213, 83},
    {247, 200, 241},
    {243, 243, 243},
    {86, 86, 86},
    {41, 104, 212},
    {141, 91, 129},
    {110, 46, 220},
    {53, 67, 101},
    {192, 32, 19},
    {69, 115, 19},
    {117, 162, 170},
    {154, 146, 106},
    {62, 158, 196},
    {247, 200, 97},
    {24, 137, 182},
    {179, 196, 69},
    {199, 199, 199},
    {130, 130, 130},
    {102, 70, 142},
    {241, 16, 228},
    {28, 21, 75},
    {75, 61, 111},
    {188, 84, 247},
    {25, 7, 127},
    {49, 174, 31},
    {197, 230, 6},
    {67, 180, 7},
    {183, 248, 222},
    {78, 155, 168},
    {206, 237, 134},
    {155, 155, 155},
    {174, 174, 174},
    {19, 26, 106},
    {181, 4, 81},
    {64, 96, 39},
    {19, 122, 184},
    {253, 63, 62},
    {105, 23, 134},
    {77, 251, 123},
    {184, 152, 12},
    {32, 192, 107},
    {197, 134, 212},
    {49, 207, 205},
    {148, 251, 152},
    {111, 111, 111},
    {218, 218, 218},
    {75, 93, 189},
    {203, 125, 91},
    {60, 68, 85},
    {241, 64, 174},
    {102, 38, 92},
    {131, 22, 252},
    {39, 57, 220},
    {23, 233, 101},
    {247, 247, 42},
    {109, 250, 252},
    {154, 242, 184},
    {117, 195, 121},
    {224, 160, 124},
    {67, 67, 67},
    {191, 64, 191},
    {54, 35, 183},
    {146, 59, 140},
    {126, 46, 147},
    {141, 107, 202},
    {40, 8, 6},
    {222, 97, 145},
    {83, 101, 57},
    {99, 181, 129},
    {134, 134, 32},
    {33, 255, 201},
    {234, 157, 231},
    {11, 189, 115},
    {172, 228, 20},
    {23, 23, 23},
    {155, 52, 171},
    {113, 47, 4},
    {230, 102, 104},
    {3, 5, 80},
    {214, 121, 211},
    {89, 89, 161},
    {173, 91, 7},
    {10, 34, 237},
    {46, 206, 18},
    {218, 241, 76},
    {85, 163, 46},
    {164, 179, 189},
    {82, 249, 164},
    {222, 222, 130},
    {237, 14, 58},
    {13, 123, 103},
    {168, 73, 50},
    {95, 113, 60},
    {163, 37, 48},
    {5, 45, 205},
    {225, 95, 52},
    {116, 92, 227},
    {17, 17, 17},
    {92, 244, 132},
    {171, 157, 20},
    {15, 177, 52},
    {208, 239, 90},
    {38, 166, 64},
    {130, 170, 238},
    {162, 117, 168},
    {87, 105, 121},
    {216, 24, 150},
    {45, 75, 170},
    {237, 94, 163},
    {122, 83, 195},
    {149, 3, 215},
    {48, 80, 80},
    {61, 61, 61},
    {0, 128, 232},
    {228, 179, 73},
    {114, 154, 240},
    {137, 214, 41},
    {104, 136, 27},
    {255, 129, 45},
    {214, 41, 77},
    {35, 54, 157},
    {132, 108, 249},
    {97, 80, 158},
    {156, 100, 53},
    {60, 109, 202},
    {207, 17, 201},
    {224, 224, 224},
    {105, 105, 105},
    {124, 148, 208},
    {145, 238, 173},
    {54, 150, 69},
    {247, 168, 35},
    {24, 184, 108},
    {189, 235, 235},
    {140, 59, 83},
    {110, 78, 14},
    {250, 18, 239},
    {191, 16, 89},
    {70, 20, 192},
    {178, 58, 11},
    {180, 180, 180},
    {149, 149, 149},
    {63, 253, 23},
    {201, 215, 220},
    {72, 232, 79},
    {179, 164, 151},
    {68, 204, 0},
    {192, 190, 142},
    {241, 111, 55},
    {28, 116, 152},
    {187, 45, 254},
    {187, 52, 43},
    {26, 103, 172},
    {246, 54, 190},
    {136, 136, 136},
    {193, 193, 193},
    {67, 212, 213},
    {183, 169, 214},
    {16, 160, 41},
    {206, 142, 84},
    {58, 178, 10},
    {156, 202, 226},
    {181, 99, 131},
    {64, 1, 243},
    {197, 83, 244},
    {253, 94, 236},
    {105, 119, 85},
    {136, 72, 180},
    {92, 92, 92},
    {237, 237, 237},
    {32, 161, 185},
    {243, 165, 99},
    {100, 251, 205},
    {49, 254, 134},
    {148, 156, 74},
    {123, 141, 5},
    {238, 240, 117},
    {203, 29, 137},
    {60, 116, 31},
    {153, 39, 153},
    {131, 117, 48},
    {39, 89, 16},
    {208, 48, 152},
    {48, 48, 48},
    {247, 151, 247},
    {109, 155, 48},
    {142, 142, 159},
    {42, 209, 151},
    {117, 242, 50},
    {224, 191, 174},
    {5, 243, 15},
    {161, 137, 229},
    {146, 90, 94},
    {126, 30, 216},
    {232, 55, 97},
    {222, 2, 68},
    {83, 5, 235},
    {164, 108, 124},
    {4, 4, 4},
    {133, 229, 241},
    {33, 159, 27},
    {212, 156, 189},
    {91, 189, 200},
    {11, 141, 56},
    {172, 132, 197},
    {89, 135, 99},
    {213, 213, 2},
    {255, 255, 255},
    {230, 6, 186},
    {3, 53, 28},
    {166, 26, 60},
    {172, 59, 213},
    {10, 66, 60},
    {234, 66, 38},
    {217, 145, 157},
    {85, 195, 254},
    {160, 192, 90},
    {7, 201, 164},
    {82, 202, 238},
    {222, 190, 80},
    {42, 151, 155},
    {139, 195, 28},
    {36, 36, 36},
    {168, 40, 224},
    {95, 65, 119},
    {210, 69, 223},
    {225, 63, 224},
    {116, 60, 54},
    {155, 45, 121},
    {170, 252, 192},
    {15, 209, 224},
    {236, 132, 50},
    {121, 183, 174},
    {38, 150, 11},
    {130, 202, 60},
    {103, 185, 193},
    {249, 249, 249},
    {80, 80, 80},
    {216, 120, 68},
    {45, 123, 225},
    {138, 3, 9},
    {149, 99, 5},
    {48, 48, 130},
    {199, 89, 21},
    {228, 210, 154},
    {114, 250, 34},
    {158, 189, 164},
    {104, 184, 81},
    {255, 225, 254},
    {20, 228, 38},
    {205, 205, 205},
    {124, 124, 124},
    {132, 12, 41},
    {97, 127, 212},
    {243, 124, 3},
    {207, 113, 27},
    {76, 68, 104},
    {185, 39, 31},
    {144, 142, 127},
    {54, 246, 150},
    {194, 201, 200},
    {24, 216, 190},
    {189, 139, 57},
    {74, 162, 242},
    {161, 161, 161},
    {168, 168, 168},
    {250, 114, 35},
    {22, 36, 49},
    {175, 112, 182},
    {178, 90, 216},
    {22, 86, 118},
    {248, 103, 182},
    {201, 183, 15},
    {72, 136, 156},
    {191, 224, 11},
    {68, 172, 210},
    {193, 223, 93},
    {53, 219, 248},
    {117, 117, 117},
    {212, 212, 212},
    {187, 77, 46},
    {80, 50, 47},
    {203, 84, 197},
    {246, 86, 108},
    {98, 11, 146},
    {134, 25, 188},
    {183, 201, 5},
    {16, 144, 98},
    {253, 139, 205},
    {58, 210, 216},
    {157, 171, 49},
    {112, 207, 76},
    {73, 73, 73},
    {0, 255, 0},
    {197, 51, 40},
    {51, 27, 235},
    {150, 70, 218},
    {105, 71, 30},
    {136, 40, 102},
    {47, 113, 1},
    {218, 109, 208},
    {243, 197, 177},
    {100, 204, 134},
    {129, 223, 169},
    {123, 237, 215},
    {238, 145, 167},
    {12, 196, 116},
    {29, 29, 29},
    {60, 20, 203},
    {153, 71, 75},
    {118, 23, 88},
    {225, 26, 63},
    {39, 105, 68},
    {208, 81, 74},
    {93, 75, 151},
    {169, 2, 141},
    {142, 238, 114},
    {42, 225, 220},
    {220, 171, 197},
    {5, 147, 221},
    {162, 233, 52},
    {86, 214, 106},
    {126, 126, 10},
    {232, 87, 179},
    {9, 104, 82},
    {175, 97, 172},
    {83, 53, 160},
    {164, 13, 173},
    {1, 64, 250},
    {227, 44, 211},
    {11, 11, 11},
    {212, 252, 108},
    {91, 141, 131},
    {174, 145, 84},
    {89, 231, 177},
    {214, 182, 208},
    {34, 138, 142},
    {3, 85, 200},
    {167, 121, 232},
    {80, 16, 126},
    {220, 91, 58},
    {10, 114, 119},
    {234, 35, 243},
    {125, 107, 159},
    {143, 112, 48},
    {55, 55, 55},
    {160, 160, 136},
    {7, 248, 239},
    {225, 234, 192},
    {41, 246, 73},
    {140, 164, 206},
    {111, 241, 29},
    {95, 33, 164},
    {210, 38, 13},
    {36, 76, 154},
    {129, 47, 86},
    {116, 12, 125},
    {155, 78, 170},
    {64, 1, 89},
    {230, 230, 230},
    {99, 99, 99},
    {236, 227, 224},
    {121, 135, 229},
    {150, 182, 37},
    {103, 216, 19},
    {239, 208, 36},
    {29, 203, 139},
    {45, 27, 51},
    {138, 98, 217},
    {106, 98, 192},
    {48, 0, 201},
    {199, 58, 198},
    {66, 42, 154},
    {186, 186, 186},
    {143, 143, 143},
    {158, 221, 118},
    {58, 199, 77},
    {203, 164, 59},
    {20, 133, 246},
    {180, 220, 144},
    {65, 191, 231},
    {97, 31, 7},
    {243, 29, 207},
    {28, 13, 159},
    {76, 36, 186},
    {185, 72, 204},
    {31, 94, 246},
    {142, 142, 142},
    {187, 187, 187},
    {194, 170, 27},
    {72, 185, 71},
    {175, 128, 73},
    {74, 193, 32},
    {201, 162, 154},
    {61, 234, 130},
    {21, 67, 227},
    {176, 17, 100},
    {71, 121, 242},
    {22, 54, 164},
    {248, 7, 101},
    {108, 100, 96},
    {98, 98, 98},
    {231, 231, 231},
    {191, 129, 217},
    {28, 205, 43},
    {243, 140, 252},
    {192, 238, 22},
    {52, 187, 42},
    {147, 229, 77},
    {127, 128, 69},
    {79, 81, 253},
    {204, 100, 142},
    {57, 7, 248},
    {98, 106, 65},
    {135, 121, 111},
    {32, 32, 8},
    {54, 54, 54},
    {16, 239, 175},
    {253, 234, 32},
    {106, 162, 116},
    {138, 241, 246},
    {157, 155, 122},
    {112, 175, 158},
    {230, 185, 169},
    {2, 169, 133},
    {50, 122, 62},
    {150, 118, 144},
    {121, 71, 82},
    {47, 18, 211},
    {219, 13, 4},
    {84, 124, 236},
    {10, 10, 10},
    {100, 171, 84},
    {129, 191, 123},
    {37, 140, 46},
    {210, 181, 32},
    {238, 161, 235},
    {13, 164, 165},
    {169, 151, 242},
    {94, 221, 233},
    {118, 118, 138},
    {225, 42, 116},
    {7, 57, 92},
    {93, 43, 70},
    {170, 98, 92},
    {14, 110, 242},
    {42, 129, 14},
    {220, 203, 23},
    {81, 208, 201},
    {167, 232, 196},
    {162, 217, 127},
    {87, 182, 183},
    {217, 135, 12},
    {44, 227, 124},
    {30, 30, 30},
    {8, 8, 128},
    {175, 81, 231},
    {91, 77, 56},
    {1, 95, 42},
    {227, 76, 7},
    {115, 69, 49},
    {91, 236, 81},
    {175, 241, 129},
    {9, 167, 229},
    {232, 199, 158},
    {213, 134, 155},
    {35, 234, 92},
    {133, 242, 103},
    {255, 0, 0},
    {74, 74, 74},
    {80, 112, 172},
    {221, 107, 113},
    {42, 34, 104},
    {125, 11, 77},
    {144, 16, 226},
    {55, 73, 133},
    {7, 152, 61},
    {225, 138, 18},
    {118, 213, 235},
    {140, 148, 133},
    {111, 145, 206},
    {251, 141, 109},
    {211, 211, 211},
    {118, 118, 118},
    {37, 44, 72},
    {129, 31, 30},
    {100, 13, 51},
    {63, 97, 139},
    {199, 9, 28},
    {73, 55, 143},
    {121, 230, 55},
    {150, 214, 245},
    {50, 217, 88},
    {239, 176, 246},
    {30, 171, 89},
    {186, 178, 98},
    {167, 167, 167},
    {162, 162, 162},
    {106, 2, 19},
    {253, 76, 121},
    {16, 80, 214},
    {66, 74, 72},
    {182, 118, 22},
    {17, 15, 254},
    {57, 167, 159},
    {203, 196, 231},
    {79, 240, 155},
    {179, 188, 67},
    {66, 223, 53},
    {196, 203, 104},
    {123, 123, 123},
    {206, 206, 206},
    {28, 109, 77},
    {191, 34, 190},
    {72, 24, 177},
    {31, 62, 36},
    {241, 122, 162},
    {101, 83, 27},
    {71, 217, 149},
    {176, 176, 3},
    {22, 226, 133},
    {201, 194, 73},
    {61, 139, 81},
    {152, 184, 5},
    {79, 79, 79},
    {250, 250, 250},
    {71, 25, 34},
    {194, 11, 124},
    {54, 102, 187},
    {248, 55, 46},
    {108, 4, 178},
    {143, 81, 97},
    {43, 125, 65},
    {27, 173, 249},
    {243, 188, 182},
    {97, 190, 97},
    {147, 133, 159},
    {127, 224, 151},
    {232, 232, 160},
    {35, 35, 35},
    {203, 4, 92},
    {58, 103, 44},
    {158, 126, 16},
    {114, 106, 15},
    {135, 73, 36},
    {33, 65, 217},
    {212, 67, 127},
    {90, 18, 30},
    {106, 194, 166},
    {138, 194, 188},
    {45, 187, 86},
    {230, 217, 123},
    {2, 202, 84},
    {166, 198, 250},
    {255, 255, 255},
    {149, 22, 66},
    {121, 40, 131},
    {236, 69, 134},
    {15, 65, 204},
    {218, 61, 72},
    {85, 29, 62},
    {161, 32, 155},
    {6, 102, 114},
    {5, 5, 5},
    {36, 236, 252},
    {211, 133, 107},
    {95, 129, 195},
    {168, 247, 34},
    {94, 190, 56},
    {210, 154, 30},
    {225, 74, 165},
    {7, 89, 137},
    {160, 0, 237},
    {85, 83, 209},
    {170, 82, 23},
    {15, 15, 32},
    {237, 27, 175},
    {120, 24, 120},
    {49, 49, 49},
    {80, 176, 24},
    {167, 217, 143},
    {3, 245, 175},
    {217, 230, 217},
    {44, 132, 174},
    {139, 221, 201},
    {174, 49, 53},
    {91, 46, 228},
    {212, 92, 10},
    {33, 15, 54},
    {227, 124, 77},
    {115, 38, 226},
    {159, 33, 58},
    {236, 236, 236},
    {93, 93, 93},
    {9, 199, 53},
    {232, 246, 213},
    {126, 222, 108},
    {133, 147, 181},
    {96, 160, 20},
    {245, 163, 195},
    {220, 11, 162},
    {42, 66, 186},
    {142, 78, 20},
    {144, 32, 169},
    {56, 42, 87},
    {195, 85, 85},
    {192, 192, 192},
    {137, 137, 137},
    {118, 182, 63},
    {153, 231, 45},
    {60, 180, 170},
    {251, 236, 191},
    {20, 252, 239},
    {177, 175, 119},
    {128, 127, 206},
    {100, 108, 223},
    {243, 101, 215},
    {199, 104, 206},
    {74, 88, 93},
    {190, 126, 150},
    {148, 148, 148},
    {181, 181, 181},
    {51, 186, 138},
    {197, 147, 65},
    {64, 144, 216},
    {186, 209, 176},
    {78, 238, 237},
    {204, 250, 19},
    {253, 43, 171},
    {16, 49, 4},
    {183, 105, 99},
    {181, 23, 196},
    {17, 111, 45},
    {252, 20, 80},
    {104, 104, 104},
    {225, 225, 225},
    {79, 145, 73},
    {187, 237, 75},
    {28, 227, 181},
    {65, 239, 126},
    {196, 171, 186},
    {51, 197, 45},
    {150, 232, 13},
    {191, 65, 109},
    {72, 40, 250},
    {201, 23, 105},
    {241, 27, 112},
    {101, 51, 201},
    {128, 0, 104},
    {60, 60, 60},
    {175, 207, 207},
    {22, 131, 87},
    {250, 210, 68},
    {110, 221, 35},
    {61, 187, 26},
    {152, 215, 214},
    {119, 201, 153},
    {226, 180, 232},
    {194, 106, 174},
    {54, 86, 240},
    {145, 47, 26},
    {143, 49, 179},
    {43, 29, 147},
    {220, 116, 4},
    {16, 16, 16},
    {243, 219, 100},
    {97, 222, 179},
    {132, 172, 78},
    {35, 165, 176},
    {127, 208, 219},
    {232, 136, 114},
    {9, 183, 147},
    {173, 205, 122},
    {158, 30, 194},
    {114, 90, 68},
    {228, 115, 253},
    {212, 35, 173},
    {90, 114, 204},
    {174, 78, 146},
    {137, 161, 110},
    {45, 219, 135},
    {216, 216, 34},
    {87, 248, 84},
    {2, 249, 31},
    {166, 166, 40},
    {81, 143, 227},
    {219, 243, 235},
    {24, 24, 24},
    {236, 36, 84},
    {15, 113, 135},
    {170, 93, 167},
    {160, 127, 74},
    {6, 6, 160},
    {227, 53, 1},
    {210, 228, 184},
    {95, 225, 17},
    {168, 136, 134},
    {13, 234, 74},
    {94, 142, 115},
    {210, 250, 203},
    {38, 210, 8},
    {128, 128, 128},
    {68, 68, 68},
    {160, 96, 61},
    {85, 99, 153},
    {217, 50, 248},
    {237, 123, 125},
    {120, 120, 170},
    {151, 105, 229},
    {166, 184, 93},
    {3, 149, 125},
    {230, 165, 220},
    {113, 191, 47},
    {44, 180, 228},
    {139, 189, 27},
    {107, 252, 93},
    {217, 217, 217},
    {112, 112, 112},
    {212, 60, 216},
    {33, 63, 125},
    {134, 70, 148},
    {159, 65, 234},
    {56, 25, 140},
    {205, 123, 251},
    {232, 150, 7},
    {126, 190, 190},
    {146, 249, 56},
    {96, 192, 198},
    {245, 195, 17},
    {27, 146, 3},
    {173, 173, 173},
    {156, 156, 156},
    {142, 46, 198},
    {109, 60, 73},
    {247, 56, 158},
    {195, 53, 135},
    {70, 102, 134},
    {176, 47, 158},
    {153, 135, 255},
    {60, 212, 120},
    {203, 188, 238},
    {20, 156, 35},
    {177, 207, 164},
    {70, 229, 110},
    {129, 129, 129},
    {200, 200, 200},
    {243, 5, 6},
    {32, 2, 222},
    {183, 8, 33},
    {190, 30, 68},
    {26, 19, 233},
    {244, 35, 43},
    {197, 243, 147},
    {64, 160, 146},
    {181, 194, 228},
    {78, 142, 61},
    {205, 155, 193},
    {57, 152, 100},
    {85, 85, 85},
    {244, 244, 244},
    {183, 9, 177},
    {68, 117, 178},
    {197, 118, 43},
    {17, 95, 102},
    {252, 116, 130},
    {107, 125, 181},
    {138, 93, 33},
    {187, 141, 153},
    {28, 212, 254},
    {241, 206, 81},
    {51, 165, 255},
    {151, 137, 223},
    {120, 152, 144},
    {41, 41, 41},
    {72, 72, 40},
    {201, 119, 187},
    {63, 95, 112},
    {154, 2, 71},
    {101, 3, 130},
    {128, 96, 185},
    {37, 83, 238},
    {209, 26, 245},
    {250, 178, 150},
    {110, 237, 104},
    {141, 155, 54},
    {119, 169, 76},
    {226, 213, 59},
    {6, 230, 154},
    {255, 255, 255},
    {54, 54, 34},
    {145, 79, 202},
    {124, 53, 181},
    {234, 109, 24},
    {43, 45, 215},
    {220, 21, 213},
    {81, 16, 11},
    {165, 70, 18},
    {255, 255, 255},
    {132, 204, 156},
    {35, 149, 251},
    {214, 137, 44},
    {9, 215, 66},
    {174, 174, 168},
    {90, 146, 246},
    {114, 58, 149},
    {228, 20, 47},
    {1, 32, 142},
    {165, 67, 66},
    {90, 66, 135},
    {174, 47, 64},
    {10, 55, 123},
    {231, 104, 72},
    {43, 43, 43},
    {216, 184, 240},
    {87, 201, 32},
    {162, 213, 207},
    {81, 238, 49},
    {220, 148, 62},
    {43, 253, 169},
    {15, 17, 85},
    {171, 62, 117},
    {92, 84, 225},
    {208, 31, 166},
    {6, 54, 234},
    {227, 86, 210},
    {119, 73, 113},
    {242, 242, 242},
    {87, 87, 87},
    {168, 231, 84},
    {13, 219, 2},
    {237, 174, 93},
    {37, 178, 213},
    {128, 223, 82},
    {101, 211, 243},
    {85, 3, 75},
    {217, 82, 42},
    {46, 110, 116},
    {120, 72, 224},
    {151, 10, 55},
    {52, 69, 197},
    {198, 198, 198},
    {131, 131, 131},
    {230, 197, 15},
    {113, 143, 101},
    {156, 148, 202},
    {107, 157, 143},
    {251, 148, 184},
    {17, 143, 23},
    {33, 95, 174},
    {134, 39, 70},
    {99, 21, 230},
    {56, 120, 95},
    {205, 28, 41},
    {78, 110, 7},
    {154, 154, 154},
    {175, 175, 175},
    {146, 154, 234},
    {54, 131, 208},
    {191, 224, 166},
    {27, 241, 208},
    {190, 254, 126},
    {73, 182, 103},
    {109, 91, 155},
    {247, 88, 76},
    {24, 73, 3},
    {70, 7, 85},
    {177, 79, 77},
    {21, 124, 25},
    {110, 110, 110},
    {219, 219, 219},
    {203, 220, 62},
    {76, 253, 218},
    {187, 196, 212},
    {177, 255, 238},
    {70, 134, 188},
    {194, 213, 189},
    {55, 200, 109},
    {31, 97, 13},
    {184, 56, 106},
    {77, 91, 29},
    {26, 114, 57},
    {244, 67, 249},
    {96, 32, 252},
    {66, 66, 66},
    {64, 191, 64},
    {181, 163, 56},
    {19, 186, 12},
    {253, 174, 19},
    {204, 171, 138},
    {56, 247, 182},
    {159, 161, 208},
    {115, 196, 216},
    {67, 21, 97},
    {198, 70, 96},
    {49, 16, 121},
    {107, 30, 103},
    {139, 61, 243},
    {44, 100, 148},
    {22, 22, 22},
    {28, 179, 44},
    {241, 175, 131},
    {102, 230, 231},
    {131, 133, 207},
    {151, 185, 148},
    {120, 248, 66},
    {236, 155, 71},
    {14, 237, 26},
    {62, 62, 162},
    {154, 50, 13},
    {117, 4, 205},
    {37, 51, 62},
    {210, 122, 36},
    {94, 94, 2},
    {110, 141, 185},
    {141, 251, 230},
    {41, 200, 177},
    {222, 240, 188},
    {226, 228, 112},
    {7, 134, 72},
    {161, 159, 115},
    {84, 251, 4},
    {18, 18, 18},
    {124, 84, 100},
    {234, 93, 83},
    {11, 125, 199},
    {81, 111, 217},
    {166, 38, 192},
    {2, 42, 110},
    {35, 244, 41},
    {214, 233, 248},
    {88, 152, 22},
    {173, 203, 42},
    {174, 158, 226},
    {91, 242, 36},
    {213, 194, 151},
    {32, 159, 223},
    {62, 62, 62},
    {1, 64, 92},
    {165, 115, 10},
    {82, 58, 17},
    {10, 87, 169},
    {231, 8, 154},
    {127, 1, 173},
    {87, 168, 204},
    {163, 181, 29},
    {5, 235, 122},
    {225, 207, 31},
    {219, 164, 117},
    {44, 157, 123},
    {143, 208, 137},
    {223, 223, 223},
    {106, 106, 106},
    {92, 52, 49},
    {209, 47, 237},
    {38, 102, 244},
    {119, 41, 163},
    {152, 57, 235},
    {61, 107, 107},
    {13, 186, 210},
    {237, 206, 142},
    {122, 146, 112},
    {128, 207, 25},
    {101, 179, 33},
    {242, 249, 74},
    {179, 179, 179},
    {150, 150, 150},
    {47, 14, 166},
    {137, 24, 157},
    {104, 72, 174},
    {51, 37, 23},
    {195, 76, 128},
    {65, 63, 15},
    {113, 239, 183},
    {156, 244, 24},
    {59, 172, 127},
    {251, 244, 106},
    {18, 239, 196},
    {182, 245, 254},
    {135, 135, 135},
    {194, 194, 194},
    {99, 117, 54},
    {247, 105, 151},
    {24, 40, 65},
    {78, 14, 212},
    {186, 51, 138},
    {29, 75, 99},
    {53, 227, 3},
    {191, 128, 116},
    {69, 210, 117},
    {189, 158, 172},
    {73, 215, 181},
    {200, 136, 244},
    {91, 91, 91},
    {238, 238, 238},
    {24, 41, 209},
    {179, 101, 35},
    {68, 92, 45},
    {176, 127, 6},
    {21, 28, 202},
    {250, 14, 133},
    {111, 113, 244},
    {75, 157, 9},
    {188, 243, 158},
    {26, 166, 25},
    {194, 181, 111},
    {55, 169, 191},
    {144, 240, 216},
    {47, 47, 47},
    {183, 88, 183},
    {77, 59, 207},
    {206, 79, 223},
    {58, 34, 39},
    {244, 115, 178},
    {96, 64, 46},
    {133, 115, 143},
    {34, 10, 102},
    {18, 218, 222},
    {253, 158, 88},
    {105, 183, 225},
    {159, 193, 4},
    {115, 165, 12},
    {228, 172, 60},
    {3, 3, 3},
    {197, 38, 177},
    {49, 111, 171},
    {148, 93, 253},
    {123, 29, 40},
    {139, 13, 184},
    {45, 5, 70},
    {216, 8, 226},
    {86, 86, 129},
    {255, 255, 255},
    {102, 134, 58},
    {131, 181, 155},
    {39, 153, 187},
    {236, 251, 149},
    {14, 142, 200},
    {170, 130, 102},
    {153, 82, 221},
    {117, 99, 31},
    {224, 1, 26},
    {5, 99, 34},
    {209, 74, 111},
    {95, 63, 207},
    {169, 23, 28},
    {12, 68, 155},
    {37, 37, 37},
    {40, 168, 96},
    {223, 193, 247},
    {83, 197, 96},
    {161, 254, 161},
    {84, 156, 214},
    {219, 237, 57},
    {234, 61, 128},
    {11, 30, 21},
    {172, 68, 114},
    {89, 23, 78},
    {166, 22, 139},
    {3, 75, 187},
    {231, 57, 65},
    {248, 248, 248},
    {81, 81, 81},
    {88, 247, 196},
    {173, 250, 97},
    {10, 130, 136},
    {213, 162, 69},
    {32, 192, 50},
    {135, 153, 85},
    {164, 19, 218},
    {82, 90, 193},
    {222, 126, 227},
    {231, 56, 208},
    {127, 97, 127},
    {147, 101, 165},
    {204, 204, 204},
    {125, 125, 125},
    {5, 140, 168},
    {225, 255, 85},
    {116, 252, 130},
    {143, 176, 91},
    {108, 228, 136},
    {249, 231, 95},
    {208, 79, 63},
    {38, 7, 38},
    {130, 10, 136},
    {152, 88, 63},
    {62, 12, 184},
    {202, 34, 114},
    {160, 160, 160},
    {169, 169, 169},
    {122, 241, 162},
    {149, 163, 176},
    {48, 240, 55},
    {242, 154, 152},
    {30, 222, 30},
    {184, 166, 246},
    {137, 119, 79},
    {104, 41, 124},
    {255, 33, 75},
    {195, 44, 83},
    {65, 95, 221},
    {180, 92, 120},
    {116, 116, 116},
    {213, 213, 213},
    {60, 204, 173},
    {207, 177, 174},
    {76, 211, 69},
    {18, 223, 142},
    {182, 150, 44},
    {66, 170, 114},
    {198, 216, 252},
    {247, 9, 69},
    {24, 24, 11},
    {189, 75, 141},
    {185, 82, 89},
    {29, 43, 177},
    {240, 80, 204},
    {72, 72, 72},
    {255, 255, 0},
    {69, 179, 167},
    {178, 154, 108},
    {22, 198, 91},
    {73, 230, 253},
    {200, 231, 38},
    {63, 129, 177},
    {154, 172, 144},
    {179, 5, 241},
    {68, 108, 102},
    {192, 32, 233},
    {250, 109, 88},
    {111, 17, 39},
    {140, 68, 244},
    {28, 28, 28},
    {187, 147, 76},
    {26, 198, 203},
    {246, 150, 215},
    {98, 154, 191},
    {55, 153, 243},
    {144, 144, 10},
    {125, 235, 119},
    {233, 193, 205},
    {206, 46, 50},
    {58, 18, 108},
    {157, 107, 133},
    {133, 20, 94},
    {34, 106, 180},
    {214, 86, 234},
    {253, 253, 137},
    {105, 215, 51},
    {136, 232, 209},
    {47, 224, 44},
    {115, 149, 64},
    {228, 204, 237},
    {2, 191, 20},
    {163, 235, 147},
    {12, 12, 12},
    {148, 60, 44},
    {123, 45, 99},
    {238, 81, 20},
    {216, 103, 50},
    {86, 54, 80},
    {162, 10, 14},
    {130, 212, 73},
    {39, 249, 105},
    {208, 144, 253},
    {93, 218, 186},
    {14, 190, 131},
    {170, 226, 179},
    {93, 202, 127},
    {207, 175, 112},
    {56, 56, 56},
    {224, 96, 200},
    {5, 83, 105},
    {161, 42, 128},
    {169, 119, 201},
    {12, 36, 78},
    {239, 113, 157},
    {222, 160, 37},
    {83, 165, 141},
    {164, 203, 26},
    {1, 175, 213},
    {84, 172, 157},
    {219, 141, 234},
    {48, 240, 233},
    {229, 229, 229},
    {100, 100, 100},
    {172, 36, 160},
    {89, 39, 6},
    {213, 118, 100},
    {231, 89, 147},
    {112, 80, 164},
    {157, 75, 11},
    {172, 154, 178},
    {10, 226, 90},
    {234, 225, 64},
    {32, 239, 121},
    {135, 249, 135},
    {98, 138, 122},
    {185, 185, 185},
    {144, 144, 144},
    {222, 30, 55},
    {42, 56, 252},
    {140, 100, 122},
    {147, 5, 119},
    {52, 92, 16},
    {193, 63, 103},
    {225, 159, 135},
    {116, 156, 80},
    {155, 141, 31},
    {108, 132, 91},
    {249, 135, 141},
    {23, 213, 158},
    {141, 141, 141},
    {188, 188, 188},
    {130, 106, 90},
    {103, 26, 166},
    {239, 64, 9},
    {202, 66, 160},
    {74, 35, 26},
    {188, 107, 3},
    {149, 195, 99},
    {48, 144, 227},
    {199, 248, 115},
    {30, 190, 204},
    {185, 199, 37},
    {76, 195, 128},
    {97, 97, 97},
    {232, 232, 232},
    {255, 65, 153},
    {20, 70, 67},
    {179, 76, 189},
    {65, 111, 150},
    {180, 60, 170},
    {19, 101, 204},
    {254, 1, 196},
    {207, 209, 125},
    {76, 227, 15},
    {185, 135, 121},
    {66, 202, 160},
    {199, 185, 47},
    {48, 208, 184},
    {53, 53, 53},
    {24, 120, 215},
    {189, 43, 95},
    {75, 3, 148},
    {201, 50, 183},
    {29, 27, 249},
    {240, 48, 30},
    {103, 58, 41},
    {129, 42, 6},
    {178, 250, 190},
    {22, 245, 17},
    {248, 199, 209},
    {63, 225, 99},
    {155, 205, 67},
    {116, 220, 12},
    {9, 9, 9},
    {68, 12, 179},
    {192, 127, 59},
    {53, 124, 158},
    {147, 117, 96},
    {111, 33, 108},
    {140, 37, 38},
    {41, 24, 115},
    {221, 94, 106},
    {246, 246, 10},
    {98, 170, 244},
    {135, 185, 219},
    {125, 139, 165},
    {233, 162, 29},
    {10, 162, 6},
    {58, 114, 189},
    {157, 12, 87},
    {112, 112, 42},
    {230, 41, 132},
    {34, 90, 254},
    {214, 55, 56},
    {89, 7, 139},
    {171, 100, 251},
    {31, 31, 31},
    {136, 136, 0},
    {47, 209, 103},
    {218, 205, 183},
    {2, 222, 193},
    {164, 140, 70},
    {83, 229, 209},
    {123, 77, 176},
    {238, 50, 192},
    {13, 100, 18},
    {169, 7, 221},
    {86, 6, 27},
    {162, 106, 219},
    {6, 115, 231},
    {254, 254, 254},
    {75, 75, 75},
    {208, 239, 45},
    {93, 234, 241},
    {169, 162, 232},
    {93, 170, 173},
    {208, 208, 162},
    {39, 185, 53},
    {5, 51, 186},
    {162, 74, 82},
    {86, 118, 12},
    {12, 20, 5},
    {239, 18, 79},
    {123, 13, 237},
    {210, 210, 210},
    {119, 119, 119},
    {164, 172, 200},
    {1, 159, 157},
    {227, 140, 178},
    {47, 145, 59},
    {136, 200, 92},
    {105, 151, 111},
    {89, 71, 214},
    {213, 23, 182},
    {34, 42, 231},
    {112, 48, 118},
    {157, 44, 216},
    {59, 50, 226},
    {166, 166, 166},
    {163, 163, 163},
    {234, 130, 146},
    {125, 203, 248},
    {144, 208, 87},
    {98, 233, 168},
    {245, 182, 86},
    {25, 134, 150},
    {41, 87, 47},
    {140, 5, 168},
    {111, 81, 123},
    {52, 60, 194},
    {193, 96, 180},
    {68, 76, 232},
    {122, 122, 122},
    {207, 207, 207},
    {155, 236, 205},
    {64, 161, 63},
    {199, 152, 49},
    {23, 181, 76},
    {178, 186, 225},
    {69, 242, 250},
    {103, 121, 117},
    {239, 112, 66},
    {30, 107, 236},
    {74, 66, 200},
    {189, 11, 209},
    {25, 56, 132},
    {78, 78, 78},
    {251, 251, 251},
    {199, 153, 161},
    {67, 138, 251},
    {181, 229, 59},
    {184, 246, 110},
    {76, 163, 82},
    {206, 145, 33},
    {59, 140, 240},
    {19, 37, 145},
    {180, 124, 246},
    {65, 31, 129},
    {19, 6, 32},
    {254, 97, 24},
    {104, 104, 32},
    {34, 34, 34},
    {76, 131, 219},
    {185, 230, 171},
    {31, 254, 144},
    {241, 233, 143},
    {198, 137, 100},
    {49, 176, 106},
    {149, 131, 63},
    {122, 177, 253},
    {74, 98, 70},
    {202, 2, 252},
    {61, 75, 229},
    {103, 89, 251},
    {130, 74, 212},
    {38, 70, 122},
    {255, 255, 255},
    {22, 149, 193},
    {248, 167, 3},
    {108, 196, 6},
    {143, 193, 76},
    {155, 252, 8},
    {116, 188, 221},
    {224, 223, 218},
    {4, 203, 243},
    {6, 6, 6},
    {52, 28, 76},
    {147, 69, 43},
    {127, 33, 36},
    {41, 119, 161},
    {222, 62, 184},
    {82, 26, 158},
    {98, 201, 38},
    {135, 217, 9},
    {32, 128, 110},
    {212, 210, 82},
    {233, 146, 87},
    {11, 194, 211},
    {173, 218, 239},
    {88, 183, 151},
    {50, 50, 50},
    {112, 16, 248},
    {230, 25, 207},
    {2, 11, 224},
    {89, 103, 89},
    {172, 4, 46},
    {11, 93, 73},
    {47, 176, 180},
    {218, 173, 101},
    {84, 219, 138},
    {161, 143, 182},
    {164, 188, 13},
    {84, 133, 3},
    {223, 224, 121},
    {235, 235, 235},
    {94, 94, 94},
    {13, 4, 192},
    {169, 55, 149},
    {94, 126, 140},
    {6, 19, 53},
    {223, 33, 148},
    {117, 35, 67},
    {93, 138, 35},
    {170, 194, 58},
    {15, 205, 148},
    {207, 223, 232},
    {40, 217, 230},
    {131, 149, 22},
    {191, 191, 191},
    {138, 138, 138},
    {86, 22, 222},
    {217, 40, 109},
    {44, 68, 27},
    {123, 109, 63},
    {148, 124, 112},
    {49, 47, 247},
    {1, 254, 79},
    {227, 236, 96},
    {115, 228, 87},
    {136, 168, 142},
    {105, 247, 188},
    {254, 190, 214},
    {147, 147, 147},
    {182, 182, 182},
    {35, 74, 59},
    {133, 84, 1},
    {96, 48, 57},
    {58, 82, 48},
    {205, 110, 110},
    {77, 123, 146},
    {125, 171, 43},
    {144, 176, 132},
    {55, 232, 226},
    {245, 214, 132},
    {25, 230, 69},
    {188, 211, 17},
    {103, 103, 103},
    {226, 226, 226},
    {111, 49, 169},
    {251, 46, 11},
    {20, 108, 220},
    {193, 111, 254},
    {68, 44, 58},
    {179, 69, 173},
    {23, 105, 140},
    {63, 193, 237},
    {200, 168, 122},
    {73, 151, 232},
    {177, 218, 49},
    {69, 147, 41},
    {192, 192, 40},
    {59, 59, 59},
    {239, 16, 143},
    {30, 11, 64},
    {186, 19, 4},
    {78, 126, 195},
    {188, 59, 154},
    {25, 88, 86},
    {246, 73, 25},
    {99, 53, 105},
    {66, 234, 46},
    {182, 214, 112},
    {17, 175, 153},
    {206, 241, 243},
    {59, 236, 35},
    {156, 180, 68},
    {15, 15, 15},
    {179, 28, 36},
    {65, 127, 83},
    {196, 108, 14},
    {51, 85, 0},
    {254, 81, 92},
    {104, 9, 241},
    {137, 56, 19},
    {46, 78, 249},
    {30, 158, 66},
    {241, 218, 196},
    {101, 243, 125},
    {149, 227, 237},
    {122, 210, 44},
    {238, 142, 210},
    {201, 98, 46},
    {61, 44, 55},
    {152, 25, 98},
    {119, 89, 180},
    {130, 122, 159},
    {39, 39, 167},
    {209, 15, 99},
    {92, 116, 108},
    {25, 25, 25},
    {108, 164, 212},
    {143, 240, 8},
    {43, 221, 40},
    {224, 191, 10},
    {4, 172, 38},
    {163, 245, 65},
    {146, 37, 248},
    {127, 65, 240},
    {232, 72, 198},
    {9, 39, 190},
    {221, 14, 242},
    {83, 122, 76},
    {165, 83, 135},
    {0, 255, 255},
    {69, 69, 69},
    {33, 223, 188},
    {213, 226, 26},
    {90, 178, 120},
    {173, 186, 61},
    {88, 216, 74},
    {215, 169, 165},
    {230, 121, 29},
    {2, 106, 50},
    {166, 102, 156},
    {81, 32, 206},
    {172, 52, 101},
    {12, 62, 155},
    {235, 125, 221},
    {216, 216, 216},
    {113, 113, 113},
    {84, 188, 88},
    {161, 191, 253},
    {6, 198, 21},
    {223, 129, 171},
    {40, 232, 60},
    {141, 187, 187},
    {168, 87, 71},
    {94, 31, 94},
    {210, 58, 120},
    {223, 64, 71},
    {117, 68, 145},
    {154, 18, 130},
    {172, 172, 172},
    {157, 157, 157},
    {15, 174, 70},
    {237, 187, 200},
    {120, 184, 31},
    {131, 244, 199},
    {102, 198, 102},
    {240, 238, 222},
    {217, 71, 191},
    {44, 37, 199},
    {139, 125, 175},
    {148, 29, 162},
    {50, 80, 37},
    {198, 102, 238},
    {128, 128, 128},
    {201, 201, 201},
    {115, 133, 133},
    {159, 129, 95},
    {56, 136, 161},
    {254, 221, 4},
    {18, 154, 130},
    {180, 226, 107},
    {133, 51, 211},
    {96, 0, 114},
    {245, 3, 165},
    {205, 15, 188},
    {77, 27, 65},
    {184, 24, 228},
    {84, 84, 84},
    {245, 245, 245},
    {56, 137, 50},
    {195, 245, 51},
    {70, 245, 171},
    {25, 214, 14},
    {188, 179, 194},
    {75, 221, 85},
    {202, 156, 97},
    {251, 77, 217},
    {20, 92, 150},
    {177, 15, 17},
    {178, 37, 127},
    {23, 9, 95},
    {248, 24, 16},
    {40, 40, 40},
    {199, 199, 167},
    {73, 246, 60},
    {190, 222, 239},
    {26, 130, 199},
    {69, 163, 98},
    {192, 160, 249},
    {53, 163, 95},
    {145, 217, 181},
    {186, 114, 214},
    {78, 78, 136},
    {204, 91, 117},
    {246, 41, 203},
    {99, 85, 187},
    {134, 102, 26},
    {255, 255, 255},
    {181, 181, 161},
    {17, 207, 75},
    {252, 180, 54},
    {107, 236, 152},
    {59, 220, 104},
    {156, 212, 150},
    {113, 175, 234},
    {229, 133, 82},
    {0, 0, 0},
    {196, 12, 220},
    {51, 101, 75},
    {151, 73, 107},
    {137, 87, 193},
    {46, 46, 40},
    {218, 18, 118},
    {241, 185, 22},
    {101, 147, 175},
    {128, 159, 14},
    {37, 195, 194},
    {122, 225, 103},
    {238, 238, 0},
    {14, 250, 143},
    {167, 167, 8},
    {44, 44, 44},
    {152, 120, 176},
    {119, 105, 255},
    {226, 22, 143},
    {209, 111, 177},
    {92, 20, 190},
    {171, 125, 41},
    {142, 144, 212},
    {43, 189, 244},
    {220, 211, 98},
    {81, 159, 38},
    {4, 156, 109},
    {163, 149, 147},
    {87, 232, 145},
    {241, 241, 241},
    {88, 88, 88},
    {232, 40, 21},
    {9, 23, 245},
    {173, 110, 29},
    {165, 51, 85},
    {0, 96, 210},
    {229, 83, 115},
    {212, 130, 202},
    {90, 210, 170},
    {174, 237, 243},
    {88, 231, 1},
    {215, 201, 119},
    {36, 181, 117},
    {197, 197, 197},
    {132, 132, 132},
    {166, 6, 78},
    {82, 48, 133},
    {219, 84, 138},
    {235, 29, 15},
    {124, 21, 56},
    {145, 15, 151},
    {160, 222, 47},
    {6, 166, 197},
    {227, 149, 103},
    {40, 136, 238},
    {141, 219, 105},
    {110, 205, 230},
    {153, 153, 153},
    {176, 176, 176},
    {210, 90, 170},
    {38, 115, 97},
    {128, 32, 230},
    {154, 114, 80},
    {62, 126, 253},
    {200, 55, 230},
    {237, 219, 27},
    {120, 216, 203},
    {151, 200, 131},
    {102, 166, 180},
    {240, 143, 14},
    {29, 243, 112},
    {109, 109, 109},
    {220, 220, 220},
    {139, 29, 125},
    {107, 94, 59},
    {251, 4, 149},
    {49, 127, 110},
    {198, 6, 60},
    {67, 85, 61},
    {182, 73, 236},
    {159, 225, 141},
    {56, 184, 234},
    {205, 218, 157},
    {18, 250, 81},
    {181, 131, 185},
    {64, 128, 29},
    {65, 65, 65},
    {96, 96, 159},
    {245, 99, 119},
    {27, 51, 100},
    {189, 110, 83},
    {77, 43, 10},
    {184, 120, 54},
    {31, 34, 81},
    {242, 69, 89},
    {195, 149, 225},
    {70, 198, 224},
    {176, 143, 249},
    {75, 189, 135},
    {203, 252, 179},
    {60, 148, 36},
    {21, 21, 21},
    {20, 60, 68},
    {177, 111, 195},
    {71, 71, 8},
    {194, 69, 144},
    {23, 57, 20},
    {248, 120, 193},
    {109, 28, 198},
    {141, 110, 153},
    {190, 190, 34},
    {26, 178, 140},
    {244, 131, 78},
    {53, 195, 141},
    {146, 186, 100},
    {126, 254, 226},
    {78, 46, 90},
    {204, 60, 167},
    {57, 57, 2},
    {159, 49, 251},
    {99, 101, 239},
    {134, 7, 199},
    {34, 31, 243},
    {211, 124, 131},
    {19, 19, 19},
    {252, 212, 228},
    {107, 221, 211},
    {139, 253, 72},
    {113, 207, 58},
    {229, 229, 128},
    {3, 213, 33},
    {51, 5, 152},
    {151, 42, 185},
    {120, 56, 245},
    {236, 11, 106},
    {46, 30, 99},
    {218, 114, 163},
    {85, 67, 24},
    {159, 32, 96},
    {63, 63, 63},
    {128, 191, 220},
    {37, 242, 137},
    {209, 186, 144},
    {14, 155, 93},
    {168, 200, 218},
    {95, 161, 77},
    {119, 9, 45},
    {226, 117, 93},
    {7, 70, 251},
    {161, 16, 94},
    {92, 36, 244},
    {171, 30, 250},
    {16, 81, 10},
    {222, 222, 222},
    {107, 107, 107},
    {220, 180, 176},
    {81, 175, 109},
    {165, 230, 116},
    {87, 137, 67},
    {215, 248, 172},
    {45, 155, 219},
    {9, 119, 39},
    {174, 15, 205},
    {90, 50, 144},
    {0, 80, 153},
    {229, 52, 160},
    {114, 122, 202},
    {178, 178, 178},
    {151, 151, 151},
    {174, 142, 39},
    {10, 151, 29},
    {231, 200, 47},
    {35, 212, 167},
    {132, 140, 191},
    {97, 158, 238},
    {81, 79, 87},
    {219, 53, 88},
    {43, 93, 207},
    {124, 116, 234},
    {145, 111, 69},
    {55, 118, 126},
    {134, 134, 134},
    {195, 195, 195},
    {227, 244, 181},
    {119, 233, 23},
    {152, 168, 193},
    {110, 173, 52},
    {249, 242, 201},
    {21, 194, 11},
    {37, 19, 179},
    {128, 64, 52},
    {101, 115, 149},
    {62, 31, 45},
    {201, 87, 53},
    {72, 8, 116},
    {90, 90, 90},
    {239, 239, 239},
    {151, 169, 81},
    {52, 229, 162},
    {195, 219, 173},
    {240, 191, 70},
    {29, 148, 162},
    {187, 205, 196},
    {79, 208, 21},
    {107, 61, 233},
    {251, 52, 222},
    {18, 47, 113},
    {67, 53, 239},
    {183, 41, 63},
    {16, 112, 88},
    {46, 46, 46},
    {56, 215, 56},
    {205, 187, 79},
    {79, 206, 96},
    {185, 162, 167},
    {180, 179, 241},
    {64, 223, 206},
    {196, 179, 206},
    {50, 249, 213},
    {26, 82, 182},
    {190, 94, 25},
    {73, 24, 2},
    {31, 65, 131},
    {242, 37, 139},
    {100, 44, 188},
    {2, 2, 2},
    {70, 165, 50},
    {177, 239, 43},
    {21, 220, 126},
    {250, 157, 168},
    {202, 205, 247},
    {61, 244, 245},
    {153, 199, 163},
    {118, 245, 98},
    {255, 255, 255},
    {70, 38, 218},
    {195, 117, 219},
    {55, 105, 12},
    {109, 123, 22},
    {142, 14, 72},
    {42, 2, 230},
    {26, 209, 94},
    {244, 227, 159},
    {96, 128, 154},
    {133, 226, 162},
    {146, 138, 47},
    {126, 158, 48},
    {233, 214, 91},
    {8, 135, 104},
    {38, 38, 38},
    {56, 88, 208},
    {159, 1, 183},
    {115, 101, 191},
    {33, 127, 33},
    {212, 28, 86},
    {91, 109, 185},
    {107, 188, 1},
    {139, 157, 149},
    {44, 195, 241},
    {216, 151, 206},
    {229, 213, 203},
    {4, 181, 242},
    {167, 248, 2},
    {247, 247, 247},
    {82, 82, 82},
    {120, 88, 37},
    {236, 59, 33},
    {14, 78, 124},
    {85, 35, 197},
    {160, 64, 178},
    {7, 25, 213},
    {37, 146, 91},
    {209, 218, 66},
    {94, 253, 100},
    {168, 247, 145},
    {96, 193, 159},
    {211, 165, 229},
    {203, 203, 203},
    {126, 126, 126},
    {7, 38, 47},
    {161, 64, 21},
    {84, 92, 98},
    {16, 49, 219},
    {235, 100, 8},
    {121, 103, 223},
    {81, 206, 190},
    {166, 134, 166},
    {3, 138, 8},
    {215, 152, 126},
    {46, 251, 9},
    {138, 225, 51},
    {159, 159, 159},
    {170, 170, 170},
    {90, 82, 66},
    {213, 99, 240},
    {32, 1, 134},
    {114, 26, 24},
    {158, 94, 158},
    {57, 39, 119},
    {9, 247, 207},
    {231, 168, 251},
    {127, 160, 203},
    {132, 236, 19},
    {97, 254, 61},
    {244, 156, 57},
    {115, 115, 115},
    {214, 214, 214},
    {44, 61, 30},
    {143, 113, 238},
    {108, 116, 165},
    {145, 95, 15},
    {54, 22, 172},
    {193, 43, 241},
    {71, 89, 125},
    {119, 137, 197},
    {152, 152, 138},
    {61, 202, 13},
    {249, 146, 25},
    {21, 163, 217},
    {176, 144, 140},
    {71, 71, 71},
    {0, 0, 255},
    {101, 19, 71},
    {242, 91, 44},
    {30, 78, 51},
    {200, 103, 126},
    {72, 104, 166},
    {191, 2, 49},
    {27, 45, 17},
    {51, 133, 113},
    {196, 235, 230},
    {65, 159, 105},
    {186, 173, 24},
    {79, 177, 199},
    {204, 132, 180},
    {27, 27, 27},
    {251, 84, 12},
    {18, 79, 163},
    {182, 87, 152},
    {66, 58, 95},
    {182, 25, 116},
    {17, 17, 138},
    {252, 107, 246},
    {106, 66, 78},
    {78, 174, 178},
    {186, 146, 236},
    {29, 235, 6},
    {196, 211, 30},
    {50, 154, 5},
    {150, 150, 170},
    {189, 62, 201},
    {73, 119, 210},
    {200, 41, 146},
    {63, 17, 156},
    {242, 21, 192},
    {100, 77, 110},
    {129, 63, 147},
    {36, 108, 20},
    {13, 13, 13},
    {20, 188, 172},
    {250, 173, 227},
    {111, 209, 147},
    {153, 167, 113},
    {118, 150, 176},
    {226, 202, 78},
    {194, 21, 9},
    {55, 10, 216},
    {144, 80, 190},
    {125, 123, 90},
    {142, 62, 3},
    {43, 98, 52},
    {221, 75, 255},
    {80, 48, 239},
    {57, 57, 57},
    {96, 224, 72},
    {133, 211, 233},
    {34, 170, 1},
    {233, 182, 137},
    {8, 231, 186},
    {175, 177, 221},
    {158, 97, 101},
    {115, 6, 109},
    {228, 12, 90},
    {2, 48, 63},
    {211, 44, 29},
    {92, 14, 107},
    {175, 113, 105},
    {228, 228, 228},
    {101, 101, 101},
    {45, 164, 33},
    {217, 167, 133},
    {86, 246, 228},
    {167, 153, 211},
    {80, 240, 68},
    {221, 139, 75},
    {236, 91, 242},
    {14, 47, 174},
    {170, 34, 0},
    {160, 112, 248},
    {8, 121, 7},
    {226, 10, 250},
    {184, 184, 184},
    {145, 145, 145},
    {94, 158, 182},
    {169, 183, 125},
    {12, 228, 250},
    {211, 196, 55},
    {36, 172, 160},
    {129, 255, 39},
    {161, 95, 199},
    {84, 61, 176},
    {219, 77, 95},
    {235, 5, 218},
    {121, 8, 13},
    {150, 86, 31},
    {140, 140, 140},
    {189, 189, 189},
    {3, 233, 218},
    {231, 153, 39},
    {112, 192, 137},
    {138, 130, 223},
    {106, 130, 249},
    {252, 170, 67},
    {213, 3, 35},
    {32, 96, 84},
    {135, 57, 51},
    {158, 62, 77},
    {57, 71, 165},
    {204, 68, 1},
    {96, 96, 96},
    {233, 233, 233},
    {127, 192, 26},
    {147, 197, 194},
    {52, 204, 61},
    {97, 207, 118},
    {244, 251, 233},
    {27, 237, 165},
    {190, 192, 132},
    {143, 17, 61},
    {108, 68, 238},
    {249, 71, 57},
    {193, 74, 33},
    {71, 57, 175},
    {176, 80, 56},
    {52, 52, 52},
    {151, 247, 88},
    {62, 171, 223},
    {202, 130, 20},
    {74, 178, 55},
    {21, 147, 146},
    {176, 239, 94},
    {71, 153, 200},
    {193, 233, 70},
    {242, 58, 254},
    {30, 126, 120},
    {184, 8, 145},
    {190, 97, 227},
    {27, 77, 195},
    {244, 92, 140},
    {8, 8, 8},
    {195, 139, 52},
    {65, 255, 187},
    {180, 252, 30},
    {19, 244, 223},
    {79, 129, 140},
    {204, 228, 102},
    {57, 231, 194},
    {157, 157, 42},
    {255, 255, 255},
    {182, 54, 74},
    {66, 10, 21},
    {198, 121, 155},
    {252, 12, 38},
    {106, 34, 156},
    {138, 34, 134},
    {185, 241, 62},
    {29, 139, 214},
    {240, 240, 169},
    {103, 169, 4},
    {50, 170, 79},
    {150, 246, 120},
    {121, 167, 107},
    {235, 163, 187},
    {32, 32, 32},
    {200, 72, 64},
    {63, 33, 215},
    {155, 14, 247},
    {129, 95, 65},
    {36, 12, 198},
    {211, 101, 81},
    {250, 204, 49},
    {111, 177, 65},
    {140, 227, 146},
    {41, 135, 94},
    {118, 166, 250},
    {226, 170, 156},
    {8, 216, 97},
    {253, 253, 253},
    {76, 76, 76},
    {144, 48, 108},
    {125, 75, 18},
    {233, 98, 168},
    {221, 43, 45},
    {80, 80, 34},
    {167, 57, 181},
    {132, 178, 59},
    {34, 202, 209},
    {214, 245, 140},
    {8, 215, 240},
    {175, 209, 15},
    {91, 173, 14},
    {209, 209, 209},
    {120, 120, 120},
    {228, 108, 136},
    {2, 32, 117},
    {163, 76, 242},
    {175, 17, 187},
    {8, 72, 219},
    {233, 23, 239},
    {216, 198, 87},
    {86, 150, 54},
    {162, 169, 104},
    {80, 144, 150},
    {221, 235, 153},
    {43, 193, 82},
    {165, 165, 165},
    {164, 164, 164},
    {170, 66, 210},
    {94, 107, 25},
    {207, 17, 23},
    {226, 106, 40},
    {118, 55, 213},
    {152, 7, 23},
    {169, 215, 175},
    {12, 132, 40},
    {239, 208, 250},
    {36, 204, 114},
    {129, 159, 244},
    {100, 235, 9},
    {121, 121, 121},
    {208, 208, 208},
    {219, 45, 141},
    {48, 82, 143},
    {136, 88, 113},
    {150, 54, 204},
    {50, 59, 98},
    {196, 115, 123},
    {231, 249, 245},
    {112, 239, 194},
    {157, 234, 109},
    {106, 226, 41},
    {252, 203, 145},
    {17, 176, 236},
    {77, 77, 77},
    {252, 252, 252},
    {135, 89, 225},
    {98, 43, 28},
    {245, 38, 123},
    {57, 119, 237},
    {204, 36, 210},
    {79, 18, 161},
    {186, 13, 113},
    {147, 165, 17},
    {52, 251, 118},
    {193, 159, 1},
    {27, 141, 119},
    {191, 161, 87},
    {72, 200, 192},
    {33, 33, 33},
    {108, 36, 60},
    {249, 39, 235},
    {23, 118, 247},
    {177, 42, 207},
    {71, 9, 227},
    {176, 49, 233},
    {21, 4, 191},
    {249, 50, 126},
    {202, 226, 198},
    {74, 130, 124},
    {188, 203, 101},
    {71, 249, 28},
    {194, 138, 148},
    {54, 182, 202},
    {255, 255, 255},
    {30, 30, 169},
    {185, 103, 67},
    {77, 100, 229},
    {206, 1, 12},
    {27, 125, 136},
    {244, 61, 94},
    {97, 95, 91},
    {131, 76, 116},
    {7, 7, 7},
    {180, 156, 204},
    {19, 197, 171},
    {254, 161, 163},
    {57, 135, 18},
    {158, 253, 248},
    {114, 186, 126},
    {66, 106, 197},
    {198, 26, 73},
    {48, 112, 222},
    {149, 19, 18},
    {106, 18, 214},
    {138, 67, 84},
    {46, 91, 111},
    {215, 56, 24},
    {51, 51, 51},
    {240, 144, 120},
    {103, 153, 79},
    {130, 138, 96},
    {121, 198, 185},
    {235, 196, 110},
    {15, 145, 189},
    {63, 65, 5},
    {155, 109, 37},
    {116, 124, 106},
    {224, 79, 245},
    {36, 60, 141},
    {211, 6, 131},
    {95, 97, 249},
    {234, 234, 234},
    {95, 95, 95},
    {140, 132, 64},
    {41, 183, 22},
    {221, 254, 13},
    {8, 184, 179},
    {160, 224, 211},
    {85, 131, 163},
    {125, 43, 194},
    {233, 3, 122},
    {11, 2, 96},
    {80, 96, 105},
    {167, 90, 103},
    {4, 21, 149},
    {190, 190, 190},
    {139, 139, 139},
    {214, 150, 94},
    {90, 167, 236},
    {171, 196, 154},
    {91, 204, 223},
    {211, 188, 48},
    {33, 223, 71},
    {1, 127, 167},
    {164, 45, 32},
    {83, 69, 183},
    {8, 40, 15},
    {233, 119, 61},
    {126, 62, 86},
    {146, 146, 146},
    {183, 183, 183},
    {162, 201, 186},
    {6, 211, 129},
    {223, 176, 185},
    {42, 161, 128},
    {142, 174, 46},
    {109, 218, 115},
    {93, 11, 203},
    {207, 112, 195},
    {39, 25, 83},
    {118, 86, 5},
    {153, 103, 197},
    {61, 84, 144},
    {102, 102, 102},
    {227, 227, 227},
    {239, 177, 42},
    {123, 173, 139},
    {148, 235, 93},
    {129, 175, 190},
    {100, 140, 218},
    {242, 133, 236},
    {31, 224, 228},
    {47, 49, 93},
    {136, 104, 58},
    {105, 55, 9},
    {50, 90, 176},
    {197, 19, 169},
    {64, 64, 168},
    {58, 58, 58},
    {112, 143, 16},
    {157, 139, 191},
    {59, 146, 132},
    {205, 253, 67},
    {252, 250, 217},
    {17, 207, 62},
    {183, 137, 89},
    {67, 148, 136},
    {98, 74, 206},
    {245, 22, 48},
    {25, 39, 241},
    {79, 113, 115},
    {187, 109, 163},
    {28, 52, 196},
    {14, 14, 14},
    {52, 155, 163},
    {193, 254, 211},
    {69, 236, 142},
    {178, 212, 128},
    {190, 145, 28},
    {73, 168, 18},
    {200, 247, 83},
    {62, 189, 74},
    {22, 22, 42},
    {178, 26, 132},
    {69, 83, 157},
    {21, 99, 109},
    {249, 82, 172},
    {110, 14, 82},
    {74, 225, 173},
    {189, 171, 183},
    {25, 152, 225},
    {246, 216, 52},
    {193, 186, 222},
    {55, 214, 24},
    {145, 206, 36},
    {124, 211, 139},
    {26, 26, 26},
    {76, 4, 52},
    {207, 49, 71},
    {59, 45, 151},
    {97, 63, 137},
    {132, 44, 166},
    {35, 117, 193},
    {19, 164, 121},
    {254, 193, 113},
    {104, 199, 70},
    {137, 167, 62},
    {158, 206, 179},
    {114, 218, 171},
    {229, 147, 199},
    {255, 0, 255},
    {70, 70, 70},
    {49, 16, 13},
    {149, 35, 89},
    {122, 19, 152},
    {45, 59, 189},
    {216, 88, 202},
    {87, 41, 37},
    {103, 248, 156},
    {130, 234, 178},
    {38, 229, 28},
    {209, 159, 79},
    {235, 243, 37},
    {16, 241, 111},
    {171, 189, 157},
    {215, 215, 215},
    {114, 114, 114},
    {116, 28, 184},
    {224, 127, 189},
    {4, 108, 146},
    {95, 1, 43},
    {168, 104, 188},
    {13, 59, 59},
    {41, 214, 198},
    {221, 158, 221},
    {82, 185, 247},
    {159, 128, 7},
    {86, 227, 113},
    {218, 209, 194},
    {171, 171, 171},
    {158, 158, 158},
    {11, 98, 178},
    {173, 123, 137},
    {88, 25, 254},
    {4, 117, 71},
    {229, 70, 229},
    {113, 111, 95},
    {89, 199, 63},
    {172, 164, 72},
    {12, 252, 47},
    {211, 220, 226},
    {34, 191, 149},
    {134, 166, 174},
    {127, 127, 127},
    {202, 202, 202},
    {83, 37, 101},
    {223, 66, 31},
    {40, 120, 17},
    {126, 94, 132},
    {146, 27, 2},
    {53, 99, 234},
    {5, 179, 83},
    {223, 128, 242},
    {117, 131, 37},
    {142, 206, 252},
    {109, 187, 161},
    {248, 215, 164},
    {83, 83, 83},
    {246, 246, 246},
    {40, 121, 129},
    {131, 54, 115},
    {102, 86, 75},
    {153, 87, 142},
    {60, 52, 66},
    {202, 93, 212},
    {75, 29, 224},
    {123, 205, 89},
    {148, 220, 23},
    {49, 143, 145},
    {242, 229, 64},
    {31, 129, 55},
    {184, 216, 80},
    {39, 39, 39},
    {135, 8, 231},
    {105, 87, 219},
    {254, 31, 175},
    {18, 10, 175},
    {196, 35, 225},
    {65, 33, 122},
    {181, 36, 222},
    {18, 90, 54},
    {58, 242, 86},
    {206, 206, 9},
    {77, 219, 245},
    {182, 233, 139},
    {67, 244, 91},
    {198, 166, 90},
    {255, 255, 255},
    {245, 118, 225},
    {25, 71, 35},
    {188, 116, 118},
    {75, 77, 120},
    {186, 93, 231},
    {29, 85, 22},
    {240, 48, 107},
    {102, 6, 209},
    {1, 1, 1},
    {68, 140, 92},
    {179, 228, 203},
    {23, 201, 235},
    {200, 151, 129},
    {62, 222, 152},
    {154, 210, 54},
    {177, 122, 86},
    {69, 52, 79},
    {192, 96, 78},
    {53, 51, 114},
    {249, 98, 230},
    {110, 110, 128},
    {141, 123, 16},
    {40, 40, 135},
    {45, 45, 45},
    {24, 248, 48},
    {246, 232, 127},
    {99, 149, 16},
    {145, 174, 240},
    {124, 180, 94},
    {235, 189, 105},
    {206, 81, 148},
    {59, 77, 69},
    {156, 20, 34},
    {113, 63, 198},
    {132, 28, 236},
    {36, 22, 19},
    {215, 105, 18},
    {240, 240, 240},
    {89, 89, 89},
    {104, 168, 148},
    {137, 151, 117},
    {46, 238, 156},
    {229, 242, 21},
    {0, 192, 180},
    {165, 147, 51},
    {148, 67, 138},
    {122, 114, 74},
    {238, 46, 180},
    {215, 104, 129},
    {88, 74, 246},
    {163, 53, 245},
    {196, 196, 196},
    {133, 133, 133},
    {39, 134, 206},
    {209, 175, 5},
    {92, 212, 11},
    {171, 220, 79},
    {92, 180, 215},
    {209, 207, 215},
    {224, 31, 111},
    {4, 13, 64},
    {163, 85, 39},
    {167, 9, 110},
    {14, 92, 232},
    {238, 78, 102},
    {152, 152, 152},
    {177, 177, 177},
    {82, 217, 43},
    {165, 243, 224},
    {0, 160, 102},
    {218, 177, 16},
    {46, 142, 78},
    {136, 246, 166},
    {173, 27, 91},
    {88, 120, 44},
    {215, 9, 195},
    {229, 38, 53},
    {113, 15, 141},
    {156, 116, 240},
    {108, 108, 108},
    {221, 221, 221},
    {12, 157, 253},
    {235, 221, 186},
    {124, 132, 21},
    {34, 143, 222},
    {134, 197, 124},
    {99, 245, 220},
    {246, 136, 172},
    {223, 33, 205},
    {40, 72, 90},
    {141, 27, 221},
    {146, 122, 208},
    {53, 3, 57},
    {192, 0, 156},
    {64, 64, 64},
    {223, 223, 32},
    {117, 226, 247},
    {154, 178, 227},
    {62, 237, 210},
    {109, 139, 233},
    {248, 183, 118},
    {23, 169, 57},
    {178, 132, 25},
    {131, 85, 161},
    {102, 102, 1},
    {240, 79, 185},
    {202, 61, 8},
    {75, 125, 51},
    {188, 20, 164},
    {20, 20, 20},
    {147, 187, 195},
    {50, 238, 67},
    {198, 198, 136},
    {67, 197, 16},
    {31, 177, 124},
    {184, 184, 130},
    {77, 187, 39},
    {205, 173, 217},
    {254, 126, 98},
    {18, 58, 228},
    {180, 67, 14}
};

QString moncal_get_QTDIR()
{
    // qDebug("get_home_dir");
    const char* qtdir= "QTDIR";
    return (QString) getenv(qtdir);
}

/* Compute approximate technical gamma from black/50% grey/white readings, */
/* (assumes an input offset gamma curve shape) */
static double tech_gamma(double bY, double gY, double wY)
{
    double grat, gioff, gvv, brat, gamma = 2.2;

    grat = gY/wY;
    brat = bY/wY;

    qDebug("~1 grat = %f, brat = %f\n",grat,brat);
    /* Itterative solution */
    for (int i = 0; i < 5; i++)
    {
        gioff = pow(brat, 1.0/gamma);
        gvv = 0.5 + (1.0 - 0.5) * gioff;
        gamma = log(grat) / log(gvv);
        qDebug("~1 gioff %f, gvv %f, gamma %f\n",gioff,gvv,gamma);
    }
    return gamma;
}

/* Compute approximate popular gamma from black/50% grey/white readings, */
/* (assumes a zero based gamma curve shape) */
static double pop_gamma(double bY, double gY, double wY)
{
    int i;
    double grat, brat, gioff, gvv, gamma = 2.2;

    grat = gY/wY;
    brat = bY/wY;

    gamma = log(grat) / log(0.5);
    return gamma;
}


/* Given an XYZ, compute the color temperature and the delta E to the locus */
static double comp_ct(
        double *de,     /* If non-NULL, return CIEDE2000 to locus */
        double lxyz[3], /* If non-NULL, return normalised XYZ on locus */
        int plank,      /* NZ if Plankian locus, 0 if Daylight locus */
        int dovct,      /* NZ if visual match, 0 if traditional correlation */
        double xyz[3]   /* Color to match */ )
{
    double ct_xyz[3]; /* XYZ on locus */
    double nxyz[3];   /* Normalised input color */
    double ct, ctde;  /* Color temperature & delta E to Black Body locus */
    icmXYZNumber wN;

    if ((ct = icx_XYZ2ill_ct(ct_xyz, plank != 0 ? icxIT_Ptemp : icxIT_Dtemp,
                                icxOT_CIE_1931_2, NULL, xyz, NULL, dovct)) < 0)
        error ("Got bad color temperature conversion\n");

    if (de != NULL)
    {
        icmAry2XYZ(wN, ct_xyz);
        icmAry2Ary(nxyz, xyz);
        nxyz[0] /= xyz[1];
        nxyz[2] /= xyz[1];
        nxyz[1] /= xyz[1];
        ctde = icmXYZCIE2K(&wN, nxyz, ct_xyz);
        *de = ctde;
    }
    if (lxyz != NULL)
    {
        icmAry2Ary(lxyz, ct_xyz);
    }
    return ct;
}


bool WhitePointFromTemp(int tempK, LPcmsCIEXYZ XYZ)
{
    cmsCIExyY WhitePoint;

    if (cmsWhitePointFromTemp(tempK, &WhitePoint))
    {
        cmsxyY2XYZ(XYZ, &WhitePoint);
        return true;
    }
    return false;
}

bool validateDevice()
{
    int score[instArraySize], winner = 0, high = 0;
    bool spyd2OK, dummy;
    char instName[64];
    
    fillListedInsts(listedInst, spyd2OK, dummy, emission);
    // qDebug("validateDevice(): listedInst[0].listSize = %i", listedInst[0].listSize);
    if (listedInst[0].listSize == 0)
        return FALSE;
    for (int i=0; i<instArraySize; i++)
        score[i]=0;
    for (int i=0; i<listedInst[0].listSize; i++)
    {
        score[i]=0;
        if (listedInst[i].listSize == cd.data.selectedInst.listSize)
        {
            score[i]++;
        }        
        if (listedInst[i].Inst == cd.data.selectedInst.Inst)
        {
            score[i]++;
        }
        else
        {
            // FIXME for some reason when more than one device is connected
            // the instrument type for the second device is not correct.
            // It show up in the section above.
            cd.data.selectedInst.Inst = listedInst[i].Inst;
        }
        if (listedInst[i].position == cd.data.selectedInst.position)
        {
            score[i]++;
        }
        if (listedInst[i].dupType == cd.data.selectedInst.dupType)
        {
            score[i]++;
        }
        if (listedInst[i].name == cd.data.selectedInst.name)
        {
            score[i]++;
        }
        if (listedInst[i].port == cd.data.selectedInst.port)
        {
            score[i]++;
        }
        // if (listedInst[i].dupType)
        //    score[i]--;
        if (i>0)
        {
            if(score[i] > high)
            {
                high = score[i];
                winner = i;
            }
        }
        else
        {
          high = score[i];
          winner = i;
        }
    }
    if (score[winner] < 5) // not a perfect match need to make sure if is OK
    {
        // not sure what needs to happen if it is not a perfect match
        qDebug("Instrument not a good match score = %i", high);
        return FALSE;
        // return TRUE;
    }
    // cd.data.selectedInst.port = listedInst[cd.data.selectedInst.position].port;
    strcpy(instName, cd.data.selectedInst.name.local8Bit());
    cd.data.selectedInst.Inst = inst_enum(instName);
    return TRUE;
}

bool MonitorCal::initInstrument()
{
    QString temp = "";
    comport=1;
    verb = 0;
    int mydebug=0;
    const bool OK = true;
    const bool notOK = false;
    inst_code ev;

    if (!instInUse && validateDevice())
    {
        instInUse = TRUE;
        // qDebug("initIntrument(): call new_inst()");
        qDebug("initIntrument(): cd.data.selectedInst.position + 1 = %i", cd.data.selectedInst.position + 1);
        qDebug("cd.data.selectedInst.Inst = %i", cd.data.selectedInst.Inst);
        if ((it = new_inst(cd.data.selectedInst.position + 1, cd.data.selectedInst.Inst, mydebug, verb)) == NULL)
        {
            qDebug("initIntrument(): failed new_inst()");
            messageText -> insertParagraph(tr((QString)"Unknown, inappropriate or no instrument detected"), -1);
            QMessageBox mb( QTranslator::tr("Failed to Initialize the Device"),
                            "Failed to initialize device.  Call to new_inst failed.",
                             // (QString) it->inst_interp_error(it, rv) + " "
                            //  + (QString) it->interp_error(it, rv),
                             QMessageBox::Critical,
                             QMessageBox::Ok | QMessageBox::Default,
                             QMessageBox::NoButton,  QMessageBox::NoButton);
 
            mb.exec();
            return notOK;
        }
        // qDebug ("about to call init_coms");
        if (setjmp(ioErrorJumpBuffer))
        {
            // There was a problem in the call to init_coms below
            // It was trapped and we need to bypass the call
            // after issuing an error message fo the user.
            QString message;
            message = (QString) getErrorMessage(); 
            QMessageBox mb( QTranslator::tr("Problem Initializing the device port."),
                            message + QTranslator::tr("\n\nThis error indicates some type of problem with accessing the device.\nA likely cause is not having access rights to the device.\nCheck your permissions to make sure you have USB access and access rights to this device."),
                            QMessageBox::Critical,
                            QMessageBox::Ok | QMessageBox::Default,
                            QMessageBox::NoButton,  QMessageBox::NoButton);

            mb.exec();
            return notOK;
        }
        
        /* Establish communications */
        // qDebug("initIntrument(): call init_coms()");
        if ((rv = it->init_coms(it, cd.data.selectedInst.position + 1, br, 15.0)) != inst_ok)
        {
            // qDebug("initIntrument(): failed init_coms()");
            messageText -> insertParagraph(tr("Failed to initialize communications with instrument or wrong instrument or bad configuration! (")
                    + (QString) it->inst_interp_error(it, rv) + " + "
                    + (QString) it->inst_interp_error(it, rv) + ")", -1);
            QMessageBox mb( QTranslator::tr("Failed to Initialize the Device"),
                            tr("Failed to initialize communications with instrument or wrong instrument or bad configuration! (")
                                    + (QString) it->inst_interp_error(it, rv) + ")",
                            QMessageBox::Critical,
                            QMessageBox::Ok | QMessageBox::Default,
                            QMessageBox::NoButton,  QMessageBox::NoButton);
 
            mb.exec();
            return notOK;
        }
        else
        {
            // qDebug ("initIntrument(): init_coms OK call set_opt_mode");
            /* set filter configuration before initializing/calibrating */
            if (fe != inst_opt_filter_unknown)
                if ((rv = it->set_opt_mode(it, inst_opt_set_filter, fe)) != inst_ok)
                {
                    // qDebug ("initIntrument(): failed set_opt_mode");
                    messageText -> insertParagraph(tr("Setting filter configuration not supported by instrument"), -1);
                    QMessageBox mb( QTranslator::tr("Failed to Initialize the Device"),
                                    tr("Setting filter configuration not supported by instrument"),
                                               QMessageBox::Critical,
                                               QMessageBox::Ok | QMessageBox::Default,
                                               QMessageBox::NoButton,  QMessageBox::NoButton);
 
                    mb.exec();
                    return notOK;
                }
            // qDebug ("Initialise the instrument");
            /* Initialise the instrument */
            if ((rv = it->init_inst(it)) != inst_ok)
            {
                
                messageText -> insertParagraph(tr("init_inst returned ") +
                        (QString) it->inst_interp_error(it, rv)
                        + "("  + (QString) it->interp_error(it, rv) + ")"
                        + tr(" Instrument initialization failed!"), -1);
                
                QMessageBox mb( QTranslator::tr("Problem Initializing Instrument."),
                                (QString) it->inst_interp_error(it, rv)
                                + "("  + (QString) it->interp_error(it, rv) + ")"
                                + tr(" Instrument initialization failed!"),
                                QMessageBox::Critical,
                                QMessageBox::Ok | QMessageBox::Default,
                                QMessageBox::NoButton,  QMessageBox::NoButton);

                mb.exec();
                return notOK;
            }
            else  /* it initialized OK so configure the instrument mode */
            {
                // qDebug ("it initialized OK so configure the instrument mode");
                inst_mode mode = inst_mode_unknown;

                cap = it->capabilities(it);
                cap2 = it->capabilities2(it);

                /* Don't fail if the instrument has only one mode, and */
                /* it's not been selected */
                if (!trans && !emiss && (cap & inst_reflection) == 0)
                {
                    if ((cap & inst_transmission) != 0 && (cap & inst_emission) == 0)
                        trans = true;
                    if ((cap & inst_transmission) == 0 && (cap & inst_emission) != 0)
                        emiss = true;
                }

                // qDebug ("it initialized OK so configure the instrument mode 1");
                if (trans)
                {
                    if ((cap & inst_trans_spot) == 0)
                    {
                        messageText -> insertParagraph (tr("Need transmission spot capability, and instrument doesn't support it"), -1);
                        QMessageBox mb( QTranslator::tr("Problem Initializing Instrument."),
                                        tr("Need transmission spot capability, and instrument doesn't support it"),
                                                QMessageBox::Critical,
                                                QMessageBox::Ok | QMessageBox::Default,
                                                QMessageBox::NoButton,  QMessageBox::NoButton);

                        mb.exec();
                        return notOK;
                    }
                }
                // qDebug ("it initialized OK so configure the instrument mode 3");
                if ((cap & inst_emis_spot) == 0)
                {
                    messageText -> insertParagraph (tr("Need emissive spot capability and instrument doesn't support it"), -1);
                    QMessageBox mb( QTranslator::tr("Problem Initializing Instrument."),
                                    tr("Need emissive spot capability and instrument doesn't support it"),
                                       QMessageBox::Critical,
                                       QMessageBox::Ok | QMessageBox::Default,
                                       QMessageBox::NoButton,  QMessageBox::NoButton);

                    mb.exec();
                    return notOK;
                }
                // Set to emmsision mode to read display
                if ((rv = it->set_mode(it, inst_mode_emis_disp)) != inst_ok)
                {
                    messageText -> insertParagraph(tr("Failed to set emission display mode. ") + (QString) it->inst_interp_error(it, rv) + " " + (QString) it->interp_error(it, rv), -1);
                    QMessageBox mb( QTranslator::tr("Problem Initializing Instrument."),
                                    tr("Failed to set emission display mode. ") + (QString) it->inst_interp_error(it, rv) + " " + (QString) it->interp_error(it, rv),
                                       QMessageBox::Critical,
                                       QMessageBox::Ok | QMessageBox::Default,
                                       QMessageBox::NoButton,  QMessageBox::NoButton);

                    mb.exec();
                    return notOK;
                }
                inst_opt_mode om;

                if (dtype == 1)
                    om = inst_opt_disp_crt;
                else
                    om = inst_opt_disp_lcd;
                // qDebug ("it initialized OK so configure the instrument mode 5");
                if ((rv = it->set_opt_mode(it,om)) != inst_ok)
                {
                    // qDebug ("it initialized OK so configure the instrument mode 5.1");
                    if (dtype==1)
                        temp = tr("CRT");
                    else
                        temp = tr("LCD");
                    // qDebug ("it initialized OK so configure the instrument mode 5.2");
                    messageText -> insertParagraph(tr("Setting display mode ") + temp + tr(" not supported by this instrument"), -1);
                    // qDebug ("it initialized OK so configure the instrument mode 5.3");
                    // qDebug ("it initialized OK so configure the instrument mode 5.4");
                    return notOK;
                }
                // qDebug ("it initialized OK so configure the instrument mode 7");
                rv = it->set_opt_mode(it, inst_opt_trig_prog);
                // qDebug ("it initialized OK so configure the instrument mode 11");
                if (rv != inst_ok)
                {
                    messageText -> insertParagraph(tr("Setting trigger mode failed with error : ")
                            + (QString) it->inst_interp_error(it, rv) + " "
                            + (QString) it->interp_error(it, rv), -1);
                    QMessageBox mb( QTranslator::tr("Problem Initializing Instrument."),
                                    tr("Setting trigger mode failed with error : ")
                                            + (QString) it->inst_interp_error(it, rv) + " "
                                            + (QString) it->interp_error(it, rv),
                                       QMessageBox::Critical,
                                       QMessageBox::Ok | QMessageBox::Default,
                                       QMessageBox::NoButton,  QMessageBox::NoButton);

                    mb.exec();
                    return notOK;
                }
                // run calibration to make sure device is calibrated 
                // before starting any measurements                 
                {
                    char id[200];
                    inst_cal_cond calc = inst_calc_disp_white;
                    
                    paintPatch(255, 255, 255);
                    it -> calibrate(it, inst_calt_all, &calc, id);
                    if (dtype==1)
                    {
                        // if this is a CRT do a freq calibration                        
                        paintPatch(255, 255, 255);
                        it -> calibrate(it, inst_calt_crt_freq, &calc, id);
                    }
                }
            }
        }
        // qDebug ("done setting up instrument");
        return OK;
    }
    // qDebug("last return notOK");
    return notOK;
}

cmsCIEXYZ MonitorCal::takeSpotReading()
{
    inst_code ev;
    cmsCIEXYZ XYZ;
    QString temp;
    ipatch val;
    double instXYZ[3];
    int dofwa = 0;          /* Setup for FWA compensation */
    int fidx = -1;          /* FWA compensated index, default = none */

    instXYZ[0] = -9999999.0;    // zero instXYZ - if there no valid
    instXYZ[1] = -9999999.0;    // measurement then this is what will
    instXYZ[2] = -9999999.0;    // be returned.  So XYZ.Z < -9999.0 is an error
    
    flashLEDsThread flashLEDs;
    rv = it->read_sample(it, "SPOT", &val);
    if ((it->capabilities2(it) & inst2_has_leds) != 0)
        flashLEDs.start();
    if (!rv == inst_ok)
    {
        messageText -> insertParagraph (tr("read_sample returned '")
            + (QString) it->inst_interp_error(it, rv) + "' ("
            + (QString) it->interp_error(it, rv) + ")", -1);


        if ((rv & inst_mask) == inst_needs_cal)
        {
            messageText -> insertParagraph (tr((QString)"Spot read failed because instruments needs calibration."), -1);
            // ev = handle_calibrate(it, inst_calt_all, inst_calc_none);
            if (ev != inst_ok)
            {
                messageText -> insertParagraph (tr((QString)"Instrument calibration failed."), -1);
            }
        // Deal with a misread
        }
        else if ((rv & inst_mask) == inst_misread)
        {
            messageText -> insertParagraph (tr((QString)"Spot read failed due to misread"), -1);

            /* Deal with a communications error */
        }
        else if ((rv & inst_mask) == inst_coms_fail)
        {

            // FIXME should only happen for serial devices
            // and not USB devices
            // Also really does nto do anything since this just drops through
            // and return control to the caller
//             int tt = it->last_comerr(it);
//
//             if (tt & (ICOM_BRK | ICOM_FER | ICOM_PER | ICOM_OER)) {
//             if (br == baud_57600) br = baud_38400;
//             else if (br == baud_38400) br = baud_9600;
//             else if (br == baud_9600) br = baud_4800;
//             else if (br == baud_2400) br = baud_1200;
//             else br = baud_1200;
//                             }

            // Issue error message
            /* Communication problem */
            messageText -> insertParagraph (tr((QString)"Spot read failed due to communication problem."), -1);

//             if ((rv = it->init_coms(it, comport, br, 15.0)) != inst_ok) {
//                 messageText -> insertParagraph (tr("init_coms returned '")
//                         + (QString) it->inst_interp_error(it, rv)
//                         + "' (" + (QString) it->interp_error(it, rv) + ")", -1);
//           }
        }

        // Some other fatal error
        else
        {
           messageText -> insertParagraph(tr("Got fatal error '")
                    + (QString) it->inst_interp_error(it, rv)
                    + "' (" + (QString) it->interp_error(it, rv) + ")", -1);
        }
    }
    /* We got a valid spot reading */
    else if (val.sp.spec_n <= 0)
    {
        if (val.XYZ_v == 0 && val.aXYZ_v == 0)
            messageText -> insertParagraph(tr((QString)"Instrument didn't return XYZ value"), -1);

        if (val.XYZ_v != 0)
        {
            for (int j = 0; j < 3; j++)
                instXYZ[j] = val.XYZ[j];
        }
        else
        {
            for (int j = 0; j < 3; j++)
                instXYZ[j] = val.aXYZ[j];
        }
    }
    else
    {
        // Compute XYZ from spectral data

        if (val.sp.spec_n <= 0) {
           messageText -> insertParagraph(tr((QString)"Instrument didn't return spectral data"), -1);
        }

        sp = val.sp;    // Transfer from vals to sp structure
        sp.norm  = 100.0;

        if (dofwa)
        {    // Setup FWA compensation
            xspect insp;     // Instrument illuminant

            if (inst_illuminant(&insp, itype) != 0)
                messageText -> insertParagraph(tr((QString)"Instrument doesn't have an FWA illuminant"), -1);

            /* Creat the base conversion object */
            if (sp2cief[fidx] == NULL)
            {
                if ((sp2cief[fidx] = new_xsp2cie(illum, &cust_illum, observ,
                                                    NULL, icSigXYZData)) == NULL)
                    messageText -> insertParagraph(tr((QString)"Creation of spectral conversion object failed"), -1);
            }

            if (sp2cief[fidx]->set_fwa(sp2cief[fidx], &insp, &sp))
                    messageText -> insertParagraph(tr((QString)"Set FWA on sp2cie failed"), -1);

        }
        else
        {
            if (fidx < 0 || sp2cief[fidx] == NULL)
            {
                /* Convert it to XYZ space using uncompensated */
                sp2cie->convert(sp2cie, instXYZ, &sp);
            }
            else
            {
                /* Convert using compensated conversion */
                sp2cief[fidx]->convert(sp2cief[fidx], instXYZ, &sp);
            }
        }

        instXYZ[0] *= 100.0;
        instXYZ[1] *= 100.0;
        instXYZ[2] *= 100.0;
    }

    XYZ.X = instXYZ[0]/100.0;
    XYZ.Y = instXYZ[1]/100.0;
    XYZ.Z = instXYZ[2]/100.0;
    if ((it->capabilities2(it) & inst2_has_leds) != 0)
        flashLEDs.wait();
    return XYZ;
}


void MonitorCal::paintPatch(int R, int G, int B)
{
    QPixmap pix;
    QColor color;

    pix.resize(pixmapColor -> width() - 6, pixmapColor -> height() -6);
    color.setRgb(R, G, B);
    pix.fill(color);
    pixmapColor -> setPixmap (pix);
    pixmapColor -> repaint();
    lprofApp -> processEvents();
}

void xyY2XYZ(LPcmsCIEXYZ Dest, LPcmsCIExyY Source)
{

    Dest -> X = Source -> x * (Source -> Y/Source -> y);
    Dest -> Y = Source -> Y;
    Dest -> Z = (1 - Source -> x - Source -> y) * (Source -> Y/Source -> y);

}

RGBtype XYZ2RGB(cmsCIEXYZ XYZ)
{
    RGBtype RGB;

    // What aobut native white point???
    if (whitePoint > 5001.0)  // D65 use a D65 matrix
    {
        //  XYZ to sRGB inverse matrix
        //  from Bruce Lindbloom web site
        //  R = |3.24071   -0.969258   0.0556352|   |X|
        //  G = |-1.53726   1.87599   -0.203996 | * |Y|
        //  B = |-0.498571  0.0415557  1.05707  |   |Z|
        // RGB values will be linear and scaled between 0 and 1.0
        // RGB values are not gamma corrected but are otherwise sRGB like
        // These are used to scale the display RGB indicators
        // while setting the display white and black points.

        RGB.R = 3.24071 * XYZ.X + -1.53726 * XYZ.Y + -0.498571 * XYZ.Z;
        RGB.G = -0.969258 * XYZ.X + 1.87599 * XYZ.Y + 0.0415557 * XYZ.Z;
        RGB.B = 0.0556352 * XYZ.X + -0.203996 * XYZ.Y + 1.05707 * XYZ.Z;
        // AdobeRGB matrix
        // RGB.R = 2.04148 * XYZ.X + -0.564977* XYZ.Y + -0.344713 * XYZ.Z;
        // RGB.G = -0.969258 * XYZ.X + 1.87599 * XYZ.Y + 0.0415557 * XYZ.Z;
        // RGB.B = 0.0134455 * XYZ.X + -0.118373 * XYZ.Y + 1.01527 * XYZ.Z;
    }
    else // D50 use a D50 Matrix
    {
        // BetaRGB matrix
        RGB.R = 1.75526 * XYZ.X + -0.483679 * XYZ.Y + -0.253000 * XYZ.Z;
        RGB.G = -0.544134 * XYZ.X + 1.50688 * XYZ.Y + 0.0215528 * XYZ.Z;
        RGB.B = 0.00634681 * XYZ.X + -0.0175762 * XYZ.Y + 1.22570 * XYZ.Z;
    }
    return RGB;
}

void MonitorCal::setDisplayWP()
{
    RGBtype targetRGB, measuredRGB;
    cmsCIEXYZ measuredXYZ, targetXYZ;
    bool nativeWP = false;
    double Rdir, Gdir, Bdir;
    int progress = 1;

//    buttonStop -> setEnabled(TRUE);
    stop = false;
    paintPatch(255, 255, 255);

    // setup target whitepoint settings
    if (whitePoint > 0.00000001)
    {               /* Daylight color temperature */
        double XYZ[3];
        if (icx_ill_sp2XYZ(XYZ, icxOT_default, NULL, icxIT_Dtemp, whitePoint, NULL) != 0)
            qDebug ("Failed to compute XYZ of target color temperature %f", whitePoint);
        targetXYZ.X=XYZ[0];
        targetXYZ.Y=XYZ[1];
        targetXYZ.Z=XYZ[2];
     }
     else
     {   /* Native white */
         targetXYZ = takeSpotReading();
         nativeWP = true;
     }

     cmsCIEXYZ tXYZ;

     tXYZ.X = targetXYZ.X/targetXYZ.Y;
     tXYZ.Y = targetXYZ.Y/targetXYZ.Y;
     tXYZ.Z = targetXYZ.Z/targetXYZ.Y;
     targetRGB = XYZ2RGB(tXYZ);

    while (!stop)
    {
        progressBar -> setProgress(progress++);
        // patch is RGB = 255 no need to change it
        // take another reading
        measuredXYZ = takeSpotReading();

        // if XYZ.Z < 0.0 then it was a bad reading
        if (measuredXYZ.Z > -99999.0)
        {
            cmsCIEXYZ mXYZ;

            mXYZ.X = measuredXYZ.X/measuredXYZ.Y;
            mXYZ.Y = measuredXYZ.Y/measuredXYZ.Y;
            mXYZ.Z = measuredXYZ.Z/measuredXYZ.Y;

            measuredRGB = XYZ2RGB(mXYZ);

            Rdir = (measuredRGB.R - targetRGB.R) * 400.0;
            Gdir = (measuredRGB.G - targetRGB.G) * 400.0;
            Bdir = (measuredRGB.B - targetRGB.B) * 400.0;

            // normalize RGB direction values to average zero
            double adj = (Rdir + Gdir + Bdir)/3.0;
            Rdir = Rdir - adj;
            Gdir = Gdir - adj;
            Bdir = Bdir - adj;
            // qDebug("Rdir = %f, Gdir = %f, Bdir = %f", Rdir, Gdir, Bdir);
            WPred -> setValue(Rdir);
            WPgreen -> setValue(Gdir);
            WPblue -> setValue(Bdir);
            // qDebug("displayTemp from setWP");
            displayTemp(mXYZ, tXYZ);
        }  //end if
        else
        {
            // need to issue an error message
            QMessageBox::critical( this, tr("Display Measurement Failed"), tr("The measurement instrument returned an error."));
        }
        if (progress > 5)
            progress = 0;
        lprofApp -> processEvents();
    } // end while
}

void MonitorCal::slotStop()
{
    stop = true;
    if (!measuring)
    {
        accept();
    }
}

double DeltaE(cmsCIEXYZ XYZ, cmsCIEXYZ targetXYZ)
{
    cmsCIEXYZ tXYZ, inXYZ;

    // normalize XYZ values
    inXYZ.X = XYZ.X/XYZ.Y;
    inXYZ.Y = XYZ.Y/XYZ.Y;
    inXYZ.Z = XYZ.Z/XYZ.Y;

    tXYZ.X = targetXYZ.X/targetXYZ.Y;
    tXYZ.Y = targetXYZ.Y/targetXYZ.Y;
    tXYZ.Z = targetXYZ.Z/targetXYZ.Y;

    // convert to Lab
    cmsCIELab WP, In;

    cmsXYZ2Lab(&tXYZ, &In, &inXYZ);
    cmsXYZ2Lab(&tXYZ, &WP, &tXYZ);

    return cmsDeltaE(&WP, &In);
}

void MonitorCal::displayTemp(cmsCIEXYZ XYZ, cmsCIEXYZ targetXYZ)
{
    QString buff;
//    double cct, cct_de;
//    double cdt, cdt_de;
    double vct, vct_de;
    double vdt, vdt_de;
    double white[3], target[3];

    /* Normalised XYZ white point */
    white[0] = XYZ.X/XYZ.Y;
    white[1] = XYZ.Y/XYZ.Y;
    white[2] = XYZ.Z/XYZ.Y;

    target[0] = targetXYZ.X/targetXYZ.Y;
    target[1] = targetXYZ.Y/targetXYZ.Y;
    target[2] = targetXYZ.Z/targetXYZ.Y;

//    cct = comp_ct(&cct_de, NULL, 1, 0, white);  /* Compute CCT */
//    cdt = comp_ct(&cdt_de, NULL, 0, 0, white);  /* Compute CDT */
//    vct = comp_ct(&vct_de, NULL, 1, 1, white);  /* Compute VCT */
    vdt = comp_ct(&vdt_de, target, 0, 1, white);  /* Compute VDT */

//    buff = QString("White Correlated Color Temperature = %1K, DE to locus = %2").arg(cct, 0, 'f', 5).arg(cct_de, 0, 'f', 5);
//    messageText -> insertParagraph(buff, -1);
//    buff = QString("White Correlated Daylight Temperature = %1K, DE to locus = %2").arg(cdt, 0, 'f', 5).arg(cdt_de, 0, 'f', 5);
//    messageText -> insertParagraph(buff, -1);
//    buff = QString("White Visual Color Temperature = %1K, DE to locus = %2"). arg(vct, 0, 'f', 5).arg(vct_de, 0, 'f', 5);
//    messageText -> insertParagraph(buff, -1);
//    buff = QString("White Visual Daylight Temperature = %1K, DE to locus = %2").arg(vdt, 0, 'f', 5).arg(vdt_de, 0, 'f', 5);
//    messageText -> insertParagraph(buff, -1);
    QString temp = QString("%1K").arg(vdt, 0, 'f', 1);
    TemperatureLabel -> setText(temp);

    cmsCIELab WP, In;

    cmsXYZ2Lab(&targetXYZ, &In, &XYZ);
    cmsXYZ2Lab(&targetXYZ, &WP, &targetXYZ);

    vdt_de = cmsDeltaE(&WP, &In);
    temp = QString("%1K").arg(vdt, 0, 'f', 1);
    temp = QString("%1").arg(vdt_de, 0, 'f', 2);
    deltaE -> setText(temp);

}


void MonitorCal::calibrationReport()
{
    double cct, cct_de;
    double cdt, cdt_de;
    double vct, vct_de;
    double vdt, vdt_de;
    double gamma, white[3], whitepoint[2];

    cmsCIEXYZ readXYZ[3];  // black, gray, white

    progressBar -> setTotalSteps(4);
    progressBar -> reset();

    paintPatch(0, 0, 0);
    readXYZ[0] = takeSpotReading();
    progressBar -> setProgress(1);

    paintPatch(127, 127, 127);
    readXYZ[1] = takeSpotReading();
    progressBar -> setProgress(2);

    paintPatch(255, 255, 255);
    readXYZ[2] = takeSpotReading();
    progressBar -> setProgress(3);

    /* Normalised XYZ white point */
    white[0] = readXYZ[2].X/readXYZ[2].Y;
    white[1] = readXYZ[2].Y/readXYZ[2].Y;
    white[2] = readXYZ[2].Z/readXYZ[2].Y;

    /* White point chromaticity coordinates */
    whitepoint[0] = white[0]/(white[0] + white[1] + white[2]);
    whitepoint[1] = white[1]/(white[0] + white[1] + white[2]);

    cct = comp_ct(&cct_de, NULL, 1, 0, white);  /* Compute CCT */
    cdt = comp_ct(&cdt_de, NULL, 0, 0, white);  /* Compute CDT */
    vct = comp_ct(&vct_de, NULL, 1, 1, white);  /* Compute VCT */
    vdt = comp_ct(&vdt_de, NULL, 0, 1, white);  /* Compute VDT */

    gamma = pop_gamma(readXYZ[0].Y, readXYZ[1].Y, readXYZ[2].Y);

    paintPatch(8, 8, 8);
    readXYZ[0] = takeSpotReading();
    progressBar -> setProgress(4);

    QString temp1 = tr(QString("Display gamma = %1\n")).arg(gamma, 0, 'f', 2);
    QString temp2 = tr(QString("Black level = %1 cd/m^2\n")).arg(readXYZ[0].Y*100, 0, 'f', 5);
    QString temp3 = QString (tr("White level = %1 cd/m^2\n")).arg(readXYZ[2].Y*100, 0, 'f', 1);
    QString temp5 = tr(QString("Contrast ratio = %1:1\n"))
            .arg(readXYZ[2].Y/readXYZ[0].Y, 0, 'f', 0);
    QString temp6 = QString (tr("White chromaticity coordinates %1, %2\n"))
            .arg(whitepoint[0], 0, 'f', 4).arg(whitepoint[1], 0, 'f', 4);
    QString temp7 = tr(QString("White Correlated Color Temperature = %1K, DE to locus = %2\n")).arg(cct, 0, 'f', 0).arg(cct_de, 0, 'f', 2);
    QString temp8 = tr(QString("White Correlated Daylight Temperature = %1K, DE to locus = %2\n")).arg(cdt, 0, 'f', 0).arg(cdt_de, 0, 'f', 2);
    QString temp9 = tr(QString("White Visual Color Temperature = %1K, DE to locus = %2\n")).arg(vct, 0, 'f', 0).arg(vct_de, 0, 'f', 2);
    QString temp10 = tr(QString("White Visual Daylight Temperature = %1K, DE to locus = %2")).arg(vdt, 0, 'f', 0).arg(vdt_de, 0, 'f', 2);
    QMessageBox::information( this, tr("Display Calibration Report"), temp1 + temp2 + temp3 + temp5 + temp6 + temp7 + temp8 + temp9 + temp10);
}

void MonitorCal::setDisplayBL()
{
    cmsCIEXYZ whiteXYZ, measuredXYZ;  // black, gray, white
    double target, direction, sv[3], val;
    int progress = 1;

    paintPatch(255, 255, 255);
    whiteXYZ = takeSpotReading();
    progressBar -> setProgress(progress++);

    target = 0.01 * whiteXYZ.Y;
    paintPatch(37, 37, 37);
    while (!stop)
    {
        measuredXYZ = takeSpotReading();
        if (measuredXYZ.X > -99999.0)
        {
            sv[0]=measuredXYZ.X * whiteXYZ.Y/whiteXYZ.X;
            sv[1]=measuredXYZ.Y;
            sv[2]=measuredXYZ.Z * whiteXYZ.Z/whiteXYZ.Z;

            val = sv[1];
            if (sv[0] > val)
                val = sv[0];
            if (sv[2] > val)
                val = sv[2];
            direction = val - target;

            blackLevel -> setValue(direction * 30000.0);
        }  // end if
        else
        {
            QMessageBox::critical( this, tr("Display Measurement Failed"), tr("The measurement instrument returned an error."));
        }  //end else
        progressBar -> setProgress(progress++);
        if (progress > 5)
            progress = 0;
        lprofApp -> processEvents();
    } // end while
}



void MonitorCal::measureDisplay(const RGBtype rgb[], const int number_of_patches)
{
    cmsCIEXYZ measuredXYZ;
    bool failed = false;
    stop = false;

    LCMSHANDLE it8 = cmsxIT8Alloc();
    cmsxIT8SetSheetType(it8, "LCMS/MEASUREMENT");
    cmsxIT8SetProperty(it8, "ORIGINATOR", (const char *) "LPROF");
    cmsxIT8SetProperty(it8, "DESCRIPTOR", (const char *) "hidden measurement sheet");
    cmsxIT8SetProperty(it8, "MANUFACTURER", (const char *) "LPROF");
    cmsxIT8SetProperty(it8, "CREATED", (const char *) "");
    cmsxIT8SetProperty(it8, "SERIAL", (const char *) "");
    cmsxIT8SetProperty(it8, "MATERIAL", (const char *) "");
    cmsxIT8SetPropertyDbl(it8, "NUMBER_OF_SETS", number_of_patches);
    cmsxIT8SetPropertyDbl(it8, "NUMBER_OF_FIELDS", 7);

    int nFields = 0;

    cmsxIT8SetDataFormat(it8, nFields++, "SAMPLE_ID");

    cmsxIT8SetDataFormat(it8, nFields++, "RGB_R");
    cmsxIT8SetDataFormat(it8, nFields++, "RGB_G");
    cmsxIT8SetDataFormat(it8, nFields++, "RGB_B");
    cmsxIT8SetDataFormat(it8, nFields++, "XYZ_X");
    cmsxIT8SetDataFormat(it8, nFields++, "XYZ_Y");
    cmsxIT8SetDataFormat(it8, nFields++, "XYZ_Z");

    progressBar -> setTotalSteps(number_of_patches);
    progressBar -> reset();

    char patchNumber[10];
    for (int i=0;i<number_of_patches;i++)
    {
        paintPatch((int) (rgb[i].R + 0.1), (int) (rgb[i].G + 0.1), (int) (rgb[i].B + 0.1));
        bool trying = true;
        int tryCount = 1;
        failed = false;
        while (trying)
        {
            measuredXYZ = takeSpotReading();
            if (measuredXYZ.X > -0.99999)
            {
                sprintf(patchNumber, "%i", i + 1);
                cmsxIT8SetDataSet(it8, patchNumber, "SAMPLE_ID", patchNumber);
                cmsxIT8SetDataSetDbl(it8, patchNumber, "RGB_R", (int) (rgb[i].R + 0.1));
                cmsxIT8SetDataSetDbl(it8, patchNumber, "RGB_G", (int) (rgb[i].G + 0.1));
                cmsxIT8SetDataSetDbl(it8, patchNumber, "RGB_B", (int) (rgb[i].B + 0.1));

                cmsxIT8SetDataSetDbl(it8, patchNumber, "XYZ_X", measuredXYZ.X * 100.0);
                cmsxIT8SetDataSetDbl(it8, patchNumber, "XYZ_Y", measuredXYZ.Y * 100.0);
                cmsxIT8SetDataSetDbl(it8, patchNumber, "XYZ_Z", measuredXYZ.Z * 100.0);
                trying = false;
            }
            else
            {
                tryCount++;
            }
            if (tryCount > 5)
            {
                failed = true;
                trying = false;
            }
            lprofApp -> processEvents();
        }
        progressBar -> setProgress(i + 1);
        if (stop)
        {
           break;
        }

    }
    if (failed)  // the measurement failed
    {
        QMessageBox::critical( this, tr("Display Measurement Failed"), tr("The measurement instrument returned an error.\nDisplay measurement aborted."));
        cd.data.OK = false;
    }
    else if (stop)  // the user stopped the measurement
    {
        QMessageBox::information( this, tr("Display Measurement User Abort"), tr("The user has aborted the measurement process.\nDisplay measurement aborted."));
        cd.data.OK = false;
    }
    else  // it completed
    {
        cmsxIT8SaveToFile(it8, measfile);
        cd.data.OK = true;
    }
    cmsxIT8Free(it8);
}

void MonitorCal::setDisplayWL()
{
    cmsCIEXYZ whiteXYZ, measuredXYZ;  // black, gray, white
    double targetwhite, direction;
    int progress = 1;

    paintPatch(255, 255, 255);
    whiteXYZ = takeSpotReading();
    targetwhite = whiteXYZ.Y;

    if (targetBrightness > 0.000000001)
    {
        targetwhite = targetBrightness;
    }
    else targetwhite = whiteXYZ.Y;

    while (!stop)
    {
        progressBar -> setProgress(progress++);
        measuredXYZ = takeSpotReading();
        if (measuredXYZ.X > -99999.0)
        {
            direction =  measuredXYZ.Y - (targetwhite/100.0);
            blackLevel -> setValue(direction * 500.0);
        }
        else
            QMessageBox::critical( this, tr("Display Measurement Failed"), tr("The measurement instrument returned an error."));
        if (progress > 5)
            progress = 0;
        lprofApp -> processEvents(1);
    }
}

void MonitorCal::setDisplayBP()
{
    RGBtype targetRGB, measuredRGB;
    cmsCIEXYZ measuredXYZ, targetXYZ;
    bool nativeWP = false;
    double Rdir, Gdir, Bdir;
    int progress = 1;

//    buttonStop -> setEnabled(TRUE);
    stop = false;
    // paintPatch(30, 30, 30);

    // setup target whitepoint settings
    if (whitePoint > 0.00000001)
    {               /* Daylight color temperature */
        double XYZ[3];
        if (icx_ill_sp2XYZ(XYZ, icxOT_default, NULL, icxIT_Dtemp, whitePoint, NULL) != 0)
            qDebug ("Failed to compute XYZ of target color temperature %f", whitePoint);
        targetXYZ.X=XYZ[0];
        targetXYZ.Y=XYZ[1];
        targetXYZ.Z=XYZ[2];
     }
     else
     {   /* Native white */
         paintPatch(255, 255, 255);
         targetXYZ = takeSpotReading();
         progressBar -> setProgress(progress++);
         nativeWP = true;
     }
    paintPatch(40, 40, 40);
    measuredXYZ = takeSpotReading();
    // targetXYZ.Y = measuredXYZ.Y;
    //normalize target white point

    cmsCIEXYZ tXYZ;
    tXYZ.X = targetXYZ.X/targetXYZ.Y;
    tXYZ.Y = targetXYZ.Y/targetXYZ.Y;
    tXYZ.Z = targetXYZ.Z/targetXYZ.Y;
    targetRGB = XYZ2RGB(tXYZ);
    while (!stop)
    {
        progressBar -> setProgress(progress++);
        // patch is RGB = 255 no need to change it
        // take another reading
        measuredXYZ = takeSpotReading();

        // if XYZ.Z < 0.0 then it was a bad reading
        if (measuredXYZ.Z > -99999.0)
        {
            cmsCIEXYZ mXYZ;
            mXYZ.X = measuredXYZ.X/measuredXYZ.Y;
            mXYZ.Y = measuredXYZ.Y/measuredXYZ.Y;
            mXYZ.Z = measuredXYZ.Z/measuredXYZ.Y;
            measuredRGB = XYZ2RGB(mXYZ);

            Rdir = (measuredRGB.R - targetRGB.R) * 100.0;
            Gdir = (measuredRGB.G - targetRGB.G) * 100.0;
            Bdir = (measuredRGB.B - targetRGB.B) * 100.0;

            // normalize RGB direction values to average zero
            double adj = (Rdir + Gdir + Bdir)/3.0;
            Rdir = Rdir - adj;
            Gdir = Gdir - adj;
            Bdir = Bdir - adj;
            // qDebug("Rdir = %f, Gdir = %f, Bdir = %f", Rdir, Gdir, Bdir);
            WPred -> setValue(Rdir);
            WPgreen -> setValue(Gdir);
            WPblue -> setValue(Bdir);
            displayTemp(mXYZ, tXYZ);
        }  //end if
        else
        {
            // need to issue an error message
            QMessageBox::critical( this, tr("Display Measurement Failed"), tr("The measurement instrument returned an error."));
        }
        if (progress > 5)
            progress = 0;
        lprofApp -> processEvents();
    } // end while
}

void ComputeRGBNeutralTranferFunctions(LPGAMMATABLE TransferCurves[3])
{
    cmsCIEXYZ WhitePoint;
    cmsCIEXYZ BlackPoint;
    cmsCIExyYTRIPLE Primaries;

    cmsxComputeMatrixShaper("", measfile, MEDIUM_TRANSMISSIVE,
                                TransferCurves, &WhitePoint,
                                &BlackPoint, &Primaries);
}

void MonitorCal::calibrateDisplay()
{
    LPGAMMATABLE TransferCurves[3], desiredCurve,
                 desiredCurves[3], resampledTcurves[3],
                 reversedTcurves[3], newTcurves[3],
                 BPcomp[3], WPcomp[3];
     cmsCIEXYZ WhitePoint;
     cmsCIEXYZ BlackPoint;
     cmsCIExyYTRIPLE Primaries;
    char table_index[5];
    double Rdir, Gdir, Bdir;
    monCal OK;  // used to signal that the calibration was OK

    cmsCIEXYZ gammaXYZ[3], measuredXYZ;  // black, gray, white

    progressBar -> setTotalSteps(calibration_set_size);
    progressBar -> reset();

    if (dtype == LCD)
       QMessageBox::warning( this, tr("LCD Display Not Yet Supported"), tr("LCD display calibration has not been fully implemented yet.  If you continue you will be using a CRT calibration algorithm."));

    setVCGammaLinear();

    cd.data.OK = false;
    measureDisplay(patches_cal, calibration_set_size);
    // cd.data.OK = true;
    if (cd.data.OK)
    {

//         cmsxComputeMatrixShaper("", measfile, MEDIUM_TRANSMISSIVE,
//                                 TransferCurves, &WhitePoint,
//                                 &BlackPoint, &Primaries);

        ComputeRGBNeutralTranferFunctions(TransferCurves);

        switch (cd.data.gammaType)
        {
            case gtGeneric:
                desiredCurves[0] = cmsBuildGamma(orig_gammaRamp -> size,
                                        cd.data.gamma);
                desiredCurves[1] = cmsBuildGamma(orig_gammaRamp -> size,
                                        cd.data.gamma);
                desiredCurves[2] = cmsBuildGamma(orig_gammaRamp -> size,
                                        cd.data.gamma);
                break;
            case gtsRGB:
                double parms[5];
                parms[0] = 2.4;
                parms[1] = 1.0 / 1.055;
                parms[2] = 0.055 / 1.055;
                parms[3] = 1.0 / 12.92;
                parms[4] = 0.04045;

                desiredCurves[0] = cmsBuildParametricGamma(
                        orig_gammaRamp -> size, 4, parms);
                desiredCurves[1] = cmsBuildParametricGamma(
                        orig_gammaRamp -> size, 4, parms);
                desiredCurves[2] = cmsBuildParametricGamma(
                         orig_gammaRamp -> size, 4, parms);
                break;
            case gtLstar:  // L*
                double y;
                LPGAMMATABLE tempGT;
                tempGT = cmsAllocGamma(orig_gammaRamp -> size);
                for (int i=0; i<orig_gammaRamp -> size; i++)
                {
                    y = (double) i / (double) (orig_gammaRamp -> size - 1);
                    if (y >  0.008856451586)
                        tempGT -> GammaTable[i] =
                               (int) (((1.16 * pow(y,1.0/3.0) - 0.16) *
                                65535.0));
                    else
                        tempGT -> GammaTable[i] =
                               (int) (((1.16 * 7.787036979 * y) * 65535.0));
                }
                desiredCurves[0] = cmsReverseGamma(orig_gammaRamp -> size, tempGT);
                desiredCurves[1] = cmsReverseGamma(orig_gammaRamp -> size, tempGT);
                desiredCurves[2] = cmsReverseGamma(orig_gammaRamp -> size, tempGT);
                cmsFreeGamma(tempGT);
                break;
        }
        reversedTcurves[0] = cmsReverseGamma(1024, TransferCurves[0]);
        reversedTcurves[1] = cmsReverseGamma(1024, TransferCurves[1]);
        reversedTcurves[2] = cmsReverseGamma(1024, TransferCurves[2]);

        resampledTcurves[0] = cmsReverseGamma(orig_gammaRamp -> size,
                                            reversedTcurves[0]);
        resampledTcurves[1] = cmsReverseGamma(orig_gammaRamp -> size,
                                            reversedTcurves[1]);
        resampledTcurves[2] = cmsReverseGamma(orig_gammaRamp -> size,
                                            reversedTcurves[2]);

        newTcurves[0] = cmsJoinGamma(desiredCurves[0], resampledTcurves[0]);
        newTcurves[1] = cmsJoinGamma(desiredCurves[1], resampledTcurves[1]);
        newTcurves[2] = cmsJoinGamma(desiredCurves[2], resampledTcurves[2]);

//        for (int i=0; i < orig_gammaRamp -> size; i++)
//        {
//            new_gammaRamp -> red[i] = (double) newTcurves[0] -> GammaTable[i] / 65535.0;
//            new_gammaRamp -> green[i] = (double) newTcurves[1] -> GammaTable[i] / 65535.0;
//            new_gammaRamp -> blue[i] = (double) newTcurves[2] -> GammaTable[i] / 65535.0;
//        }
//
//         setVCGamma();
//         {  //  Calcuate BP offsets for new gamma curves
//             RGBtype targetRGB, measuredRGB;
//             cmsCIEXYZ targetXYZ;
//             bool nativeWP = false;
//
//             // setup target whitepoint settings
//             if (whitePoint > 0.00000001)
//             {               // Daylight color temperature
//                 double XYZ[3];
//                 if (icx_ill_sp2XYZ(XYZ, icxOT_default, NULL, icxIT_Dtemp, whitePoint, NULL) != 0)
//                     qDebug ("Failed to compute XYZ of target color temperature %f", whitePoint);
//                 targetXYZ.X=XYZ[0];
//                 targetXYZ.Y=XYZ[1];
//                 targetXYZ.Z=XYZ[2];
//             }
//             else
//             {   // Native white
//                 paintPatch(255, 255, 255);
//                 targetXYZ = takeSpotReading();
//                 nativeWP = true;
//             }
//
//             paintPatch(25, 25, 25);
//             measuredXYZ = takeSpotReading();
//
//             // normalize target white point
//             cmsCIEXYZ tXYZ;
//             tXYZ.X = targetXYZ.X/targetXYZ.Y;
//             tXYZ.Y = targetXYZ.Y/targetXYZ.Y;
//             tXYZ.Z = targetXYZ.Z/targetXYZ.Y;
//             targetRGB = XYZ2RGB(tXYZ);
//             if (measuredXYZ.Z > -99999.0)
//             {
//                 // normalize target black point
//                 cmsCIEXYZ mXYZ;
//                 mXYZ.X = measuredXYZ.X/measuredXYZ.Y;
//                 mXYZ.Y = measuredXYZ.Y/measuredXYZ.Y;
//                 mXYZ.Z = measuredXYZ.Z/measuredXYZ.Y;
//                 measuredRGB = XYZ2RGB(mXYZ);
//
//                 double k = 220.0;
//                 Rdir = measuredRGB.R - targetRGB.R;
//                 Gdir = measuredRGB.G - targetRGB.G;
//                 Bdir = measuredRGB.B - targetRGB.B;
//                 qDebug("dirs = %f  %f  %f", Rdir, Gdir, Bdir);
//
//                 // normalize RGB direction values so that high value = zero
//                 double high = Rdir;
//                 if (high < Gdir) high = Gdir;
//                 if (high < Bdir) high = Bdir;
//
//                 Rdir = Rdir - high;
//                 Gdir = Gdir - high;
//                 Bdir = Bdir - high;
//
//                 // then make all values positive.
//
//                 Rdir = -Rdir;
//                 Gdir = -Gdir;
//                 Bdir = -Bdir;
//
//                 // Gamma correct and scale values
//                 Rdir = pow(Rdir, 1.0/cd.data.gamma) * k;
//                 Gdir = pow(Gdir, 1.0/cd.data.gamma) * k;
//                 Bdir = pow(Bdir, 1.0/cd.data.gamma) * k;
//
//                 qDebug("Black point RGB direction = %f  %f  %f", Rdir, Gdir, Bdir);
//
//                 qDebug("After adj Rdir = %f, Gdir = %f, Bdir = %f", Rdir, Gdir, Bdir);
//             }  //end if
//             else
//             {
//                 // need to issue an error message
//                 QMessageBox::critical( this, tr("Display Measurement Failed"), tr("The measurement instrument returned an error."));
//             }
//             lprofApp -> processEvents();
//         }
//
//         BPcomp[0] = cmsBuildGamma(orig_gammaRamp -> size, 1.0);
//         BPcomp[1] = cmsBuildGamma(orig_gammaRamp -> size, 1.0);
//         BPcomp[2] = cmsBuildGamma(orig_gammaRamp -> size, 1.0);
//
//         for (int l=0; l < orig_gammaRamp -> size; l++)
//         {
//             double correction = (double) orig_gammaRamp -> size /
//                           ((double) orig_gammaRamp -> size + pow((double) l + .00000000001, 2.2));
//
//             BPcomp[0] -> GammaTable[l] = BPcomp[0] -> GammaTable[l] + (int)(Rdir * correction - 0.5);
//             BPcomp[1] -> GammaTable[l] = BPcomp[1] -> GammaTable[l] + (int)(Gdir * correction - 0.5);
//             BPcomp[2] -> GammaTable[l] = BPcomp[2] -> GammaTable[l] + (int)(Bdir * correction - 0.5);
//             qDebug("mutipler %f  offsets  %i   %i   %i  row %i", correction, (int)(Rdir * correction - 0.5), (int)(Gdir * correction - 0.5), (int)(Bdir * correction - 0.5), l);
//         }
//
//         BPcomp[0] = cmsReverseGamma(orig_gammaRamp -> size, BPcomp[0]);
//         BPcomp[1] = cmsReverseGamma(orig_gammaRamp -> size, BPcomp[1]);
//         BPcomp[2] = cmsReverseGamma(orig_gammaRamp -> size, BPcomp[2]);
//
//         desiredCurves[0] = cmsJoinGamma(desiredCurves[0], BPcomp[0]);
//         desiredCurves[1] = cmsJoinGamma(desiredCurves[1], BPcomp[1]);
//         desiredCurves[2] = cmsJoinGamma(desiredCurves[2], BPcomp[2]);
//
//         newTcurves[0] = cmsJoinGamma(desiredCurves[0], resampledTcurves[0]);
//         newTcurves[1] = cmsJoinGamma(desiredCurves[1], resampledTcurves[1]);
//         newTcurves[2] = cmsJoinGamma(desiredCurves[2], resampledTcurves[2]);


        // write out CGATS data and setup the video card gamma table
        LCMSHANDLE it8 = cmsxIT8Alloc();
        cmsxIT8SetSheetType(it8, "LCMS/MEASUREMENT");
        cmsxIT8SetProperty(it8, "ORIGINATOR", (const char *) "LPROF");
        cmsxIT8SetProperty(it8, "DESCRIPTOR", (const char *) "hidden measurement sheet calibration for vcgt");
        cmsxIT8SetProperty(it8, "MANUFACTURER", (const char *) "LPROF");
        cmsxIT8SetProperty(it8, "CREATED", (const char *) "");
        cmsxIT8SetProperty(it8, "SERIAL", (const char *) "");
        cmsxIT8SetProperty(it8, "MATERIAL", (const char *) "");
        cmsxIT8SetPropertyDbl(it8, "NUMBER_OF_SETS", orig_gammaRamp -> size);
        cmsxIT8SetPropertyDbl(it8, "NUMBER_OF_FIELDS", 4);

        int nFields = 0;

        cmsxIT8SetDataFormat(it8, nFields++, "SAMPLE_ID");
        cmsxIT8SetDataFormat(it8, nFields++, "RGB_R");
        cmsxIT8SetDataFormat(it8, nFields++, "RGB_G");
        cmsxIT8SetDataFormat(it8, nFields++, "RGB_B");

        for (int i=0; i < orig_gammaRamp -> size; i++)
        {
            new_gammaRamp -> red[i] = (double) newTcurves[0] -> GammaTable[i] / 65535.0;
            new_gammaRamp -> green[i] = (double) newTcurves[1] -> GammaTable[i] / 65535.0;
            new_gammaRamp -> blue[i] = (double) newTcurves[2] -> GammaTable[i] / 65535.0;

            sprintf(table_index, "%i", i + 1);
            cmsxIT8SetDataSet(it8, table_index, "SAMPLE_ID", table_index);
            cmsxIT8SetDataSetDbl(it8, table_index, "RGB_R", double (newTcurves[0] -> GammaTable[i]));
            cmsxIT8SetDataSetDbl(it8, table_index, "RGB_G", double (newTcurves[1] -> GammaTable[i]));
            cmsxIT8SetDataSetDbl(it8, table_index, "RGB_B", double (newTcurves[2] -> GammaTable[i]));
        }
        // write out CGATS data and free emeory
        cmsxIT8SaveToFile(it8, calfile);
        cmsxIT8Free(it8);

        // Update video card gamma
        setVCGamma();

        // Free the memory for the gamma tables.
        cmsFreeGamma(newTcurves[0]);
        cmsFreeGamma(newTcurves[1]);
        cmsFreeGamma(newTcurves[2]);

        cmsFreeGamma(desiredCurves[0]);
        cmsFreeGamma(desiredCurves[1]);
        cmsFreeGamma(desiredCurves[2]);

        cmsFreeGamma(TransferCurves[0]);
        cmsFreeGamma(TransferCurves[1]);
        cmsFreeGamma(TransferCurves[2]);

        cmsFreeGamma(resampledTcurves[0]);
        cmsFreeGamma(resampledTcurves[1]);
        cmsFreeGamma(resampledTcurves[2]);

        cmsFreeGamma(reversedTcurves[0]);
        cmsFreeGamma(reversedTcurves[1]);
        cmsFreeGamma(reversedTcurves[2]);

        QString mess = tr("Calibration file ") + (QString) calfile + tr(" has been created.");
        messageText -> insertParagraph(mess, -1);
        QMessageBox::information( this, tr("Display Calibration File Created"), tr("The calibration process has completed"));

        OK.calOK(TRUE);

//         cmsFreeGamma(BPcomp[0]);
//         cmsFreeGamma(BPcomp[1]);
//         cmsFreeGamma(BPcomp[2]);
//
           // FIXME This blows up - why?
//         cmsFreeGamma(resampledTcurves[0]);
//         cmsFreeGamma(resampledTcurves[1]);
//         cmsFreeGamma(resampledTcurves[2]);

    }
    else
    {
        OK.calOK(FALSE);  // measurement failed
    }
}

void MonitorCal::Go()
{
    stop = false;
    measuring = true;

    // setup the video card gamma object for the calibration tables
#if defined (Q_WS_WIN)
    screen = (int) 0;
#elif defined(Q_WS_MAC)
    screen = (int) 0;
#elif defined(Q_WS_X11) // this is an X11 system
    screen = desktop -> screenNumber(pixmapColor -> mapToGlobal(pixmapColor -> pos()));
#else
   // This means this is being built on a system that is not currently supported.
   // This does not mean that this code will not work on the unsupported system.
   // Rather that the developers don't know what will work and have not
   // coded anything for your system.
   // Users running into this error message are asked to provide bug reports
   // and, even better, patches.
   // There is also a section of src/videoLUTutils/LUTutil.h that will also need
   // to be updated at the same time as this.
   // Please see the
   // #else
   // ....
   // #error
   // section of src/videoLUTutils/LUTutil.h for more details
#error "Your system is not supported yet. The video gamma setting code will need to be updated to support your system."
#endif

    // qDebug("setup instrument");
    // setup instrument
    it = NULL;
    if (initInstrument())    
    {
        // get the current video card gamma table
        getCurrentVCGamma();
        goButton -> setEnabled(FALSE);
        // qDebug("start switch");
        switch (op)
        {
            case opCalReport:
                    calibrationReport();
                    break;
            case opUncalReport:
                    setVCGammaLinear();
                    // calibrationReport();
                    bigCalibrationReport();
                    restoreVCGamma();
                    break;
            case opBigCalReport:
                    bigCalibrationReport();
                    break;
            case opSetWhitePoint:
                    setVCGammaLinear();
                    setDisplayWP();
                    restoreVCGamma();
                    break;
            case opSetBlackLevel:
                    setVCGammaLinear();
                    setDisplayBL();
                    restoreVCGamma();
                    break;
            case opSetWhiteLevel:
                    setVCGammaLinear();
                    setDisplayWL();
                    restoreVCGamma();
                    break;
            case opSetBlackPoint:
                    setVCGammaLinear();
                    setDisplayBP();
                    restoreVCGamma();
                    break;
            case opMeasure:
                    switch (sSize)
                    {
                        case smallest:
                            measureDisplay(patches_small, small_size);
                            break;
                        case medium:
                            measureDisplay(patches_medium, medium_size);
                            break;
                        case large:
                            measureDisplay(patches_large, large_size);
                            break;
                        case huge:
                            measureDisplay(patches_huge, huge_size);
                            break;
                    }
                    if (cd.data.OK)
                    {
                            QString mess = tr("Measurement file ") + (QString) measfile + tr(" has been created.");
                            messageText -> insertParagraph(mess, -1);
                            QMessageBox::information( this, tr("Display Measurement File Created"), tr("The display measurement process has completed"));
                    }
                    break;
            case opCalibrate:
                    // setup gamma ramp for
                    new_gammaRamp = new gamma_ramp(screen, 
                            pixmapColor -> mapToGlobal(pixmapColor -> pos()).x(), 
                            pixmapColor -> mapToGlobal(pixmapColor -> pos()).y());
                    if (new_gammaRamp -> size == 0)
                       qDebug("there is a problem with gamma table creation");
                    setVCGammaLinear();
                    calibrateDisplay();
                    delete new_gammaRamp;
                    break;
        }
        it -> del(it);
        instInUse = FALSE;
        goButton -> setEnabled(FALSE);
        delete orig_gammaRamp;
        accept();
    }
    else
    {
        // initialization failed - clean up as needed
        instInUse = FALSE;
        qDebug("about to it -> del(it)");
        if (it != NULL) it -> del(it);
        qDebug("about to reject");
        reject();
    }    
}

void MonitorCal::setupDialogLayout()
{
    QString mess;

    switch (op)
    {
        case opCalReport:
                frameRGBscales -> hide();
                frameBWLevel -> hide();
                WPandDEframe -> hide();
                mess = tr("Place meter over measurement area. Press Go to start measurements.\nCalibration report will pop up when measurements are done");
                messageText -> insertParagraph(mess, -1);
                break;
        case opUncalReport:
                frameRGBscales -> hide();
                frameBWLevel -> hide();
                WPandDEframe -> hide();
                mess = tr("Place meter over measurement area. Press Go to start measurements.\nCalibration report will pop up when measurements are done");
                messageText -> insertParagraph(mess, -1);
                break;
        case opSetWhitePoint:
                frameBWLevel -> hide();
                progressBar -> setTotalSteps (5);
                progressBar -> setPercentageVisible (false);
                mess = tr("Place meter over measurement area. Press Go to start measurements.\nAdjust display R,G & B gain to center red, green and blue bars.\nWhen done press the Done button.");
                messageText -> insertParagraph(mess, -1);
                break;
        case opSetBlackLevel:
                frameRGBscales -> hide();
                WPandDEframe -> hide();
                progressBar -> setTotalSteps (5);
                progressBar -> setPercentageVisible (false);
                mess = tr("Place meter over measurement area. Press Go to start measurements.\nAdjust display CRT brightness control to center the dark gray bar.\nWhen done press the Done button.");
                messageText -> insertParagraph(mess, -1);
                break;
        case opSetWhiteLevel:
                frameRGBscales -> hide();
                WPandDEframe -> hide();
                progressBar -> setTotalSteps (5);
                progressBar -> setPercentageVisible (false);
                mess = tr("Place meter over measurement area. Press Go to start measurements.\nAdjust display CRT contrast or LDC brightness control to center the dark gray bar.\nWhen done press the Done button.");
                messageText -> insertParagraph(mess, -1);
                break;
        case opSetBlackPoint:
                frameBWLevel -> hide();
                progressBar -> setTotalSteps (5);
                progressBar -> setPercentageVisible (false);
                mess = tr("Place meter over measurement area. Press Go to start measurements.\nAdjust display R,G & B gain to center red, green and blue bars.\nWhen done press the Done button.");
                messageText -> insertParagraph(mess, -1);
                break;
        case opMeasure:
                frameRGBscales -> hide();
                frameBWLevel -> hide();
                WPandDEframe -> hide();
                mess = tr("Place meter over measurement area.  Press Go to start measurement process.\nPress Done to abort measurement process.");
                messageText -> insertParagraph(mess, -1);
                break;
        case opCalibrate:
                frameRGBscales -> hide();
                frameBWLevel -> hide();
                WPandDEframe -> hide();
                mess = tr("Place meter over measurement area.  Press Go to start measurement process.\nPress Done to abort measurement process.");
                messageText -> insertParagraph(mess, -1);
                break;
        case opBigCalReport:
                frameRGBscales -> hide();
                frameBWLevel -> hide();
                WPandDEframe -> hide();
                mess = tr("Place meter over measurement area. Press Go to start measurements.\nCalibration report will pop up when measurements are done");
                messageText -> insertParagraph(mess, -1);
                break;

    }
}

void getInData()
{
    cd.get(&cd);
    cd.data.OK = false;
    dtype = cd.data.disp;
    whitePoint = cd.data.wp;
    op = cd.data.op;
    targetBrightness = cd.data.displayBr;
    sSize = cd.data.size;
    strcpy(measfile, cd.data.measureFile);
    strcpy(calfile, cd.data.calibFile);
}

void MonitorCal::slotPosChanged()
{
    QRect frameR = mainFrame -> frameRect();
    int h = frameR.height();
    int w = frameR.width();
    double percentH = (double) sliderHpos -> value()/1000.0;
    double percentV = (double) sliderVpos -> value()/1000.0;
    int x = (int) ((double) w * percentH) - (pixmapColor -> height()/2);
    int y = (int) ((double) h * percentV) - (pixmapColor -> width()/2);
    pixmapColor -> move(x, y);
}

void MonitorCal::slotSizeChanged()
{
    pixmapColor -> resize(500 - sliderSize -> value(), 500 - sliderSize -> value());
    slotPosChanged();
}

MonitorCal::MonitorCal(QWidget *parent)
    : MonitorCalBase(parent, "", 0, Qt::WStyle_SysMenu | Qt::WStyle_MinMax | WStyle_StaysOnTop)
{
    messageText -> setTextFormat( Qt::LogText);

    for (int i = 0; i < 26; i++)
       sp2cief[i] = NULL;

    messageText -> setText(" ");
    pixmapColor -> setUpdatesEnabled( TRUE );
    getInData();
    setupDialogLayout();
    slotPosChanged();
    slotSizeChanged();
    measuring = false;
    desktop = new QDesktopWidget();
}

MonitorCal::~MonitorCal()
{
   cd.put(cd);
}

void MonitorCal::getCurrentVCGamma()
{
    // qDebug("getCurrentVCGamma()");
    orig_gammaRamp = new gamma_ramp(screen, 
                         pixmapColor -> mapToGlobal(pixmapColor -> pos()).x(),
                         pixmapColor -> mapToGlobal(pixmapColor -> pos()).y());
    orig_gammaRamp -> getGammaRamp();
}

void MonitorCal::bigCalibrationReport()
{
    double cct, cct_de;
    double cdt, cdt_de;
    double vct, vct_de;
    double vdt, vdt_de;
    double gamma, white[3], whitepoint[2], Rgamma, Ggamma, Bgamma;
    const int size = 17;
    QString Delta_E_lines[size];

    cmsCIEXYZ redXYZ[2], greenXYZ[2], blueXYZ[2], locusXYZ[size], targetXYZ, readXYZ;  // black, gray, white

    progressBar -> setTotalSteps(size + 7);
    progressBar -> reset();

    for (int i=0; i<size; i++)
    {
        int rgb;

        rgb = (int) ((double) i * 15.875);
        if (i == size - 1)
            paintPatch(255,255,255);
        else
            paintPatch(rgb, rgb, rgb);
        locusXYZ[i] = takeSpotReading();
        progressBar -> setProgress(i+1);
    }

    paintPatch(127, 0, 0);
    redXYZ[0] = takeSpotReading();
    progressBar -> setProgress(size + 1);

    paintPatch(255, 0, 0);
    redXYZ[1] = takeSpotReading();
    progressBar -> setProgress(size + 2);

    paintPatch(0, 127, 0);
    greenXYZ[0] = takeSpotReading();
    progressBar -> setProgress(size + 3);

    paintPatch(0, 255, 0);
    greenXYZ[1] = takeSpotReading();
    progressBar -> setProgress(size + 4);

    paintPatch(0, 0, 127);
    blueXYZ[0] = takeSpotReading();
    progressBar -> setProgress(size + 5);

    paintPatch(0, 0, 255);
    blueXYZ[1] = takeSpotReading();
    progressBar -> setProgress(size + 6);

    // setup target whitepoint settings
    if (whitePoint > 0.00000001)
    {               /* Daylight color temperature */
        double XYZ[3];
        if (icx_ill_sp2XYZ(XYZ, icxOT_default, NULL, icxIT_Dtemp, whitePoint, NULL) != 0)
            qDebug ("Failed to compute XYZ of target color temperature %f", whitePoint);
        targetXYZ.X=XYZ[0];
        targetXYZ.Y=XYZ[1];
        targetXYZ.Z=XYZ[2];
    }
    else
    {   /* Native white */
        targetXYZ = locusXYZ[size-1];
    }

    /* Normalised XYZ white point */
    white[0] = locusXYZ[size - 1].X/locusXYZ[size - 1].Y;
    white[1] = locusXYZ[size - 1].Y/locusXYZ[size - 1].Y;
    white[2] = locusXYZ[size - 1].Z/locusXYZ[size - 1].Y;

    /* White point chromaticity coordinates */
    whitepoint[0] = white[0]/(white[0] + white[1] + white[2]);
    whitepoint[1] = white[1]/(white[0] + white[1] + white[2]);

    cct = comp_ct(&cct_de, NULL, 1, 0, white);  /* Compute CCT */
    cdt = comp_ct(&cdt_de, NULL, 0, 0, white);  /* Compute CDT */
    vct = comp_ct(&vct_de, NULL, 1, 1, white);  /* Compute VCT */
    vdt = comp_ct(&vdt_de, NULL, 0, 1, white);  /* Compute VDT */

    gamma = pop_gamma(locusXYZ[0].Y, locusXYZ[size/2].Y, locusXYZ[size-1].Y);
    Rgamma = pop_gamma(locusXYZ[0].Y, redXYZ[0].Y, redXYZ[1].Y);
    Ggamma = pop_gamma(locusXYZ[0].Y, greenXYZ[0].Y, greenXYZ[1].Y);
    Bgamma = pop_gamma(locusXYZ[0].Y, blueXYZ[0].Y, blueXYZ[1].Y);

    paintPatch(8, 8, 8);
    readXYZ = takeSpotReading();
    progressBar -> setProgress(size + 7);

    QString temp1 = tr(QString("Display gamma = %1\n")).arg(gamma, 0, 'f', 2);
    QString temp1_1 = tr(QString("Display red gamma = %1\n")).arg(Rgamma, 0, 'f', 2);
    QString temp1_2 = tr(QString("Display green gamma = %1\n")).arg(Rgamma, 0, 'f', 2);
    QString temp1_3 = tr(QString("Display blue gamma = %1\n")).arg(Rgamma, 0, 'f', 2);
    QString temp2 = tr(QString("Black level = %1 cd/m^2\n")).arg(readXYZ.Y*100, 0, 'f', 5);
    QString temp3 = tr(QString("White level = %1 cd/m^2\n")).arg(locusXYZ[size - 1].Y*100, 0, 'f', 1);
    QString temp5 = tr(QString("Contrast ratio = %1:1\n"))
            .arg(locusXYZ[size - 1].Y/readXYZ.Y, 0, 'f', 0);
    QString temp6 = QString (tr("White chromaticity coordinates %1, %2\n"))
            .arg(whitepoint[0], 0, 'f', 4).arg(whitepoint[1], 0, 'f', 4);
    QString temp7 = tr(QString("White Correlated Color Temperature = %1K, DE to locus = %2\n")).arg(cct, 0, 'f', 0).arg(cct_de, 0, 'f', 2);
    QString temp8 = tr(QString("White Correlated Daylight Temperature = %1K, DE to locus = %2\n")).arg(cdt, 0, 'f', 0).arg(cdt_de, 0, 'f', 2);
    QString temp9 = tr(QString("White Visual Color Temperature = %1K, DE to locus = %2\n")).arg(vct, 0, 'f', 0).arg(vct_de, 0, 'f', 2);
    QString temp10 = tr(QString("White Visual Daylight Temperature = %1K, DE to locus = %2\n\n")).arg(vdt, 0, 'f', 0).arg(vdt_de, 0, 'f', 2);
    for (int i=0; i<size-1; i++)
    {
        int rgb;

        rgb = (int) (((double) i) * 15.875);
        Delta_E_lines[i] = tr(QString("At RGB= %1 DE to locus = %2\n")).arg(rgb).arg(DeltaE(locusXYZ[i],targetXYZ) , 0, 'f', 2);
    }
    QMessageBox::information( this, tr("Display Calibration Report"),
                              temp1 + temp1_1 + temp1_2 + temp1_3 +
                                      temp2 + temp3 + temp5 + temp6 +
                                      temp7 + temp8 + temp9 + temp10 +
                                      Delta_E_lines[0] + Delta_E_lines[1] +
                                      Delta_E_lines[2] + Delta_E_lines[3] +
                                      Delta_E_lines[4] + Delta_E_lines[5] +
                                      Delta_E_lines[6] + Delta_E_lines[7] +
                                      Delta_E_lines[8] + Delta_E_lines[9] +
                                      Delta_E_lines[10] + Delta_E_lines[11] +
                                      Delta_E_lines[12] + Delta_E_lines[13] +
                                      Delta_E_lines[14] + Delta_E_lines[15]);
}

void MonitorCal::setVCGammaLinear()
{
	orig_gammaRamp -> setLinearGammaRamp();
}

void MonitorCal::restoreVCGamma()
{
    orig_gammaRamp -> setGammaRamp();
}

void MonitorCal::setVCGamma()
{
    new_gammaRamp -> setGammaRamp();
}

void MonitorCal::slotHelpButton()
{
    QString path = get_global_help_path()->local8Bit() +  "/monmeasurement.html";
    // qDebug(path);
    getHelp()->openAssistant ();
    getHelp()->showPage(path);
}
